@@ -635,7 +635,7 @@ setClass("denseLU", contains = "LU",
 	 validity = function(object) .Call(LU_validate, object))
 
 setClass("sparseLU", contains = "LU",
-	 representation(L = "dgCMatrix", U = "dgCMatrix",
+	 representation(L = "dtCMatrix", U = "dtCMatrix",
 			p = "integer", q = "integer"))
 
 ##--- QR ---
@@ -645,7 +645,7 @@ setClass("sparseQR", contains = "MatrixFactorization",
 			p = "integer", R = "dgCMatrix", q = "integer"),
 	 validity = function(object) .Call(sparseQR_validate, object))
 
-##-- "SPQR" ---> ./spqr.R  for noew
+##-- "SPQR" ---> ./spqr.R  for now
 
 ## "denseQR" -- ?  (``a version of''  S3 class "qr")
 
@@ -714,7 +714,7 @@ setClassUnion("xMatrix", ## those Matrix classes with an 'x' slot
 ## e.g. for "Arith" methods:
 setClassUnion("numLike", members = c("numeric", "logical"))
 
-setClassUnion("numIndex", members = "numeric")
+##setClassUnion("numIndex", members = "numeric")
 
 ## Note "rle" is a sealed oldClass (and "virtual" as w/o prototype)
 setClass("rleDiff", representation(first = "numLike", rle = "rle"),
