@@ -119,7 +119,8 @@ prepanel.default.cloud <-
 panel.3dscatter <-
     function(x, y, z, rot.mat = diag(4), distance,
              groups = NULL,
-             type = 'p',
+             type = "p",
+             xlim, ylim, zlim,
              xlim.scaled,
              ylim.scaled,
              zlim.scaled,
@@ -134,8 +135,20 @@ panel.3dscatter <-
              pch = if (is.null(groups)) "+" else superpose.symbol$pch,
              cross,
              ...,
-             subscripts = TRUE)
+             .scale = FALSE,
+             subscripts = TRUE,
+             identifier = "3dscatter")
 {
+    if (.scale)
+    {
+        ## x, y, z, are usually already scaled to lie within a
+        ## bounding box.  However, setting .scale=TRUE will do the
+        ## scaling here; this may be helpful for calls to
+        ## panel.3dscatter() in user-supplied panel functions.
+        x <- xlim.scaled[1] + diff(xlim.scaled) * (x-xlim[1])/diff(xlim)
+        y <- ylim.scaled[1] + diff(ylim.scaled) * (y-ylim[1])/diff(ylim)
+        z <- zlim.scaled[1] + diff(zlim.scaled) * (z-zlim[1])/diff(zlim)
+    }
 
     ##cloud.3d <- list(col=1, cex=1, lty=1, lwd=1, pch=1)
     plot.symbol <- trellis.par.get("plot.symbol")
@@ -180,7 +193,7 @@ panel.3dscatter <-
         ## this is too much to make it worth the effort.)
 
         ## 'points' type
-        if (any(c('p', 'b', 'o') %in% type))
+        if (any(c("p", "b", "o") %in% type))
         {
             id <-
                 ((x >= xlim.scaled[1]) & (x <= xlim.scaled[2]) &
@@ -207,19 +220,22 @@ panel.3dscatter <-
 
                 lsegments(x0 = m0[1,], y0 = m0[2,],
                           x1 = m1[1,], y1 = m1[2,],
-                          col = rep(col.line[ord], each = 3))
+                          col = rep(col.line[ord], each = 3),
+                          ...,
+                          identifier = paste(identifier, "points", sep = "."))
             }
             else
             {
                 lpoints(x = m[1, ord], y = m[2, ord],
                         col = col.point[ord],
                         pch = pch[ord],
-                        cex = cex[ord])
+                        cex = cex[ord], ...,
+                        identifier = identifier)
             }
         }
 
         ## 'lines' type
-        if (any(c('l', 'b', 'o') %in% type))
+        if (any(c("l", "b", "o") %in% type))
         {
             ord <- if (is.null(groups)) TRUE else sort.list(groups)
             tmplen <- length(x)
@@ -244,12 +260,15 @@ panel.3dscatter <-
             ord <- sort.list(pmax(m0[3,], m1[3,]))
             lsegments(x0 = m0[1, ord], y0 = m0[2, ord],
                       x1 = m1[1, ord], y1 = m1[2, ord],
-                      col = tmpcol0[ord])
+                      col = tmpcol0[ord],
+                      lwd = lwd[ord],
+                      lty = lty[ord], ...,
+                      identifier = paste(identifier, "lines", sep = "."))
         }
 
 
         ## 'histogram' type
-        if ('h' %in% type)
+        if ("h" %in% type)
         {
             id <-
                 ((x >= xlim.scaled[1]) & (x <= xlim.scaled[2]) &
@@ -267,9 +286,10 @@ panel.3dscatter <-
                       other.end[1,ord], other.end[2,ord],
                       col = col.line[ord],
                       lty = lty[ord],
-                      lwd = lwd[ord])
+                      lwd = lwd[ord], ...,
+                      identifier = paste(identifier, "hist", sep = "."))
         }
-        if (any(!(type %in% c('p', 'h', 'l', 'b', 'o'))))
+        if (any(!(type %in% c("p", "h", "l", "b", "o"))))
         {
             warning("'type' has unsupported values")
         }
@@ -302,6 +322,7 @@ panel.3dwire <-
              shade = FALSE,
              shade.colors.palette = trellis.par.get("shade.colors")$palette,
              light.source = c(0, 0, 1000),
+             xlim, ylim, zlim,
              xlim.scaled,
              ylim.scaled,
              zlim.scaled,
@@ -311,10 +332,12 @@ panel.3dwire <-
              col.groups = superpose.polygon$col,
              polynum = 100,
              ...,
+             .scale = FALSE,
              drape = FALSE,
              at,
              col.regions = regions$col,
-             alpha.regions = regions$alpha)
+             alpha.regions = regions$alpha,
+             identifier = "3dwire")
 {
 
     ## a faster version of panel.3dwire that takes advantage of grid
@@ -337,6 +360,16 @@ panel.3dwire <-
     ## time. The difference is mostly in C code, but some distinctions
     ## need to be made here as well
 
+    if (.scale)
+    {
+        ## x, y, z, are usually already scaled to lie within a
+        ## bounding box.  However, setting .scale=TRUE will do the
+        ## scaling here; this may be helpful for calls to
+        ## panel.3dscatter() in user-supplied panel functions.
+        x[] <- xlim.scaled[1] + diff(xlim.scaled) * (x-xlim[1])/diff(xlim)
+        y[] <- ylim.scaled[1] + diff(ylim.scaled) * (y-ylim[1])/diff(ylim)
+        z[] <- zlim.scaled[1] + diff(zlim.scaled) * (z-zlim[1])/diff(zlim)
+    }
 
 
 
@@ -360,9 +393,6 @@ panel.3dwire <-
         else col.regions[floor(1+(1:numcol-1)*(numcol.r-1)/(numcol-1))]
 
 
-
-
-
     ## 2004-03-12 new experimental stuff: when x, y, z are all
     ## matrices of the same dimension, they represent a 3-D surface
     ## parametrized on a 2-D grid (the details of the parametrizing
@@ -403,11 +433,7 @@ panel.3dwire <-
 
     if (shade)
     {
-
-        shade.colors.palette <-
-            if (is.character(shade.colors.palette)) get(shade.colors.palette)
-            else eval(shade.colors.palette)
-
+        shade.colors.palette <- getFunctionOrName(shade.colors.palette)
         pol.x <- numeric(polynum * 3)
         pol.y <- numeric(polynum * 3)
 
@@ -417,6 +443,8 @@ panel.3dwire <-
         if (!missing(alpha)) pol.alpha <- alpha
 
         count <- 0 ## counts number of polygons stacked up so far
+        ## counts number of times grid.polygon has been called
+        gridpolycount <- 0 
 
         wirePolygon <-
 
@@ -444,8 +472,12 @@ panel.3dwire <-
 
                     if (count == polynum)
                     {
+                        gridpolycount <<- gridpolycount + 1
                         grid.polygon(x = pol.x, y = pol.y, id.lengths = rep(3, polynum),
                                      default.units = "native",
+                                     name = trellis.grobname(paste(identifier,
+                                       "polygons", gridpolycount, sep = "."),
+                                       type = "panel"),
                                      gp = gpar(fill = pol.fill,
                                      col = pol.col,
                                      lty = lty, lwd = lwd,
@@ -478,6 +510,9 @@ panel.3dwire <-
         {
             grid.polygon(x = pol.x[1:(count * 3)], y = pol.y[1:(count * 3)],
                          default.units = "native", id.lengths = rep(3, count),
+                         name = trellis.grobname(paste(identifier,
+                           "polygons", gridpolycount + 1, sep = "."),
+                           type = "panel"),
                          gp = gpar(fill = rep(pol.fill, length.out = count),
                          col = rep(pol.col, length.out = count),
                          lty = lty, lwd = lwd,
@@ -513,11 +548,12 @@ panel.3dwire <-
                 else vector(mode(col), polynum)
             pol.alpha <- superpose.polygon$alpha
         }
-        ## FIXME: should alpha override alpha.regions?
-        if (!missing(alpha)) pol.alpha <- alpha
-
+        ## FIXME: should alpha override alpha.regions? NO.
+        ## if (!missing(alpha)) pol.alpha <- alpha
 
         count <- 0 ## counts number of polygons stacked up so far
+        ## counts number of times grid.polygon has been called
+        gridpolycount <- 0 
 
         wirePolygon <-
 
@@ -553,12 +589,16 @@ panel.3dwire <-
 
                     if (count == polynum) {
 
+                        gridpolycount <<- gridpolycount + 1
                         grid.polygon(x = pol.x, y = pol.y, id.lengths = rep(4, polynum),
                                      default.units = "native",
+                                     name = trellis.grobname(paste(identifier,
+                                       "polygons", gridpolycount, sep = "."),
+                                       type = "panel"),
                                      gp = gpar(fill = pol.fill,
-                                     col = pol.col,
-                                     lty = lty, lwd = lwd,
-                                     alpha = pol.alpha))
+                                               col = pol.col, ## FIXME: should be adjustcolor(col, alpha.f = alpha)
+                                               lty = lty, lwd = lwd,
+                                               alpha = pol.alpha))
                         count <<- 0
                     }
                 }
@@ -584,6 +624,9 @@ panel.3dwire <-
         {
             grid.polygon(x = pol.x[1:(count * 4)], y = pol.y[1:(count * 4)],
                          default.units = "native", id.lengths = rep(4, count),
+                         name = trellis.grobname(paste(identifier,
+                           "polygons", gridpolycount + 1, sep = "."),
+                           type = "panel"),
                          gp = gpar(fill = rep(pol.fill, length.out = count),
                          col = rep(pol.col, length.out = count),
                          lty = lty, lwd = lwd,
@@ -598,14 +641,6 @@ panel.3dwire <-
 
 
 
-
-
-
-
-
-
-
-
 panel.cloud <-
     function(x, y, subscripts, z,
              groups = NULL,
@@ -632,8 +667,8 @@ panel.cloud <-
 
              scpos,
              ...,
-             at)    ## this is same as 'at' in wireframe
-
+             at,    ## this is same as 'at' in wireframe
+             identifier = "cloud")
 {
 
     ## x, y, z can be matrices and we want to retain them as matrices.
@@ -685,47 +720,47 @@ panel.cloud <-
 
         xlabelinfo <-
             calculateAxisComponents(xlim,
-                                    at = scales.3d$x$at,
+                                    at = scales.3d$x.scales$at,
                                     num.limit = NULL,
-                                    labels = scales.3d$x$labels,
-                                    logsc = scales.3d$x$log,
-                                    abbreviate = scales.3d$x$abbreviate,
-                                    minlength = scales.3d$x$minlength,
-                                    format.posixt = scales.3d$x$format,
-                                    n = scales.3d$x$tick.number)
+                                    labels = scales.3d$x.scales$labels,
+                                    logsc = scales.3d$x.scales$log,
+                                    abbreviate = scales.3d$x.scales$abbreviate,
+                                    minlength = scales.3d$x.scales$minlength,
+                                    format.posixt = scales.3d$x.scales$format,
+                                    n = scales.3d$x.scales$tick.number)
 
 
         ylabelinfo <-
             calculateAxisComponents(ylim,
-                                    at = scales.3d$y$at,
+                                    at = scales.3d$y.scales$at,
                                     num.limit = NULL,
-                                    labels = scales.3d$y$labels,
-                                    logsc = scales.3d$y$log,
-                                    abbreviate = scales.3d$y$abbreviate,
-                                    minlength = scales.3d$y$minlength,
-                                    format.posixt = scales.3d$y$format,
-                                    n = scales.3d$y$tick.number)
+                                    labels = scales.3d$y.scales$labels,
+                                    logsc = scales.3d$y.scales$log,
+                                    abbreviate = scales.3d$y.scales$abbreviate,
+                                    minlength = scales.3d$y.scales$minlength,
+                                    format.posixt = scales.3d$y.scales$format,
+                                    n = scales.3d$y.scales$tick.number)
 
 
         zlabelinfo <-
             calculateAxisComponents(zlim,
-                                    at = scales.3d$z$at,
+                                    at = scales.3d$z.scales$at,
                                     num.limit = NULL,
-                                    labels = scales.3d$z$labels,
-                                    logsc = scales.3d$z$log,
-                                    abbreviate = scales.3d$z$abbreviate,
-                                    minlength = scales.3d$z$minlength,
-                                    format.posixt = scales.3d$z$format,
-                                    n = scales.3d$z$tick.number)
+                                    labels = scales.3d$z.scales$labels,
+                                    logsc = scales.3d$z.scales$log,
+                                    abbreviate = scales.3d$z.scales$abbreviate,
+                                    minlength = scales.3d$z.scales$minlength,
+                                    format.posixt = scales.3d$z.scales$format,
+                                    n = scales.3d$z.scales$tick.number)
 
 
         x.at <- xlabelinfo$at
         y.at <- ylabelinfo$at
         z.at <- zlabelinfo$at
 
-        x.at.lab <- xlabelinfo$lab
-        y.at.lab <- ylabelinfo$lab
-        z.at.lab <- zlabelinfo$lab
+        x.at.lab <- xlabelinfo$labels
+        y.at.lab <- ylabelinfo$labels
+        z.at.lab <- zlabelinfo$labels
 
         xlim <- xlabelinfo$num.limit
         ylim <- ylabelinfo$num.limit
@@ -904,6 +939,8 @@ panel.cloud <-
             }
             else cmin$z + clen$z * (at - zlim[1])/diff(zlim)
 
+        ## same?: zero.scaled <- cmin$z + clen$z * (0 - zlim[1])/diff(zlim)
+        ## yes.                  cmin$z - clen$z *       zlim[1]/diff(zlim)
         zero.scaled <- cmin$z - clen$z * zlim[1]/diff(zlim)
         ## needed in panel.3dscatter for type = 'h'
 
@@ -991,7 +1028,8 @@ panel.cloud <-
                   corners[2, nxt[!mark]],
                   col = par.box.final$col,
                   lwd = par.box.final$lwd,
-                  lty = 2)
+                  lty = 2,
+                  identifier = paste(identifier, "back.box", sep = "."))
 
 
         ## The following portion of code is responsible for drawing
@@ -1071,12 +1109,8 @@ panel.cloud <-
                 x <- sort(unique(x))
                 y <- sort(unique(y))
             }
-
             z <- list(NULL) ## hopefully becomes garbage, collected if necessary
-
-            panel.3d.wireframe <-
-                if (is.character(panel.3d.wireframe)) get(panel.3d.wireframe)
-                else eval(panel.3d.wireframe)
+            panel.3d.wireframe <- getFunctionOrName(panel.3d.wireframe)
 
             pargs <- list(x = x, y = y, z = tmp,
                           rot.mat = rot.mat,
@@ -1092,18 +1126,13 @@ panel.cloud <-
                           zero.scaled = zero.scaled,
                           ...)
 
-
             if (!("..." %in% names(formals(panel.3d.wireframe))))
                 pargs <- pargs[intersect(names(pargs), names(formals(panel.3d.wireframe)))]
             do.call("panel.3d.wireframe", pargs)
-
         }
-        else {
-
-            panel.3d.cloud <-
-                if (is.character(panel.3d.cloud)) get(panel.3d.cloud)
-                else eval(panel.3d.cloud)
-
+        else
+        {
+            panel.3d.cloud <- getFunctionOrName(panel.3d.cloud)
             pargs <- list(x = x, y = y, z = z,
                           rot.mat = rot.mat,
                           distance = distance,
@@ -1117,16 +1146,11 @@ panel.cloud <-
                           zlim.scaled = zlim.scaled,
                           zero.scaled = zero.scaled,
                           ...)
-
             if (!("..." %in% names(formals(panel.3d.cloud))))
                 pargs <- pargs[intersect(names(pargs), names(formals(panel.3d.cloud)))]
             do.call("panel.3d.cloud", pargs)
         }
 
-
-
-
-
         ## This draws the front of the bounding box
 
         lsegments(corners[1, pre[mark]],
@@ -1135,9 +1159,11 @@ panel.cloud <-
                   corners[2, nxt[mark]],
                   col = par.box.final$col,
                   lty = par.box.final$lty,
-                  lwd = par.box.final$lwd)
+                  lwd = par.box.final$lwd,
+                  identifier = paste(identifier, "front.box", sep = "."))
 
-        ## Next part for axes
+        ## Next part for axes. FIXME: ignoring axis.text$lineheight
+        ## because seems overkill, but could add that too.
 
         axis.text <- trellis.par.get("axis.text")
         axis.line <- trellis.par.get("axis.line")
@@ -1146,10 +1172,10 @@ panel.cloud <-
             if (is.logical(scales.3d$x.scales$col.line)) axis.line$col
             else scales.3d$x.scales$col.line
         xaxis.lty <-
-            if (is.logical(scales.3d$x.scales$lty)) axis.line$lwd
+            if (is.logical(scales.3d$x.scales$lty)) axis.line$lty
             else scales.3d$x.scales$lty
         xaxis.lwd <-
-            if (is.logical(scales.3d$x.scales$lwd)) axis.line$lty
+            if (is.logical(scales.3d$x.scales$lwd)) axis.line$lwd
             else scales.3d$x.scales$lwd
         xaxis.col.text <-
             if (is.logical(scales.3d$x.scales$col)) axis.text$col
@@ -1175,10 +1201,10 @@ panel.cloud <-
             if (is.logical(scales.3d$y.scales$col.line)) axis.line$col
             else scales.3d$y.scales$col.line
         yaxis.lty <-
-            if (is.logical(scales.3d$y.scales$lty)) axis.line$lwd
+            if (is.logical(scales.3d$y.scales$lty)) axis.line$lty
             else scales.3d$y.scales$lty
         yaxis.lwd <-
-            if (is.logical(scales.3d$y.scales$lwd)) axis.line$lty
+            if (is.logical(scales.3d$y.scales$lwd)) axis.line$lwd
             else scales.3d$y.scales$lwd
         yaxis.col.text <-
             if (is.logical(scales.3d$y.scales$col)) axis.text$col
@@ -1204,10 +1230,10 @@ panel.cloud <-
             if (is.logical(scales.3d$z.scales$col.line)) axis.line$col
             else scales.3d$z.scales$col.line
         zaxis.lty <-
-            if (is.logical(scales.3d$z.scales$lty)) axis.line$lwd
+            if (is.logical(scales.3d$z.scales$lty)) axis.line$lty
             else scales.3d$z.scales$lty
         zaxis.lwd <-
-            if (is.logical(scales.3d$z.scales$lwd)) axis.line$lty
+            if (is.logical(scales.3d$z.scales$lwd)) axis.line$lwd
             else scales.3d$z.scales$lwd
         zaxis.col.text <-
             if (is.logical(scales.3d$z.scales$col)) axis.text$col
@@ -1236,20 +1262,26 @@ panel.cloud <-
                         length = 0.02, unit = "npc",
                         lty = xaxis.lty,
                         lwd = xaxis.lwd,
-                        col = xaxis.col.line)
+                        col = xaxis.col.line,
+                        identifier = paste(identifier, "x.axis.arrow",
+                          sep = "."))
             }
             else {
                 lsegments(x0 = x.at[1,], y0 = x.at[2,], x1 = x.at.end[1,], y1 = x.at.end[2,],
                           lty = xaxis.lty,
                           col = xaxis.col.line,
-                          lwd = xaxis.lwd)
+                          lwd = xaxis.lwd,
+                          identifier = paste(identifier, "x.axis.ticks",
+                            sep = "."))
                 ltext(x.at.lab, x = x.labs[1,], y = x.labs[2,],
                       cex = xaxis.cex,
                       srt = xaxis.rot,
                       font = xaxis.font,
                       fontfamily = xaxis.fontfamily,
                       fontface = xaxis.fontface,
-                      col = xaxis.col.text)
+                      col = xaxis.col.text,
+                      identifier = paste(identifier, "x.axis.labels",
+                        sep = "."))
             }
         }
 
@@ -1260,20 +1292,29 @@ panel.cloud <-
                         length = 0.02, unit = "npc",
                         lty = yaxis.lty,
                         lwd = yaxis.lwd,
-                        col = yaxis.col.line)
+                        col = yaxis.col.line,
+                        identifier = paste(identifier, "y.axis.arrow",
+                            sep = "."))
+
             }
             else {
                 lsegments(x0 = y.at[1,], y0 = y.at[2,], x1 = y.at.end[1,], y1 = y.at.end[2,],
                           lty = yaxis.lty,
                           col = yaxis.col.line,
-                          lwd = yaxis.lwd)
+                          lwd = yaxis.lwd,
+                          identifier = paste(identifier, "y.axis.ticks",
+                            sep = "."))
+
                 ltext(y.at.lab, x = y.labs[1,], y = y.labs[2,],
                       cex = yaxis.cex,
                       srt = yaxis.rot,
                       font = yaxis.font,
                       fontfamily = yaxis.fontfamily,
                       fontface = yaxis.fontface,
-                      col = yaxis.col.text)
+                      col = yaxis.col.text,
+                      identifier = paste(identifier, "y.axis.labels",
+                        sep = "."))
+
             }
         }
         if (scales.3d$z.scales$draw) {
@@ -1283,20 +1324,29 @@ panel.cloud <-
                         length = 0.02, unit = "npc",
                         lty = zaxis.lty,
                         lwd = zaxis.lwd,
-                        col = zaxis.col.line)
+                        col = zaxis.col.line,
+                        identifier = paste(identifier, "z.axis.arrow",
+                          sep = "."))
+
             }
             else {
                 lsegments(x0 = z.at[1,], y0 = z.at[2,], x1 = z.at.end[1,], y1 = z.at.end[2,],
                           lty = zaxis.lty,
                           col = zaxis.col.line,
-                          lwd = zaxis.lwd)
+                          lwd = zaxis.lwd,
+                          identifier = paste(identifier, "z.axis.ticks",
+                            sep = "."))
+
                 ltext(z.at.lab, x = z.labs[1,], y = z.labs[2,],
                       cex = zaxis.cex,
                       srt = zaxis.rot,
                       font = zaxis.font,
                       fontfamily = zaxis.fontfamily,
                       fontface = zaxis.fontface,
-                      col = zaxis.col.text)
+                      col = zaxis.col.text,
+                      identifier = paste(identifier, "z.axis.labels",
+                        sep = "."))
+
             }
         }
 
@@ -1308,13 +1358,13 @@ panel.cloud <-
         ## supplied in the *lab lists
 
         xlab <-
-            grobFromLabelList(xlab, name = trellis.grobname("xlab"))
+            grobFromLabelList(xlab, name = trellis.grobname("xlab", type=""))
 #                              rot = if (is.null(xlab$rot)) 0 else xlab$rot)
         ylab <-
-            grobFromLabelList(ylab, name = trellis.grobname("ylab"))
+            grobFromLabelList(ylab, name = trellis.grobname("ylab", type=""))
 #                              rot = if (is.null(ylab$rot)) 0 else ylab$rot)
         zlab <-
-            grobFromLabelList(zlab, name = trellis.grobname("zlab"))
+            grobFromLabelList(zlab, name = trellis.grobname("zlab", type=""))
 #                              rot = if (is.null(zlab$rot)) 0 else zlab$rot)
 
         if (!is.null(xlab))
@@ -1391,12 +1441,14 @@ wireframe.formula <-
     function(x,
              data = NULL,
              panel = lattice.getOption("panel.wireframe"),
+             default.prepanel = lattice.getOption("prepanel.default.wireframe"),
              ...)
 {
     ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(wireframe)
     ccall <- match.call()
     ccall$data <- data
     ccall$panel <- panel
+    ccall$default.prepanel <- default.prepanel
     ccall[[1]] <- quote(lattice::cloud)
     ans <- eval.parent(ccall)
     ans$call <- ocall
@@ -1513,6 +1565,7 @@ cloud.formula <-
              ...,
              lattice.options = NULL,
              default.scales = list(distance = c(1, 1, 1), arrows = TRUE, axs = axs.default),
+             default.prepanel = lattice.getOption("prepanel.default.cloud"),
              colorkey = any(drape),
              col.regions,
              alpha.regions,
@@ -1562,13 +1615,6 @@ cloud.formula <-
 
     if (!is.function(panel)) panel <- eval(panel)
     if (!is.function(strip)) strip <- eval(strip)
-
-    prepanel <-
-        if (is.function(prepanel)) prepanel
-        else if (is.character(prepanel)) get(prepanel)
-        else eval(prepanel)
-
-
     cond <- form$condition
     z <- form$left
     x <- form$right.x
@@ -1826,7 +1872,7 @@ cloud.formula <-
 
 
     more.comp <-
-        c(limits.and.aspect(prepanel.default.cloud,
+        c(limits.and.aspect(default.prepanel,
                             prepanel = prepanel,
                             have.xlim = have.xlim, xlim = xlim,
                             have.ylim = have.ylim, ylim = ylim,
@@ -1840,8 +1886,7 @@ cloud.formula <-
     foo[names(more.comp)] <- more.comp
 
 
-    if (is.null(foo$legend) && !is.null(groups) &&
-        (is.list(auto.key) || (is.logical(auto.key) && auto.key)))
+    if (is.null(foo$legend) && needAutoKey(auto.key, groups))
     {
         foo$legend <-
             list(list(fun = "drawSimpleKey",
