@@ -1,7 +1,7 @@
 ## code for fast REML computation. key feature is that first and 
 ## second derivatives come at no increase in leading order 
 ## computational cost, relative to evaluation! 
-## (c) Simon N. Wood, 2010-2012
+## (c) Simon N. Wood, 2010-2014
 
 Sl.setup <- function(G) {
 ## Sets up a list representing a block diagonal penalty matrix.
@@ -19,7 +19,7 @@ Sl.setup <- function(G) {
 ##   - Applies to cols/params from start:stop.
 ##   - If numeric then X[,start:stop]%*%diag(D) is repara X[,start:stop],
 ##     b.orig = D*b.repara
-##   - If matrix then X[,start:stop]%*%diag(D) is repara X[,start:stop],
+##   - If matrix then X[,start:stop]%*%D is repara X[,start:stop],
 ##     b.orig = D%*%b.repara
 ## The penalties in Sl are in the same order as those in G
 ## Also returns attribute "E" a square root of the well scaled total
@@ -58,20 +58,11 @@ Sl.setup <- function(G) {
   } ## finished paraPen
 
   ## now work through the smooths....
-  for (i in 1:length(G$smooth)) {
+  if (length(G$smooth)) for (i in 1:length(G$smooth)) {
 
     if (!is.null(G$smooth[[i]]$fixed)&&G$smooth[[i]]$fixed) m <- 0 else
     m <- length(G$smooth[[i]]$S)
 
-   # if (m>0&&!is.null(G$smooth[[i]]$fx)) { ## drop any fixed 
-   #   for (j in m:1) {
-   #     if (G$smooth[[i]]$fx[j]==TRUE) { 
-   #       G$smooth[[i]]$S[[j]] <- NULL
-   #       m <- m - 1
-   #     }
-   #   }
-   # }
-
     if (m>0) {
       Sl[[b]] <- list()
       Sl[[b]]$start <- G$smooth[[i]]$first.para
@@ -109,13 +100,13 @@ Sl.setup <- function(G) {
         for (j in 1:m) {
           Sl[[b]] <- list()
           ind <- sbStart[j]:sbStop[j]
-          Sl[[b]]$S <- list(G$smooth[[i]]$S[[j]][ind,ind])
+          Sl[[b]]$S <- list(G$smooth[[i]]$S[[j]][ind,ind,drop=FALSE])
           Sl[[b]]$start <- G$smooth[[i]]$first.para + sbStart[j]-1
           Sl[[b]]$stop <- G$smooth[[i]]$first.para + sbStop[j]-1
+          Sl[[b]]$rank <- G$smooth[[i]]$rank[j]
           b <- b + 1
         }
       } else { ## not possible to split
-        #Sl[[b]]$rank <- nrow(G$smooth[[i]]$S[[1]]) - G$smooth[[i]]$null.space.dim+1 
         Sl[[b]]$S <- G$smooth[[i]]$S
       
       } ## additive block finished
@@ -128,6 +119,8 @@ Sl.setup <- function(G) {
   ## Singletons need to be transformed to identity penalties, while 
   ## multiples need to be projected into total penalty range space. 
 
+  if (length(Sl)==0) return(Sl) ## nothing to do
+
   np <- ncol(G$X)
   E <- matrix(0,np,np) ## well scaled square root penalty
   lambda <- rep(0,0)
@@ -151,9 +144,10 @@ Sl.setup <- function(G) {
         ind <- rep(FALSE,length(D))
         ind[1:Sl[[b]]$rank] <- TRUE ## index penalized elements
         D[ind] <- 1/sqrt(D[ind]);D[!ind] <- 1
-        D <- t(D*t(U)) ## D <- U%*%diag(D)
+        Sl[[b]]$D <- t(D*t(U)) ## D <- U%*%diag(D)
+        Sl[[b]]$Di <- t(U)/D
         ## so if X is smooth model matrix X%*%D is re-parameterized form 
-        Sl[[b]]$D <- D; Sl[[b]]$ind <- ind
+        Sl[[b]]$ind <- ind
       }
       ## add penalty square root into E  
       ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
@@ -176,8 +170,7 @@ Sl.setup <- function(G) {
       }
       Sl[[b]]$ind <- rep(FALSE,ncol(U))
       Sl[[b]]$ind[ind] <- TRUE ## index penalized within sub-range
-      ## Sl[[b]]$stop = Sl[[b]]$start + Sl[[b]]$rank - 1
-      ## Sl[[b]]$ind <- ind ## IS THIS NEEDED?? (if so should be logical!!)
+     
       ## now compute well scaled sqrt
       S.norm <- norm(Sl[[b]]$S[[1]])
       St <- Sl[[b]]$S[[1]]/S.norm
@@ -198,20 +191,37 @@ Sl.setup <- function(G) {
   Sl ## the penalty list
 } ## end of Sl.setup
 
-Sl.initial.repara <- function(Sl,X,inverse=FALSE) {
+Sl.initial.repara <- function(Sl,X,inverse=FALSE,both.sides=TRUE,cov=TRUE) {
 ## Routine to apply initial Sl re-parameterization to model matrix X,
 ## or, if inverse==TRUE, to apply inverse re-para to parameter vector 
-## or cov matrix
+## or cov matrix. if inverse is TRUE and both.sides=FALSE then 
+## re-para only applied to rhs, as appropriate for a choleski factor.
+  if (length(Sl)==0) return(X) ## nothing to do
   if (inverse) { ## apply inverse re-para
-    if (is.matrix(X)) { ## then assume it's a covariance matrix
-      for (b in 1:length(Sl)) { 
-        ind <- Sl[[b]]$start:Sl[[b]]$stop
-        if (is.matrix(Sl[[b]]$D)) { 
-          X[ind,] <- Sl[[b]]$D%*%X[ind,,drop=FALSE]
-          X[,ind] <- X[,ind,drop=FALSE]%*%t(Sl[[b]]$D) 
-        } else {
-          X[,ind] <- t(Sl[[b]]$D * t(X[,ind,drop=FALSE]))
-          X[ind,] <- Sl[[b]]$D * X[ind,,drop=FALSE]
+    if (is.matrix(X)) { 
+      if (cov) { ## then it's a covariance matrix
+        for (b in 1:length(Sl)) { 
+          ind <- Sl[[b]]$start:Sl[[b]]$stop
+          if (is.matrix(Sl[[b]]$D)) { 
+            if (both.sides) X[ind,] <- Sl[[b]]$D%*%X[ind,,drop=FALSE]
+            X[,ind] <- X[,ind,drop=FALSE]%*%t(Sl[[b]]$D) 
+          } else { ## Diagonal D
+            X[,ind] <- t(Sl[[b]]$D * t(X[,ind,drop=FALSE]))
+            if (both.sides) X[ind,] <- Sl[[b]]$D * X[ind,,drop=FALSE]
+          } 
+        } 
+      } else { ## regular matrix: need to use Di
+         for (b in 1:length(Sl)) { 
+          ind <- Sl[[b]]$start:Sl[[b]]$stop
+          if (is.matrix(Sl[[b]]$D)) { 
+            Di <- if(is.null(Sl[[b]]$Di)) t(Sl[[b]]$D) else Sl[[b]]$Di
+            if (both.sides) X[ind,] <- t(Di)%*%X[ind,,drop=FALSE]
+            X[,ind] <- X[,ind,drop=FALSE]%*%Di 
+          } else { ## Diagonal D
+            Di <- 1/Sl[[b]]$D
+            X[,ind] <- t(Di * t(X[,ind,drop=FALSE]))
+            if (both.sides) X[ind,] <- Di * X[ind,,drop=FALSE]
+          } 
         } 
       }
     } else { ## it's a parameter vector
@@ -227,15 +237,18 @@ Sl.initial.repara <- function(Sl,X,inverse=FALSE) {
     X[,ind] <- t(Sl[[b]]$D*t(X[,ind,drop=FALSE])) ## X[,ind]%*%diag(Sl[[b]]$D)
   }
   X
-} ## end Sl.initial.repra
+} ## end Sl.initial.repara
 
 ldetS <- function(Sl,rho,fixed,np,root=FALSE) {
-## Get log generalized derivative of S stored blockwise in an Sl list.
+## Get log generalized determinant of S stored blockwise in an Sl list.
 ## Any multi-term blocks will be re-parameterized using gam.reparam, and
 ## a re-parameterization object supplied in the returned object.
+## rho contains log smoothing parameters, fixed is an array indicating whether they 
+## are fixed (or free). np is the number of coefficients. root indicates
+## whether or not to return E. 
 ## Returns: Sl, with modified rS terms, if needed and rho added to each block
 ##          rp, a re-parameterization list
-##          E a total penalty square root such that E'E = S_tot
+##          E a total penalty square root such that E'E = S_tot (if root==TRUE)
 ##          ldetS,ldetS1,ldetS2 the value, grad vec and Hessian
   n.deriv <- sum(!fixed)
   k.deriv <- k.sp <- k.rp <- 1
@@ -295,15 +308,17 @@ ldetS <- function(Sl,rho,fixed,np,root=FALSE) {
   list(ldetS=ldS,ldet1=d1.ldS,ldet2=d2.ldS,Sl=Sl,rp=rp,E=E)
 } ## end ldetS
 
-Sl.repara <- function(rp,X,inverse=FALSE) {
-## Apply re-parameterization from ldetS to X, blockwise
-## if inverse==TRUE applies the inverse of the re-para to
-## vector X or cov matrix X...
+Sl.repara <- function(rp,X,inverse=FALSE,both.sides=TRUE) {
+## Apply re-parameterization from ldetS to X, blockwise.
+## If X is a matrix it is assumed to be a model matrix
+## whereas if X is a vector it is assumed to be a parameter vector.
+## If inverse==TRUE applies the inverse of the re-para to
+## parameter vector X or cov matrix X...
   nr <- length(rp);if (nr==0) return(X)
   if (inverse) {
     if (is.matrix(X)) { ## X is a cov matrix
       for (i in 1:nr) {
-        X[rp[[i]]$ind,]  <- 
+        if (both.sides) X[rp[[i]]$ind,]  <- 
                      rp[[i]]$Qs %*% X[rp[[i]]$ind,,drop=FALSE]
         X[,rp[[i]]$ind]  <- 
                      X[,rp[[i]]$ind,drop=FALSE] %*% t(rp[[i]]$Qs)
@@ -312,7 +327,11 @@ Sl.repara <- function(rp,X,inverse=FALSE) {
      for (i in 1:nr) X[rp[[i]]$ind]  <- rp[[i]]$Qs %*% X[rp[[i]]$ind]
     }
   } else { ## apply re-para to X
-    for (i in 1:nr) X[,rp[[i]]$ind]  <- X[,rp[[i]]$ind]%*%rp[[i]]$Qs
+    if (is.matrix(X)) {
+      for (i in 1:nr) X[,rp[[i]]$ind]  <- X[,rp[[i]]$ind]%*%rp[[i]]$Qs
+    } else {
+      for (i in 1:nr) X[rp[[i]]$ind]  <- t(rp[[i]]$Qs) %*% X[rp[[i]]$ind]
+    }
   }
   X
 } ## end Sl.repara
@@ -393,11 +412,11 @@ Sl.termMult <- function(Sl,A,full=FALSE) {
       ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
       if (full) { ## return zero answer with all zeroes in place
         B <- A*0
-        if (Amat) B[ind,] <- Sl[[b]]$lambda*A[ind,] else  
+        if (Amat) B[ind,] <- Sl[[b]]$lambda*A[ind,,drop=FALSE] else  
                   B[ind] <- Sl[[b]]$lambda*A[ind]
         SA[[k]] <- B
       } else { ## strip zero rows from answer
-        if (Amat) SA[[k]] <- Sl[[b]]$lambda*A[ind,] else
+        if (Amat) SA[[k]] <- Sl[[b]]$lambda*A[ind,,drop=FALSE] else
                   SA[[k]] <- as.numeric(Sl[[b]]$lambda*A[ind])
         attr(SA[[k]],"ind") <- ind
       }
@@ -407,11 +426,11 @@ Sl.termMult <- function(Sl,A,full=FALSE) {
         k <- k + 1
         if (full) { ## return answer with all zeroes in place
           B <- A*0
-          if (Amat) B[ind,] <- Sl[[b]]$Srp[[i]]%*%A[ind,] else  
+          if (Amat) B[ind,] <- Sl[[b]]$Srp[[i]]%*%A[ind,,drop=FALSE] else  
                     B[ind] <- Sl[[b]]$Srp[[i]]%*%A[ind]
           SA[[k]] <- B
         } else { ## strip zero rows from answer
-          if (Amat) SA[[k]] <- Sl[[b]]$Srp[[i]]%*%A[ind,] else
+          if (Amat) SA[[k]] <- Sl[[b]]$Srp[[i]]%*%A[ind,,drop=FALSE] else
                     SA[[k]] <- as.numeric(Sl[[b]]$Srp[[i]]%*%A[ind])
           attr(SA[[k]],"ind") <- ind
         }
@@ -432,7 +451,7 @@ d.detXXS <- function(Sl,PP) {
     d1[i] <- sum(diag(SPP[[i]][,indi,drop=FALSE]))
     for (j in i:nd) {
       indj <- attr(SPP[[j]],"ind")
-      d2[i,j] <- d2[j,i] <- -sum(t(SPP[[i]][,indj])*SPP[[j]][,indi])
+      d2[i,j] <- d2[j,i] <- -sum(t(SPP[[i]][,indj,drop=FALSE])*SPP[[j]][,indi,drop=FALSE])
     }
     d2[i,i] <- d2[i,i] + d1[i]
   }
@@ -472,7 +491,7 @@ Sl.ift <- function(Sl,R,X,y,beta,piv,rp) {
                                 (k==j)*sum(beta*Skb[[k]])
     }
   }
-  list(rss =sum(rsd^2),bSb=sum(beta*Sb),rss1=rss1,bSb1=bSb1,rss2=rss2,bSb2=bSb2)
+  list(rss =sum(rsd^2),bSb=sum(beta*Sb),rss1=rss1,bSb1=bSb1,rss2=rss2,bSb2=bSb2,d1b=db)
 } ## end Sl.ift
 
 Sl.fit <- function(Sl,X,y,rho,fixed,log.phi=0,phi.fixed=TRUE,rss.extra=0,nobs=NULL,Mp=0) {
@@ -524,8 +543,8 @@ Sl.fit <- function(Sl,X,y,rho,fixed,log.phi=0,phi.fixed=TRUE,rss.extra=0,nobs=NU
   #list(reml=dift$rss,reml1=dift$rss1,reml2=dift$rss2)
   #list(reml=dift$bSb,reml1=dift$bSb1,reml2=dift$bSb2) 
   list(reml=as.numeric(reml),reml1=reml1,reml2=reml2,beta=beta[rp],PP=PP,
-       rp=ldS$rp,rss=dift$rss+rss.extra,nobs=nobs)
-}
+       rp=ldS$rp,rss=dift$rss+rss.extra,nobs=nobs,d1b=dift$d1b)
+} ## Sl.fit
 
 fast.REML.fit <- function(Sl,X,y,rho,L=NULL,rho.0=NULL,log.phi=0,phi.fixed=TRUE,
                  rss.extra=0,nobs=NULL,Mp=0,conv.tol=.Machine$double.eps^.5) {
@@ -754,7 +773,12 @@ Sl.postproc <- function(Sl,fit,undrop,X0,cov=FALSE,scale = -1) {
   beta <- rep(0,np)
   beta[undrop] <- Sl.repara(fit$rp,fit$beta,inverse=TRUE)
   beta <- Sl.initial.repara(Sl,beta,inverse=TRUE)
-  if (cov) {
+ 
+  if (cov) { 
+    d1b <- matrix(0,np,ncol(fit$d1b))
+    ## following construction a bit ugly due to Sl.repara assumptions...
+    d1b[undrop,] <- t(Sl.repara(fit$rp,t(fit$d1b),inverse=TRUE,both.sides=FALSE))
+    for (i in 1:ncol(d1b)) d1b[,i] <- Sl.initial.repara(Sl,as.numeric(d1b[,i]),inverse=TRUE) ## d beta / d rho matrix
     PP <- matrix(0,np,np)
     PP[undrop,undrop] <-  Sl.repara(fit$rp,fit$PP,inverse=TRUE)
     PP <- Sl.initial.repara(Sl,PP,inverse=TRUE)
@@ -768,10 +792,20 @@ Sl.postproc <- function(Sl,fit,undrop,X0,cov=FALSE,scale = -1) {
     ## edf <- rowSums(PP*crossprod(X0)) ## diag(PP%*%(t(X0)%*%X0))
     if (scale<=0) scale <- fit$rss/(fit$nobs - sum(edf))
     Vp <- PP * scale ## cov matrix
+    ## sp uncertainty correction... 
+    M <- ncol(d1b) 
+    ev <- eigen(fit$outer.info$hess,symmetric=TRUE)
+    ind <- ev$values <= 0
+    ev$values[ind] <- 0;ev$values[!ind] <- 1/sqrt(ev$values[!ind])
+    rV <- (ev$values*t(ev$vectors))[,1:M]
+    Vc <- crossprod(rV%*%t(d1b))
+    Vc <- Vp + Vc  ## Bayesian cov matrix with sp uncertainty
+    edf2 <- rowSums(Vc*crossprod(X0))/scale
+
     ##bias <- as.numeric(beta-F%*%beta) ## estimate of smoothing bias in beta
-    return(list(beta=beta,Vp=Vp,Ve=F%*%Vp,edf=edf,edf1=edf1,hat=hat,F=F))
+    return(list(beta=beta,Vp=Vp,Vc=Vc,Ve=F%*%Vp,edf=edf,edf1=edf1,edf2=edf2,hat=hat,F=F))
   } else return(list(beta=beta))
-}
+} ## Sl.postproc
 
 
 ## USEAGE SEQUENCE:
