@@ -85,18 +85,21 @@ getGroups.data.frame <-
   if (length(vlist) == 1) return(vlist[[1]]) # ignore level - only one choice
   ## make the list into a data frame with appropriate names
   value <- do.call("data.frame", vlist)
+  row.names(value) <- row.names(object) ## needed for fitted.lme
   if (missing(level)) return(value)
   if (is.character(level)) {
     nlevel <- match(level, names(vlist))
     if (any(aux <- is.na(nlevel))) {
-        stop(gettextf("level of %s does not match formula ",
-                      level[aux], sQuote(deparse(form))), domain = NA)
+        stop(gettextf("level of %s does not match formula %s",
+                      level[aux], sQuote(deparse(form))),
+             domain = "NA")
     }
   } else {
     nlevel <- as.numeric(level)
     if (any(aux <- is.na(match(nlevel, 1:ncol(value))))) {
-        stop(gettextf("level of %s does not match formula ",
-                      level[aux], sQuote(deparse(form))), domain = NA)
+        stop(gettextf("level of %s does not match formula %s",
+                      level[aux], sQuote(deparse(form))),
+             domain = "NA")
     }
   }
   if (length(nlevel) > 1)  return(value[, nlevel]) # multicolumn selection
