@@ -24,6 +24,7 @@
 */
 
 /* Copyright (C) 2004--2008	The R Foundation
+   Copyright (C) 2013		The R Core Team
 
    Additions for R, Chris Jackson
    Find and replace dialog boxes and dialog handlers */
@@ -577,7 +578,7 @@ static int handle_message_dialog(window w)
 
     show(w);
     while (d->hit == NOT_CHOSEN_YET) {
-	if (!peekevent()) WaitMessage();
+	waitevent();
 	doevent();
     }
     hide(w);
