@@ -150,6 +150,7 @@ void  gdrawrect(drawing d, int width, int style, rgb c, rect r, int fast,
 void  gfillrect(drawing d, rgb fill, rect r);
 void  gcopy(drawing d, drawing d2, rect r);
 void  gcopyalpha(drawing d, drawing d2, rect r, int alpha);
+void  gcopyalpha2(drawing d, image src, rect r);
 void  gdrawellipse(drawing d, int width, rgb border, rect r, int fast,
 		   int lend, int ljoin, float lmitre);
 void  gfillellipse(drawing d, rgb fill, rect r);
@@ -171,6 +172,8 @@ void  gcharmetric(drawing d, font f, int c, int *ascent, int *descent,
 		  int *width);
 font  gnewfont(drawing d, const char *face, int style, int size,
 	       double rot, int usePoints);
+font  gnewfont2(drawing d, const char *face, int style, int size,
+		double rot, int usePoints, int quality);
 int   ghasfixedwidth(font f);
 field newfield_no_border(const char *text, rect r);
 
