@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998--2008  The R Development Core Team.
+ *  Copyright (C) 1998--2013  The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,8 +26,6 @@
 
 #define BYTECODE
 
-#define USE_CHAR_HASHING
-
 /* probably no longer needed */
 #define NEW_CONDITION_HANDLING
 
@@ -78,20 +76,27 @@ Rcomplex Rf_ComplexFromInteger(int, int*);
 Rcomplex Rf_ComplexFromReal(double, int*);
 
 #define CALLED_FROM_DEFN_H 1
-#include <Rinternals.h>		/*-> Arith.h, Complex.h, Error.h, Memory.h
-				  PrtUtil.h, Utils.h */
+#include <Rinternals.h>		/*-> Arith.h, Boolean.h, Complex.h, Error.h,
+				  Memory.h, PrtUtil.h, Utils.h */
 #undef CALLED_FROM_DEFN_H
 extern0 SEXP	R_CommentSymbol;    /* "comment" */
 extern0 SEXP	R_DotEnvSymbol;     /* ".Environment" */
 extern0 SEXP	R_ExactSymbol;	    /* "exact" */
-extern0 SEXP	R_LastvalueSymbol;  /* ".Last.value" */
-extern0 SEXP	R_NaRmSymbol;	    /* "na.rm" */
 extern0 SEXP	R_RecursiveSymbol;  /* "recursive" */
-extern0 SEXP	R_SourceSymbol;     /* "source" */
-extern0 SEXP	R_SrcfileSymbol;    /* "srcfile" */
-extern0 SEXP	R_SrcrefSymbol;     /* "srcref" */
+extern0 SEXP	R_WholeSrcrefSymbol;   /* "wholeSrcref" */
 extern0 SEXP	R_TmpvalSymbol;     /* "*tmp*" */
 extern0 SEXP	R_UseNamesSymbol;   /* "use.names" */
+extern0 SEXP	R_DoubleColonSymbol;   /* "::" */
+extern0 SEXP	R_TripleColonSymbol;   /* ":::" */
+extern0 SEXP    R_ConnIdSymbol;  /* "conn_id" */
+extern0 SEXP    R_DevicesSymbol;  /* ".Devices" */
+
+extern0 SEXP    R_dot_Generic;  /* ".Generic" */
+extern0 SEXP    R_dot_Methods;  /* ".Methods" */
+extern0 SEXP    R_dot_Group;  /* ".Group" */
+extern0 SEXP    R_dot_Class;  /* ".Class" */
+extern0 SEXP    R_dot_GenericCallEnv;  /* ".GenericCallEnv" */
+extern0 SEXP    R_dot_GenericDefEnv;  /* ".GenericDefEnv" */
 
 extern0 SEXP	R_StringHash;       /* Global hash of CHARSXPs */
 
@@ -100,52 +105,61 @@ extern0 SEXP	R_StringHash;       /* Global hash of CHARSXPs */
 #define CHAR_RW(x)	((char *) CHAR(x))
 
 /* CHARSXP charset bits */
+#define BYTES_MASK (1<<1)
 #define LATIN1_MASK (1<<2)
 #define UTF8_MASK (1<<3)
+/* (1<<4) is taken by S4_OBJECT_MASK */
+#define CACHED_MASK (1<<5)
+#define ASCII_MASK (1<<6)
+#define HASHASH_MASK 1
+/**** HASHASH uses the first bit -- see HASHASH_MASK defined below */
+
 #ifdef USE_RINTERNALS
+# define IS_BYTES(x) ((x)->sxpinfo.gp & BYTES_MASK)
+# define SET_BYTES(x) (((x)->sxpinfo.gp) |= BYTES_MASK)
 # define IS_LATIN1(x) ((x)->sxpinfo.gp & LATIN1_MASK)
 # define SET_LATIN1(x) (((x)->sxpinfo.gp) |= LATIN1_MASK)
+# define IS_ASCII(x) ((x)->sxpinfo.gp & ASCII_MASK)
+# define SET_ASCII(x) (((x)->sxpinfo.gp) |= ASCII_MASK)
 # define IS_UTF8(x) ((x)->sxpinfo.gp & UTF8_MASK)
 # define SET_UTF8(x) (((x)->sxpinfo.gp) |= UTF8_MASK)
 # define ENC_KNOWN(x) ((x)->sxpinfo.gp & (LATIN1_MASK | UTF8_MASK))
+# define SET_CACHED(x) (((x)->sxpinfo.gp) |= CACHED_MASK)
+# define IS_CACHED(x) (((x)->sxpinfo.gp) & CACHED_MASK)
 #else
 /* Needed only for write-barrier testing */
+int IS_BYTES(SEXP x);
+void SET_BYTES(SEXP x);
 int IS_LATIN1(SEXP x);
 void SET_LATIN1(SEXP x);
-void UNSET_LATIN1(SEXP x);
+int IS_ASCII(SEXP x);
+void SET_ASCII(SEXP x);
 int IS_UTF8(SEXP x);
 void SET_UTF8(SEXP x);
-void UNSET_UTF8(SEXP x);
 int ENC_KNOWN(SEXP x);
+int SET_CACHED(SEXP x);
+int IS_CACHED(SEXP x);
 #endif
 /* macros and declarations for managing CHARSXP cache */
-#ifdef USE_CHAR_HASHING
-# define USE_ATTRIB_FIELD_FOR_CHARSXP_CACHE_CHAINS
-# ifdef USE_ATTRIB_FIELD_FOR_CHARSXP_CACHE_CHAINS
-#  define CXHEAD(x) (x)
-#  define CXTAIL(x) ATTRIB(x)
+# define CXHEAD(x) (x)
+# define CXTAIL(x) ATTRIB(x)
 SEXP (SET_CXTAIL)(SEXP x, SEXP y);
-# else
-#  define CXHEAD(x) CAR(x)
-#  define CXTAIL(x) CDR(x)
-# endif /* USE_ATTRIB_FIELD_FOR_CHARSXP_CACHE_CHAINS */
-#endif /* USE_CHAR_HASHING */
-
-
-#include "Internal.h"		/* do_FOO */
 
 #include "Errormsg.h"
 
-#if defined(Win32) || defined(HAVE_AQUA)
 extern void R_ProcessEvents(void);
+#ifdef Win32
+extern void R_WaitEvent(void);
 #endif
 
+#ifdef R_USE_SIGNALS
 #ifdef Win32
 # include <psignal.h>
 #else
 # include <signal.h>
 # include <setjmp.h>
 #endif
+#endif
 
 #ifdef Unix
 # define OSTYPE      "unix"
@@ -167,15 +181,12 @@ extern void R_ProcessEvents(void);
 
 /*  Heap and Pointer Protection Stack Sizes.  */
 
-/* These are for future protection: will need to be different on Win64 */
-typedef unsigned long R_ulong_t;
-typedef long R_long_t;
-
+/* These are all required by C99 */
 #ifdef HAVE_INTTYPES_H
 # include <inttypes.h>
 #endif
-/* According to POSIX inttypes.h should include stdint.h, but let's be
-   sure. */
+/* According to POSIX inttypes.h should include stdint.h,
+   but let's be sure. */
 #ifdef HAVE_STDINT_H
 # include <stdint.h>
 #endif
@@ -183,26 +194,11 @@ typedef long R_long_t;
 # include <limits.h>
 #endif
 
-/* NB: will need a 64-bit type, ULONG64 or size_t, for Win64 */
 #if defined HAVE_DECL_SIZE_MAX && HAVE_DECL_SIZE_MAX
   typedef size_t R_size_t;
-/* final precaution in case we don't have the right headers */
-# ifdef SIZE_MAX
-#  define R_SIZE_T_MAX SIZE_MAX
-# else
-#  define R_SIZE_T_MAX ULONG_MAX
-# endif
+# define R_SIZE_T_MAX SIZE_MAX
 #else
-  typedef unsigned long R_size_t;
-# define R_SIZE_T_MAX ULONG_MAX
-#endif
-
-/* These are optional C99 types */
-#if !defined(HAVE_INTPTR_T) && !defined(intptr_t)
- typedef long intptr_t;
-#endif
-#if !defined(HAVE_UINTPTR_T) && !defined(uintptr_t)
- typedef unsigned long uintptr_t;
+# error SIZE_MAX is required for C99
 #endif
 
 
@@ -213,7 +209,7 @@ typedef long R_long_t;
 /*	R_NSIZE	   The number of cons cells	 */
 /*	R_VSIZE	   The vector heap size in bytes */
 /*  These values are defaults and can be overridden in config.h
-    The maxima and minima are in ../unix/sys-common.c */
+    The maxima and minima are in startup.c */
 
 #ifndef R_PPSSIZE
 #define	R_PPSSIZE	50000L
@@ -227,33 +223,16 @@ typedef long R_long_t;
 
 /* some commonly needed headers */
 #include <math.h>
-#include <float.h>
-#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 
 /* declare substitutions */
-#if !defined(acosh) && defined(HAVE_DECL_ACOSH) && !HAVE_DECL_ACOSH
-extern double acosh(double x);
-#endif
-#if !defined(asinh) && defined(HAVE_DECL_ASINH) && !HAVE_DECL_ASINH
-extern double asinh(double x);
-#endif
-#if !defined(atanh) && defined(HAVE_DECL_ATANH) && !HAVE_DECL_ATANH
-extern double atanh(double x);
-#endif
-#if !defined(snprintf) && defined(HAVE_DECL_SNPRINTF) && !HAVE_DECL_SNPRINTF
-extern int snprintf (char *s, size_t n, const char *format, ...);
-#endif
 #if !defined(strdup) && defined(HAVE_DECL_STRDUP) && !HAVE_DECL_STRDUP
 extern char *strdup(const char *s1);
 #endif
 #if !defined(strncascmp) && defined(HAVE_DECL_STRNCASECMP) && !HAVE_DECL_STRNCASECMP
 extern int strncasecmp(const char *s1, const char *s2, size_t n);
 #endif
-#if !defined(vsnprintf) && defined(HAVE_DECL_VSNPRINTF) && !HAVE_DECL_VSNPRINTF
-extern int vsnprintf (char *str, size_t count, const char *fmt, va_list arg);
-#endif
 
 /* Glibc manages to not define this in -pedantic -ansi */
 #if defined(HAVE_PUTENV) && !defined(putenv) && defined(HAVE_DECL_PUTENV) && !HAVE_DECL_PUTENV
@@ -279,6 +258,9 @@ extern int putenv(char *string);
 /* seems this is now defined by MinGW to be 259, whereas FILENAME_MAX
    and MAX_PATH are 260.  It is not clear that this really is in bytes,
    but might be chars for the Unicode interfaces.
+
+   260 is d:\ plus 256 chars plus nul.  Some but not all API calls
+   allow filepaths of the form \\?\D:\very_long_path .
 */
 #    define PATH_MAX 260
 #  else
@@ -288,6 +270,7 @@ extern int putenv(char *string);
 # endif
 #endif
 
+#ifdef R_USE_SIGNALS
 #ifdef HAVE_POSIX_SETJMP
 # define SIGJMP_BUF sigjmp_buf
 # define SIGSETJMP(x,s) sigsetjmp(x,s)
@@ -303,16 +286,13 @@ extern int putenv(char *string);
 # define SETJMP(x) setjmp(x)
 # define LONGJMP(x,i) longjmp(x,i)
 #endif
-
-/* Availability of timing: on Unix, we need times(2).
-   On Windows and the Mac, we can do without.
-*/
-#if (defined(HAVE_TIMES) || defined(Win32))
-# define _R_HAVE_TIMING_ 1
 #endif
 
 #define HSIZE	   4119	/* The size of the hash table for symbols */
-#define MAXIDSIZE   256	/* Largest symbol size possible */
+#define MAXIDSIZE 10000	/* Largest symbol size,
+			   in bytes excluding terminator.
+			   Was 256 prior to 2.13.0, now just a sanity check.
+			*/
 
 /* The type of the do_xxxx functions. */
 /* These are the built-in R functions. */
@@ -405,9 +385,10 @@ typedef struct {
 #define SET_PRSEEN(x,v)	(((x)->sxpinfo.gp)=(v))
 
 /* Hashing Macros */
-#define HASHASH(x)      ((x)->sxpinfo.gp)
+#define HASHASH(x)      ((x)->sxpinfo.gp & HASHASH_MASK)
 #define HASHVALUE(x)    TRUELENGTH(x)
-#define SET_HASHASH(x,v) (((x)->sxpinfo.gp)=(v))
+#define SET_HASHASH(x,v) ((v) ? (((x)->sxpinfo.gp) |= HASHASH_MASK) : \
+			  (((x)->sxpinfo.gp) &= (~HASHASH_MASK)))
 #define SET_HASHVALUE(x,v) SET_TRUELENGTH(x, v)
 
 /* Vector Heap Structure */
@@ -437,6 +418,19 @@ typedef struct {
 #define LOCK_BINDING(b) ((b)->sxpinfo.gp |= BINDING_LOCK_MASK)
 #define UNLOCK_BINDING(b) ((b)->sxpinfo.gp &= (~BINDING_LOCK_MASK))
 
+#define BASE_SYM_CACHED_MASK (1<<13)
+#define SET_BASE_SYM_CACHED(b) ((b)->sxpinfo.gp |= BASE_SYM_CACHED_MASK)
+#define UNSET_BASE_SYM_CACHED(b) ((b)->sxpinfo.gp &= (~BASE_SYM_CACHED_MASK))
+#define BASE_SYM_CACHED(b) ((b)->sxpinfo.gp & BASE_SYM_CACHED_MASK)
+
+#define SPECIAL_SYMBOL_MASK (1<<12)
+#define SET_SPECIAL_SYMBOL(b) ((b)->sxpinfo.gp |= SPECIAL_SYMBOL_MASK)
+#define UNSET_SPECIAL_SYMBOL(b) ((b)->sxpinfo.gp &= (~SPECIAL_SYMBOL_MASK))
+#define IS_SPECIAL_SYMBOL(b) ((b)->sxpinfo.gp & SPECIAL_SYMBOL_MASK)
+#define SET_NO_SPECIAL_SYMBOLS(b) ((b)->sxpinfo.gp |= SPECIAL_SYMBOL_MASK)
+#define UNSET_NO_SPECIAL_SYMBOLS(b) ((b)->sxpinfo.gp &= (~SPECIAL_SYMBOL_MASK))
+#define NO_SPECIAL_SYMBOLS(b) ((b)->sxpinfo.gp & SPECIAL_SYMBOL_MASK)
+
 #else /* USE_RINTERNALS */
 
 typedef struct VECREC *VECP;
@@ -458,14 +452,25 @@ void (SET_ACTIVE_BINDING_BIT)(SEXP b);
 void (LOCK_BINDING)(SEXP b);
 void (UNLOCK_BINDING)(SEXP b);
 
+void (SET_BASE_SYM_CACHED)(SEXP b);
+void (UNSET_BASE_SYM_CACHED)(SEXP b);
+Rboolean (BASE_SYM_CACHED)(SEXP b);
+
+void (SET_SPECIAL_SYMBOL)(SEXP b);
+void (UNSET_SPECIAL_SYMBOL)(SEXP b);
+Rboolean (IS_SPECIAL_SYMBOL)(SEXP b);
+void (SET_NO_SPECIAL_SYMBOLS)(SEXP b);
+void (UNSET_NO_SPECIAL_SYMBOLS)(SEXP b);
+Rboolean (NO_SPECIAL_SYMBOLS)(SEXP b);
+
 #endif /* USE_RINTERNALS */
 
-#ifdef BYTECODE
-# ifdef BC_INT_STACK
+typedef SEXP R_bcstack_t;
+#ifdef BC_INT_STACK
 typedef union { void *p; int i; } IStackval;
-# endif
 #endif
 
+#ifdef R_USE_SIGNALS
 /* Stack entry for pending promises */
 typedef struct RPRSTACK {
     SEXP promise;
@@ -492,12 +497,12 @@ typedef struct RCNTXT {
     SEXP handlerstack;          /* condition handler stack */
     SEXP restartstack;          /* stack of available restarts */
     struct RPRSTACK *prstack;   /* stack of pending promises */
-#ifdef BYTECODE
     SEXP *nodestack;
-# ifdef BC_INT_STACK
+#ifdef BC_INT_STACK
     IStackval *intstack;
-# endif
 #endif
+    SEXP srcref;	        /* The source line in effect */
+    int browserfinish;     /* should browser finish this context without stopping */
 } RCNTXT, *context;
 
 /* The Various Context Types.
@@ -541,6 +546,7 @@ BUI   0 0 0 0 0 0 0 1 = 64
 #define IS_RESTART_BIT_SET(flags) ((flags) & CTXT_RESTART)
 #define SET_RESTART_BIT_ON(flags) (flags |= CTXT_RESTART)
 #define SET_RESTART_BIT_OFF(flags) (flags &= ~CTXT_RESTART)
+#endif
 
 /* Miscellaneous Definitions */
 #define streql(s, t)	(!strcmp((s), (t)))
@@ -591,8 +597,9 @@ FUNTAB	R_FunTab[];	    /* Built in functions */
 #define extern0 extern
 #endif
 
-/* extern int	errno; already have errno.h ! */
-extern int	gc_inhibit_torture INI_as(1);
+LibExtern SEXP  R_SrcfileSymbol;    /* "srcfile" */
+LibExtern SEXP  R_SrcrefSymbol;     /* "srcref" */
+
 
 LibExtern Rboolean R_interrupts_suspended INI_as(FALSE);
 LibExtern int R_interrupts_pending INI_as(0);
@@ -606,34 +613,40 @@ extern0 R_size_t R_VSize  INI_as(R_VSIZE);/* Size of the vector heap */
 extern0 SEXP	R_NHeap;	    /* Start of the cons cell heap */
 extern0 SEXP	R_FreeSEXP;	    /* Cons cell free list */
 extern0 R_size_t R_Collected;	    /* Number of free cons cells (after gc) */
-LibExtern int	R_Is_Running;	    /* for Windows memory manager */
+extern0 int	R_Is_Running;	    /* for Windows memory manager */
 
 /* The Pointer Protection Stack */
-extern0 int	R_PPStackSize	INI_as(R_PPSSIZE); /* The stack size (elements) */
-extern0 int	R_PPStackTop;	    /* The top of the stack */
-extern0 SEXP*	R_PPStack;	    /* The pointer protection stack */
+LibExtern int	R_PPStackSize	INI_as(R_PPSSIZE); /* The stack size (elements) */
+LibExtern int	R_PPStackTop;	    /* The top of the stack */
+LibExtern SEXP*	R_PPStack;	    /* The pointer protection stack */
 
 /* Evaluation Environment */
-LibExtern SEXP	R_CurrentExpr;	    /* Currently evaluating expression */
+extern0 SEXP	R_CurrentExpr;	    /* Currently evaluating expression */
 extern0 SEXP	R_ReturnedValue;    /* Slot for return-ing values */
 extern0 SEXP*	R_SymbolTable;	    /* The symbol table */
-LibExtern RCNTXT R_Toplevel;	    /* Storage for the toplevel environment */
-LibExtern RCNTXT* R_ToplevelContext;  /* The toplevel environment */
-LibExtern RCNTXT* R_GlobalContext;    /* The global environment */
-extern0 Rboolean R_Visible;	    /* Value visibility flag */
-LibExtern int	R_EvalDepth	INI_as(0);	/* Evaluation recursion depth */
-extern0 int	R_BrowseLevel	INI_as(0);	/* how deep the browser is */
+#ifdef R_USE_SIGNALS
+extern0 RCNTXT R_Toplevel;	      /* Storage for the toplevel context */
+extern0 RCNTXT* R_ToplevelContext;  /* The toplevel context */
+LibExtern RCNTXT* R_GlobalContext;    /* The global context */
+extern0 RCNTXT* R_SessionContext;   /* The session toplevel context */
+#endif
+extern Rboolean R_Visible;	    /* Value visibility flag */
+extern0 int	R_EvalDepth	INI_as(0);	/* Evaluation recursion depth */
 extern0 int	R_BrowseLines	INI_as(0);	/* lines/per call in browser */
 
 extern0 int	R_Expressions	INI_as(5000);	/* options(expressions) */
 extern0 int	R_Expressions_keep INI_as(5000);	/* options(expressions) */
 extern0 Rboolean R_KeepSource	INI_as(FALSE);	/* options(keep.source) */
+extern0 Rboolean R_CBoundsCheck	INI_as(FALSE);	/* options(CBoundsCheck) */
 extern0 int	R_WarnLength	INI_as(1000);	/* Error/warning max length */
+extern0 int	R_nwarnings	INI_as(50);
 extern uintptr_t R_CStackLimit	INI_as((uintptr_t)-1);	/* C stack limit */
 extern uintptr_t R_CStackStart	INI_as((uintptr_t)-1);	/* Initial stack address */
-extern0 int	R_CStackDir	INI_as(1);	/* C stack direction */
-extern0 Rboolean R_WarnEscapes  INI_as(TRUE);   /* Warn on unrecognized escapes */
+extern int	R_CStackDir	INI_as(1);	/* C stack direction */
+
+#ifdef R_USE_SIGNALS
 extern0 struct RPRSTACK *R_PendingPromises INI_as(NULL); /* Pending promise stack */
+#endif
 
 /* File Input/Output */
 LibExtern Rboolean R_Interactive INI_as(TRUE);	/* TRUE during interactive use*/
@@ -652,14 +665,16 @@ extern0 char   *Sys_TempDir	INI_as(NULL);	/* Name of per-session dir
 extern0 char	R_StdinEnc[31]  INI_as("");	/* Encoding assumed for stdin */
 
 /* Objects Used In Parsing  */
-extern0 SEXP	R_CommentSxp;	    /* Comments accumulate here */
-extern0 int	R_ParseError	INI_as(0); /* Line where parse error occured */
-extern0 SEXP	R_ParseErrorFile;   /* Source file where parse error was seen */
+LibExtern int	R_ParseError	INI_as(0); /* Line where parse error occurred */
+extern0 int	R_ParseErrorCol;    /* Column of start of token where parse error occurred */
+extern0 SEXP	R_ParseErrorFile;   /* Source file where parse error was seen.  Either a
+				       STRSXP or (when keeping srcrefs) a SrcFile ENVSXP */
 #define PARSE_ERROR_SIZE 256	    /* Parse error messages saved here */
-extern0 char	R_ParseErrorMsg[PARSE_ERROR_SIZE] INI_as("");
+LibExtern char	R_ParseErrorMsg[PARSE_ERROR_SIZE] INI_as("");
 #define PARSE_CONTEXT_SIZE 256	    /* Recent parse context kept in a circular buffer */
-extern0 char	R_ParseContext[PARSE_CONTEXT_SIZE] INI_as("");
-extern0 int	R_ParseContextLast INI_as(0); /* last character in context buffer */
+LibExtern char	R_ParseContext[PARSE_CONTEXT_SIZE] INI_as("");
+LibExtern int	R_ParseContextLast INI_as(0); /* last character in context buffer */
+LibExtern int	R_ParseContextLine; /* Line in file of the above */
 
 /* Image Dump/Restore */
 extern int	R_DirtyImage	INI_as(0);	/* Current image dirty */
@@ -681,7 +696,7 @@ extern0 Rboolean R_warn_partial_match_dollar INI_as(FALSE);
 extern0 Rboolean R_warn_partial_match_attr INI_as(FALSE);
 extern0 Rboolean R_ShowWarnCalls INI_as(FALSE);
 extern0 Rboolean R_ShowErrorCalls INI_as(FALSE);
-extern0 int R_NShowCalls INI_as(50);
+extern0 int	R_NShowCalls INI_as(50);
 
 LibExtern Rboolean utf8locale  INI_as(FALSE);  /* is this a UTF-8 locale? */
 LibExtern Rboolean mbcslocale  INI_as(FALSE);  /* is this a MBCS locale? */
@@ -692,7 +707,9 @@ extern0   Rboolean WinUTF8out  INI_as(FALSE);  /* Use UTF-8 for output */
 extern0   void WinCheckUTF8(void);
 #endif
 
-extern0 char OutDec	INI_as('.');  /* decimal point used for output */
+extern char OutDec	INI_as('.');  /* decimal point used for output */
+extern0 Rboolean R_DisableNLinBrowser	INI_as(FALSE);
+extern0 char R_BrowserLastCommand	INI_as('n');
 
 /* Initialization of the R environment when it is embedded */
 extern int Rf_initEmbeddedR(int argc, char **argv);
@@ -700,19 +717,35 @@ extern int Rf_initEmbeddedR(int argc, char **argv);
 /* GUI type */
 
 extern char	*R_GUIType	INI_as("unknown");
+extern Rboolean R_isForkedChild		INI_as(FALSE); /* was this forked? */
 
-#ifdef BYTECODE
-#define R_BCNODESTACKSIZE 10000
+extern0 double cpuLimit			INI_as(-1.0);
+extern0 double cpuLimit2	       	INI_as(-1.0);
+extern0 double cpuLimitValue		INI_as(-1.0);
+extern0 double elapsedLimit		INI_as(-1.0);
+extern0 double elapsedLimit2		INI_as(-1.0);
+extern0 double elapsedLimitValue       	INI_as(-1.0);
+
+void resetTimeLimits(void);
+
+#define R_BCNODESTACKSIZE 100000
 extern0 SEXP *R_BCNodeStackBase, *R_BCNodeStackTop, *R_BCNodeStackEnd;
-# ifdef BC_INT_STACK
-#define R_BCINTSTACKSIZE 10000
+#ifdef BC_INT_STACK
+# define R_BCINTSTACKSIZE 10000
 extern0 IStackval *R_BCIntStackBase, *R_BCIntStackTop, *R_BCIntStackEnd;
-# endif
 #endif
+extern0 int R_jit_enabled INI_as(0);
+extern0 int R_compile_pkgs INI_as(0);
+extern SEXP R_cmpfun(SEXP);
+extern void R_init_jit_enabled(void);
+extern void R_initAsignSymbols(void);
+
+LibExtern int R_num_math_threads INI_as(1);
+LibExtern int R_max_num_math_threads INI_as(1);
 
 /* Pointer  type and utilities for dispatch in the methods package */
 typedef SEXP (*R_stdGen_ptr_t)(SEXP, SEXP, SEXP); /* typedef */
-R_stdGen_ptr_t R_get_standardGeneric_ptr(void); /* get method */
+//R_stdGen_ptr_t R_get_standardGeneric_ptr(void); /* get method */
 R_stdGen_ptr_t R_set_standardGeneric_ptr(R_stdGen_ptr_t, SEXP); /* set method */
 LibExtern SEXP R_MethodsNamespace;
 SEXP R_deferred_default_method(void);
@@ -735,11 +768,16 @@ typedef struct {
 
 LibExtern AccuracyInfo R_AccuracyInfo;
 
-extern0 unsigned int max_contour_segments INI_as(25000);
+extern unsigned int max_contour_segments INI_as(25000);
 
-extern0 Rboolean known_to_be_latin1 INI_as(FALSE);
+/* used in package utils */
+extern Rboolean known_to_be_latin1 INI_as(FALSE);
 extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 
+/* pre-allocated boolean values */
+LibExtern SEXP R_TrueValue INI_as(NULL);
+LibExtern SEXP R_FalseValue INI_as(NULL);
+LibExtern SEXP R_LogicalNAValue INI_as(NULL);
 
 #ifdef __MAIN__
 # undef extern
@@ -752,8 +790,11 @@ extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 
 /*--- FUNCTIONS ------------------------------------------------------ */
 
+# define allocCharsxp		Rf_allocCharsxp
+# define asVecSize		Rf_asVecSize
 # define begincontext		Rf_begincontext
 # define check_stack_balance	Rf_check_stack_balance
+# define check1arg		Rf_check1arg
 # define CheckFormals		Rf_CheckFormals
 # define CleanEd		Rf_CleanEd
 # define CoercionWarning       	Rf_CoercionWarning
@@ -761,16 +802,17 @@ extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 # define ComplexFromLogical	Rf_ComplexFromLogical
 # define ComplexFromReal	Rf_ComplexFromReal
 # define ComplexFromString	Rf_ComplexFromString
-# define copyListMatrix		Rf_copyListMatrix
 # define copyMostAttribNoTs	Rf_copyMostAttribNoTs
 # define CustomPrintValue	Rf_CustomPrintValue
 # define DataFrameClass		Rf_DataFrameClass
 # define ddfindVar		Rf_ddfindVar
 # define deparse1		Rf_deparse1
+# define deparse1w		Rf_deparse1w
 # define deparse1line		Rf_deparse1line
 # define deparse1s		Rf_deparse1s
 # define DispatchGroup		Rf_DispatchGroup
 # define DispatchOrEval		Rf_DispatchOrEval
+# define DispatchAnyOrEval      Rf_DispatchAnyOrEval
 # define dynamicfindVar		Rf_dynamicfindVar
 # define EncodeRaw              Rf_EncodeRaw
 # define EncodeString           Rf_EncodeString
@@ -787,10 +829,10 @@ extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 # define framedepth		Rf_framedepth
 # define frameSubscript		Rf_frameSubscript
 # define get1index		Rf_get1index
+# define GetOptionCutoff       	Rf_GetOptionCutoff
 # define getVar			Rf_getVar
 # define getVarInFrame		Rf_getVarInFrame
 # define InitArithmetic		Rf_InitArithmetic
-# define InitColors		Rf_InitColors
 # define InitConnections	Rf_InitConnections
 # define InitEd			Rf_InitEd
 # define InitFunctionHashing	Rf_InitFunctionHashing
@@ -809,6 +851,7 @@ extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 # define IntegerFromString	Rf_IntegerFromString
 # define internalTypeCheck	Rf_internalTypeCheck
 # define isValidName		Rf_isValidName
+# define installTrChar		Rf_installTrChar
 # define ItemName		Rf_ItemName
 # define jump_to_toplevel	Rf_jump_to_toplevel
 # define KillAllDevices		Rf_KillAllDevices
@@ -844,6 +887,7 @@ extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 # define PrintValueEnv		Rf_PrintValueEnv
 # define PrintValueRec		Rf_PrintValueRec
 # define PrintVersion		Rf_PrintVersion
+# define PrintVersion_part_1	Rf_PrintVersion_part_1
 # define PrintVersionString    	Rf_PrintVersionString
 # define PrintWarnings		Rf_PrintWarnings
 # define promiseArgs		Rf_promiseArgs
@@ -854,6 +898,7 @@ extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 # define Seql			Rf_Seql
 # define Scollate		Rf_Scollate
 # define sortVector		Rf_sortVector
+# define SrcrefPrompt		Rf_SrcrefPrompt
 # define ssort			Rf_ssort
 # define StringFromComplex	Rf_StringFromComplex
 # define StringFromInteger	Rf_StringFromInteger
@@ -861,6 +906,7 @@ extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 # define StringFromReal		Rf_StringFromReal
 # define strIsASCII		Rf_strIsASCII
 # define StrToInternal		Rf_StrToInternal
+# define strmat2intmat		Rf_strmat2intmat
 # define substituteList		Rf_substituteList
 # define tsConform		Rf_tsConform
 # define tspgets		Rf_tspgets
@@ -871,10 +917,11 @@ extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 # define ucstoutf8		Rf_ucstoutf8
 # define utf8toucs		Rf_utf8toucs
 # define utf8towcs		Rf_utf8towcs
-# define vectorSubscript	Rf_vectorSubscript
+# define vectorIndex		Rf_vectorIndex
 # define warningcall		Rf_warningcall
 # define WarningMessage		Rf_WarningMessage
 # define wcstoutf8		Rf_wcstoutf8
+# define wtransChar		Rf_wtransChar
 # define yychar			Rf_yychar
 # define yylval			Rf_yylval
 # define yynerrs		Rf_yynerrs
@@ -886,8 +933,9 @@ extern0 Rboolean known_to_be_utf8 INI_as(FALSE);
 #define	R_FILE		2
 #define R_TEXT		3
 
-/* The maximum length of input line which will be asked for */
-#define CONSOLE_BUFFER_SIZE 1024
+/* The maximum length of input line which will be asked for,
+   in bytes, including the terminator */
+#define CONSOLE_BUFFER_SIZE 4096
 int	R_ReadConsole(const char *, unsigned char *, int, int);
 void	R_WriteConsole(const char *, int); /* equivalent to R_WriteConsoleEx(a, b, 0) */
 void	R_WriteConsoleEx(const char *, int, int);
@@ -940,40 +988,42 @@ SEXP Rf_EnsureString(SEXP);
 
 /* Other Internally Used Functions */
 
+SEXP Rf_allocCharsxp(R_len_t);
 SEXP Rf_append(SEXP, SEXP); /* apparently unused now */
-void begincontext(RCNTXT*, int, SEXP, SEXP, SEXP, SEXP, SEXP);
+R_xlen_t asVecSize(SEXP x);
+void check1arg(SEXP, SEXP, const char *);
 void Rf_checkArityCall(SEXP, SEXP, SEXP);
 void CheckFormals(SEXP);
 void R_check_locale(void);
 void check_stack_balance(SEXP op, int save);
 void CleanEd(void);
-void copyListMatrix(SEXP, SEXP, Rboolean);
 void copyMostAttribNoTs(SEXP, SEXP);
-void CustomPrintValue(SEXP,SEXP);
+void CustomPrintValue(SEXP, SEXP);
 void DataFrameClass(SEXP);
 SEXP ddfindVar(SEXP, SEXP);
 SEXP deparse1(SEXP,Rboolean,int);
+SEXP deparse1w(SEXP,Rboolean,int);
 SEXP deparse1line(SEXP,Rboolean);
 SEXP deparse1s(SEXP call);
+int DispatchAnyOrEval(SEXP, SEXP, const char *, SEXP, SEXP, SEXP*, int, int);
 int DispatchOrEval(SEXP, SEXP, const char *, SEXP, SEXP, SEXP*, int, int);
 int DispatchGroup(const char *, SEXP,SEXP,SEXP,SEXP,SEXP*);
 SEXP duplicated(SEXP, Rboolean);
-SEXP dynamicfindVar(SEXP, RCNTXT*);
-void endcontext(RCNTXT*);
+R_xlen_t any_duplicated(SEXP, Rboolean);
+R_xlen_t any_duplicated3(SEXP, SEXP, Rboolean);
 int envlength(SEXP);
-SEXP evalList(SEXP, SEXP, SEXP);
+SEXP evalList(SEXP, SEXP, SEXP, int);
 SEXP evalListKeepMissing(SEXP, SEXP);
 int factorsConform(SEXP, SEXP);
 void findcontext(int, SEXP, SEXP);
 SEXP findVar1(SEXP, SEXP, SEXPTYPE, int);
 void FrameClassFix(SEXP);
-int framedepth(RCNTXT*);
 SEXP frameSubscript(int, SEXP, SEXP);
-int get1index(SEXP, SEXP, int, int, int, SEXP);
+R_xlen_t get1index(SEXP, SEXP, R_xlen_t, int, int, SEXP);
+int GetOptionCutoff(void);
 SEXP getVar(SEXP, SEXP);
 SEXP getVarInFrame(SEXP, SEXP);
 void InitArithmetic(void);
-void InitColors(void);
 void InitConnections(void);
 void InitEd(void);
 void InitFunctionHashing(void);
@@ -991,16 +1041,14 @@ void InitStringHash(void);
 void Init_R_Variables(SEXP);
 void InitTempDir(void);
 void initStack(void);
-void R_InsertRestartHandlers(RCNTXT *, Rboolean);
 void internalTypeCheck(SEXP, SEXP, SEXPTYPE);
 Rboolean isMethodsDispatchOn(void);
 int isValidName(const char *);
-void R_JumpToContext(RCNTXT *, int, SEXP);
 void jump_to_toplevel(void);
 void KillAllDevices(void);
 SEXP levelsgets(SEXP, SEXP);
 void mainloop(void);
-SEXP makeSubscript(SEXP, SEXP, int *, SEXP);
+SEXP makeSubscript(SEXP, SEXP, R_xlen_t *, SEXP);
 SEXP markKnown(const char *, SEXP);
 SEXP mat2indsub(SEXP, SEXP, SEXP);
 SEXP matchArg(SEXP, SEXP*);
@@ -1012,6 +1060,8 @@ SEXP mkCLOSXP(SEXP, SEXP, SEXP);
 SEXP mkFalse(void);
 SEXP mkPRIMSXP (int, int);
 SEXP mkPROMISE(SEXP, SEXP);
+SEXP R_mkEVPROMISE(SEXP, SEXP);
+SEXP R_mkEVPROMISE_NR(SEXP, SEXP);
 SEXP mkQUOTE(SEXP);
 SEXP mkSYMSXP(SEXP, SEXP);
 SEXP mkTrue(void);
@@ -1019,14 +1069,15 @@ SEXP NewEnvironment(SEXP, SEXP, SEXP);
 void onintr(void);
 RETSIGTYPE onsigusr1(int);
 RETSIGTYPE onsigusr2(int);
-int OneIndex(SEXP, SEXP, int, int, SEXP*, int, SEXP);
+R_xlen_t OneIndex(SEXP, SEXP, R_xlen_t, int, SEXP*, int, SEXP);
 SEXP parse(FILE*, int);
-void PrintDefaults(SEXP);
+void PrintDefaults(void);
 void PrintGreeting(void);
 void PrintValueEnv(SEXP, SEXP);
 void PrintValueRec(SEXP, SEXP);
-void PrintVersion(char *);
-void PrintVersionString(char *);
+void PrintVersion(char *, size_t len);
+void PrintVersion_part_1(char *, size_t len);
+void PrintVersionString(char *, size_t len);
 void PrintWarnings(void);
 void process_site_Renviron(void);
 void process_system_Renviron(void);
@@ -1040,6 +1091,7 @@ SEXP R_LoadFromFile(FILE*, int);
 SEXP R_NewHashedEnv(SEXP, SEXP);
 extern int R_Newhashpjw(const char *);
 FILE* R_OpenLibraryFile(const char *);
+SEXP R_Primitive(const char *);
 void R_RestoreGlobalEnv(void);
 void R_RestoreGlobalEnvFromFile(const char *, Rboolean);
 void R_SaveGlobalEnv(void);
@@ -1052,14 +1104,13 @@ int R_SetOptionWidth(int);
 void R_Suicide(const char *);
 void R_getProcTime(double *data);
 int R_isMissing(SEXP symbol, SEXP rho);
+const char *sexptype2char(SEXPTYPE type);
 void sortVector(SEXP, Rboolean);
+void SrcrefPrompt(const char *, SEXP);
 void ssort(SEXP*,int);
 int StrToInternal(const char *);
+SEXP strmat2intmat(SEXP, SEXP, SEXP);
 SEXP substituteList(SEXP, SEXP);
-SEXP R_syscall(int,RCNTXT*);
-int R_sysparent(int,RCNTXT*);
-SEXP R_sysframe(int,RCNTXT*);
-SEXP R_sysfunction(int,RCNTXT*);
 Rboolean tsConform(SEXP,SEXP);
 SEXP tspgets(SEXP, SEXP);
 SEXP type2symbol(SEXPTYPE);
@@ -1069,11 +1120,26 @@ void unmarkPhase(void);
 #endif
 SEXP R_LookupMethod(SEXP, SEXP, SEXP, SEXP);
 int usemethod(const char *, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP*);
-SEXP Rf_vectorSubscript(int, SEXP, int*, SEXP (*)(SEXP,SEXP),
-                        SEXP (*)(SEXP, int), SEXP, SEXP);
+SEXP vectorIndex(SEXP, SEXP, int, int, int, SEXP, Rboolean);
+
+#ifdef R_USE_SIGNALS
+void begincontext(RCNTXT*, int, SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP dynamicfindVar(SEXP, RCNTXT*);
+void endcontext(RCNTXT*);
+int framedepth(RCNTXT*);
+void R_InsertRestartHandlers(RCNTXT *, Rboolean);
+void R_JumpToContext(RCNTXT *, int, SEXP);
+SEXP R_syscall(int,RCNTXT*);
+int R_sysparent(int,RCNTXT*);
+SEXP R_sysframe(int,RCNTXT*);
+SEXP R_sysfunction(int,RCNTXT*);
+
+void R_run_onexits(RCNTXT *);
+void R_restore_globals(RCNTXT *);
+#endif
 
 /* ../main/bind.c */
-SEXP ItemName(SEXP, int);
+SEXP ItemName(SEXP, R_xlen_t);
 
 /* ../main/errors.c : */
 void ErrorMessage(SEXP, int, ...);
@@ -1087,15 +1153,9 @@ void R_SetMaxNSize(R_size_t);
 R_size_t R_Decode2Long(char *p, int *ierr);
 void R_SetPPSize(R_size_t);
 
-void R_run_onexits(RCNTXT *);
-void R_restore_globals(RCNTXT *);
-
 /* ../main/devices.c, used in memory.c, gnuwin32/extra.c */
 #define R_MaxDevices 64
 
-/* ../main/identical.c : */
-Rboolean compute_identical(SEXP x, SEXP y);
-
 /* ../../main/printutils.c : */
 typedef enum {
     Rprt_adj_left = 0,
@@ -1105,14 +1165,15 @@ typedef enum {
 } Rprt_adj;
 
 int	Rstrlen(SEXP, int);
-const char *EncodeRaw(Rbyte);
+const char *EncodeRaw(Rbyte, const char *);
 const char *EncodeString(SEXP, int, int, Rprt_adj);
 const char *EncodeReal2(double, int, int, int);
+const char *EncodeChar(SEXP);
 
 
 /* main/sort.c */
 void orderVector1(int *indx, int n, SEXP key, Rboolean nalast,
-		  Rboolean decreasing);
+		  Rboolean decreasing, SEXP rho);
 
 /* main/subset.c */
 SEXP R_subset3_dflt(SEXP, SEXP, SEXP);
@@ -1120,16 +1181,14 @@ SEXP R_subset3_dflt(SEXP, SEXP, SEXP);
 /* main/subassign.c */
 SEXP R_subassign3_dflt(SEXP, SEXP, SEXP, SEXP);
 
-#ifdef SUPPORT_MBCS /* implies we have this header */
 #include <wchar.h>
-#endif
 
 /* main/util.c */
 void UNIMPLEMENTED_TYPE(const char *s, SEXP x);
 void UNIMPLEMENTED_TYPEt(const char *s, SEXPTYPE t);
 Rboolean Rf_strIsASCII(const char *str);
 int utf8clen(char c);
-#ifdef SUPPORT_MBCS
+
 typedef unsigned short ucs2_t;
 size_t mbcsToUcs2(const char *in, ucs2_t *out, int nout, int enc);
 /* size_t mbcsMblen(char *in);
@@ -1142,22 +1201,23 @@ size_t ucstoutf8(char *s, const unsigned int wc);
 size_t mbtoucs(unsigned int *wc, const char *s, size_t n);
 size_t wcstoutf8(char *s, const wchar_t *wc, size_t n);
 
+SEXP Rf_installTrChar(SEXP);
+
+const wchar_t *wtransChar(SEXP x); /* from sysutils.c */
+
 #define mbs_init(x) memset(x, 0, sizeof(mbstate_t))
 size_t Mbrtowc(wchar_t *wc, const char *s, size_t n, mbstate_t *ps);
-/* void mbcsToLatin1(const char *in, char *out); */
 Rboolean mbcsValid(const char *str);
+Rboolean utf8Valid(const char *str);
 char *Rf_strchr(const char *s, int c);
 char *Rf_strrchr(const char *s, int c);
-#else
-#define Rf_strchr(s, c) strchr(s, c)
-#define Rf_strrchr(s, c) strrchr(s, c)
-#endif
+
 #ifdef Win32
 void R_fixslash(char *s);
 void R_fixbackslash(char *s);
 wchar_t *filenameToWchar(const SEXP fn, const Rboolean expand);
-#endif
-#if defined(Win32) && defined(SUPPORT_UTF8_WIN32)
+
+#if defined(SUPPORT_UTF8_WIN32)
 #define mbrtowc(a,b,c,d) Rmbrtowc(a,b)
 #define wcrtomb(a,b,c) Rwcrtomb(a,b)
 #define mbstowcs(a,b,c) Rmbstowcs(a,b,c)
@@ -1167,6 +1227,7 @@ size_t Rwcrtomb(char *s, const wchar_t wc);
 size_t Rmbstowcs(wchar_t *wc, const char *s, size_t n);
 size_t Rwcstombs(char *s, const wchar_t *wc, size_t n);
 #endif
+#endif
 
 FILE *RC_fopen(const SEXP fn, const char *mode, const Rboolean expand);
 int Seql(SEXP a, SEXP b);
@@ -1180,7 +1241,7 @@ double R_atof(const char *str);
 void set_rl_word_breaks(const char *str);
 
 /* From localecharset.c */
-extern char *locale2charset(const char *);
+extern const char *locale2charset(const char *);
 
 /* Localization */
 
@@ -1194,11 +1255,10 @@ extern char *locale2charset(const char *);
 #endif
 #define gettext_noop(String) String
 #define N_(String) gettext_noop (String)
-#define P_(StringS, StringP, N) ngettext (StringS, StringP, N)
 #else /* not NLS */
 #define _(String) (String)
 #define N_(String) String
-#define P_(String, StringP, N) (N > 1 ? StringP: String)
+#define ngettext(String, StringP, N) (N > 1 ? StringP: String)
 #endif
 
 
@@ -1212,8 +1272,6 @@ extern char *locale2charset(const char *);
 } while(0)
 
 
-/* FreeBSD defines alloca in stdlib.h, _and_ does not allow a definition
-   as here.  (Since it uses GCC, it should use the first clause.) */
 #ifdef __GNUC__
 # undef alloca
 # define alloca(x) __builtin_alloca((x))
@@ -1221,18 +1279,32 @@ extern char *locale2charset(const char *);
 # ifdef HAVE_ALLOCA_H
 #  include <alloca.h>
 # endif
-# if !HAVE_DECL_ALLOCA  && !defined(__FreeBSD__)
-extern char *alloca(size_t);
+# if !HAVE_DECL_ALLOCA
+extern void *alloca(size_t);
 # endif
 #endif
 
-/* Or use typedef? */
+/* Required by C99, but might be slow */
 #ifdef HAVE_LONG_DOUBLE
 # define LDOUBLE long double
 #else
 # define LDOUBLE double
 #endif
 
+/* int_fast64_t is required by C99/C11
+   Alternative would be to use intmax_t.
+ */
+#ifdef HAVE_INT64_T
+# define LONG_INT int64_t
+# define LONG_INT_MAX INT64_MAX
+#elif defined(HAVE_INT_FAST64_T)
+# define LONG_INT int_fast64_t
+# define LONG_INT_MAX INT_FAST64_MAX
+#endif
+
+// for reproducibility for now: use exp10 or pown later if accurate enough.
+#define Rexp10(x) pow(10.0, x)
+
 #endif /* DEFN_H_ */
 /*
  *- Local Variables:
