@@ -1,6 +1,8 @@
 #  File src/library/base/R/datetime.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2014 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -16,33 +18,28 @@
 
 Sys.time <- function() .POSIXct(.Internal(Sys.time()))
 
-Sys.timezone <- function() Sys.getenv("TZ", names = FALSE)
+## overridden on Windows
+Sys.timezone <- function(location = TRUE)
+{
+    tz <- Sys.getenv("TZ", names = FALSE)
+    if(!location || nzchar(tz)) return(Sys.getenv("TZ", unset = NA_character_))
+    lt <- normalizePath("/etc/localtime") # Linux, OS X, ...
+    if (grepl(pat <- "^/usr/share/zoneinfo/", lt)) sub(pat, "", lt)
+    else NA_character_
+}
 
 as.POSIXlt <- function(x, tz = "", ...) UseMethod("as.POSIXlt")
 
-as.POSIXlt.Date <- function(x, ...)
-{
-    ## <FIXME>
-    ## Move names handling to C code eventually ...
-    y <- .Internal(Date2POSIXlt(x))
-    names(y$year) <- names(x)
-    y
-    ## </FIXME>
-}
+as.POSIXlt.Date <- function(x, ...) .Internal(Date2POSIXlt(x))
 
 as.POSIXlt.date <- as.POSIXlt.dates <- function(x, ...)
     as.POSIXlt(as.POSIXct(x), ...)
 
 as.POSIXlt.POSIXct <- function(x, tz = "", ...)
 {
-    tzone <- attr(x, "tzone")
-    if((missing(tz) || is.null(tz)) && !is.null(tzone)) tz <- tzone[1L]
-    ## <FIXME>
-    ## Move names handling to C code eventually ...
-    y <- .Internal(as.POSIXlt(x, tz))
-    names(y$year) <- names(x)
-    y
-    ## </FIXME>
+    if((missing(tz) || is.null(tz)) &&
+       !is.null(tzone <- attr(x, "tzone"))) tz <- tzone[1L]
+    .Internal(as.POSIXlt(x, tz))
 }
 
 as.POSIXlt.factor <- function(x, ...)
@@ -56,7 +53,7 @@ as.POSIXlt.character <- function(x, tz = "", format, ...)
 {
     x <- unclass(x) # precaution PR7826
     if(!missing(format)) {
-        res <- strptime(x, format, tz=tz)
+        res <- strptime(x, format, tz = tz)
         if(nzchar(tz)) attr(res, "tzone") <- tz
         return(res)
     }
@@ -65,14 +62,14 @@ as.POSIXlt.character <- function(x, tz = "", format, ...)
         res <- strptime(x, "%Y/%m/%d")
         if(nzchar(tz)) attr(res, "tzone") <- tz
         return(res)
-    } else if(all(!is.na(strptime(xx, f <- "%Y-%m-%d %H:%M:%OS", tz=tz))) ||
-            all(!is.na(strptime(xx, f <- "%Y/%m/%d %H:%M:%OS", tz=tz))) ||
-            all(!is.na(strptime(xx, f <- "%Y-%m-%d %H:%M", tz=tz))) ||
-            all(!is.na(strptime(xx, f <- "%Y/%m/%d %H:%M", tz=tz))) ||
-            all(!is.na(strptime(xx, f <- "%Y-%m-%d", tz=tz))) ||
-            all(!is.na(strptime(xx, f <- "%Y/%m/%d", tz=tz)))
+    } else if(all(!is.na(strptime(xx, f <- "%Y-%m-%d %H:%M:%OS", tz = tz))) ||
+            all(!is.na(strptime(xx, f <- "%Y/%m/%d %H:%M:%OS", tz = tz))) ||
+            all(!is.na(strptime(xx, f <- "%Y-%m-%d %H:%M", tz = tz))) ||
+            all(!is.na(strptime(xx, f <- "%Y/%m/%d %H:%M", tz = tz))) ||
+            all(!is.na(strptime(xx, f <- "%Y-%m-%d", tz = tz))) ||
+            all(!is.na(strptime(xx, f <- "%Y/%m/%d", tz = tz)))
             ) {
-        res <- strptime(x, f, tz=tz)
+        res <- strptime(x, f, tz = tz)
         if(nzchar(tz)) attr(res, "tzone") <- tz
         return(res)
     }
@@ -90,9 +87,11 @@ as.POSIXlt.default <- function(x, tz = "", ...)
 
     if(inherits(x, "POSIXlt")) return(x)
     if(is.logical(x) && all(is.na(x)))
-        return(as.POSIXlt(as.POSIXct.default(x), tz=tz))
-    stop(gettextf("do not know how to convert '%s' to class \"POSIXlt\"",
-                  deparse(substitute(x))))
+        return(as.POSIXlt(as.POSIXct.default(x), tz = tz))
+    stop(gettextf("do not know how to convert '%s' to class %s",
+                  deparse(substitute(x)),
+                  dQuote("POSIXlt")),
+         domain = NA)
 }
 
 as.POSIXct <- function(x, tz = "", ...) UseMethod("as.POSIXct")
@@ -138,7 +137,7 @@ as.POSIXct.POSIXlt <- function(x, tz = "", ...)
 as.POSIXct.numeric <- function(x, tz = "", origin, ...)
 {
     if(missing(origin)) stop("'origin' must be supplied")
-    as.POSIXct(origin, tz=tz, ...) + x
+    .POSIXct(as.POSIXct(origin, tz = "GMT", ...) + x, tz)
 }
 
 as.POSIXct.default <- function(x, tz = "", ...)
@@ -148,8 +147,10 @@ as.POSIXct.default <- function(x, tz = "", ...)
 	return(as.POSIXct(as.POSIXlt(x, tz, ...), tz, ...))
     if(is.logical(x) && all(is.na(x)))
         return(.POSIXct(as.numeric(x)))
-    stop(gettextf("do not know how to convert '%s' to class \"POSIXct\"",
-                  deparse(substitute(x))))
+    stop(gettextf("do not know how to convert '%s' to class %s",
+                  deparse(substitute(x)),
+                  dQuote("POSIXct")),
+         domain = NA)
 }
 
 as.double.POSIXlt <- function(x, ...) as.double(as.POSIXct(x))
@@ -175,7 +176,7 @@ format.POSIXlt <- function(x, format = "", usetz = FALSE, ...)
                 }
         format <- if(all(times[!is.na(times)] == 0)) "%Y-%m-%d"
         else if(np == 0L) "%Y-%m-%d %H:%M:%S"
-        else paste("%Y-%m-%d %H:%M:%OS", np, sep="")
+        else paste0("%Y-%m-%d %H:%M:%OS", np)
     }
     ## <FIXME>
     ## Move names handling to C code eventually ...
@@ -206,7 +207,7 @@ format.POSIXct <- function(x, format = "", tz = "", usetz = FALSE, ...)
     if(!inherits(x, "POSIXct")) stop("wrong class")
     if(missing(tz) && !is.null(tzone <- attr(x, "tzone"))) tz <- tzone
     structure(format.POSIXlt(as.POSIXlt(x, tz), format, usetz, ...),
-              names=names(x))
+              names = names(x))
 }
 
 ## could handle arrays for max.print
@@ -214,10 +215,10 @@ print.POSIXct <- function(x, ...)
 {
     max.print <- getOption("max.print", 9999L)
     if(max.print < length(x)) {
-        print(format(x[seq_len(max.print)], usetz=TRUE), ...)
+        print(format(x[seq_len(max.print)], usetz = TRUE), ...)
         cat(' [ reached getOption("max.print") -- omitted',
             length(x) - max.print, 'entries ]\n')
-    } else print(format(x, usetz=TRUE), ...)
+    } else print(format(x, usetz = TRUE), ...)
     invisible(x)
 }
 
@@ -225,17 +226,22 @@ print.POSIXlt <- function(x, ...)
 {
     max.print <- getOption("max.print", 9999L)
     if(max.print < length(x)) {
-        print(format(x[seq_len(max.print)], usetz=TRUE), ...)
+        print(format(x[seq_len(max.print)], usetz = TRUE), ...)
         cat(' [ reached getOption("max.print") -- omitted',
             length(x) - max.print, 'entries ]\n')
-   } else print(format(x, usetz=TRUE), ...)
+   } else print(format(x, usetz = TRUE), ...)
     invisible(x)
 }
 
-summary.POSIXct <- function(object, digits = 15, ...)
+summary.POSIXct <- function(object, digits = 15L, ...)
 {
-    x <- summary.default(unclass(object), digits=digits, ...)[1L:6L]# no NA's
-    class(x) <- oldClass(object)
+    x <- summary.default(unclass(object), digits = digits, ...)
+    if(m <- match("NA's", names(x), 0)) {
+        NAs <- as.integer(x[m])
+        x <- x[-m]
+        attr(x, "NAs") <- NAs
+    }
+    class(x) <- c("summaryDefault", "table", oldClass(object))
     attr(x, "tzone") <- attr(object, "tzone")
     x
 }
@@ -271,12 +277,12 @@ summary.POSIXlt <- function(object, digits = 15, ...)
                          secs = x, mins = 60*x, hours = 60*60*x,
                          days = 60*60*24*x, weeks = 60*60*24*7*x))
     if(!inherits(e1, "POSIXt"))
-        stop("Can only subtract from POSIXt objects")
+        stop("can only subtract from \"POSIXt\" objects")
     if (nargs() == 1) stop("unary '-' is not defined for \"POSIXt\" objects")
     if(inherits(e2, "POSIXt")) return(difftime(e1, e2))
     if (inherits(e2, "difftime")) e2 <- coerceTimeUnit(e2)
     if(!is.null(attr(e2, "class")))
-        stop("can only subtract numbers from POSIXt objects")
+        stop("can only subtract numbers from \"POSIXt\" objects")
     e1 <- as.POSIXct(e1)
     .POSIXct(unclass(e1) - e2, attr(e1, "tzone"))
 }
@@ -292,7 +298,7 @@ Ops.POSIXt <- function(e1, e2)
         stop(gettextf("'%s' not defined for \"POSIXt\" objects", .Generic),
              domain = NA)
     if(inherits(e1, "POSIXlt") || is.character(e1)) e1 <- as.POSIXct(e1)
-    if(inherits(e2, "POSIXlt") || is.character(e1)) e2 <- as.POSIXct(e2)
+    if(inherits(e2, "POSIXlt") || is.character(e2)) e2 <- as.POSIXct(e2)
     check_tzones(e1, e2)
     NextMethod(.Generic)
 }
@@ -391,35 +397,30 @@ as.list.POSIXct <- function(x, ...)
 }
 
 is.na.POSIXlt <- function(x) is.na(as.POSIXct(x))
+anyNA.POSIXlt <- function(x) anyNA(as.POSIXct(x))
 
 ## <FIXME> check the argument validity
 ## This is documented to remove the timezone
-c.POSIXct <- function(..., recursive=FALSE)
+c.POSIXct <- function(..., recursive = FALSE)
     .POSIXct(c(unlist(lapply(list(...), unclass))))
 
 ## we need conversion to POSIXct as POSIXlt objects can be in different tz.
-c.POSIXlt <- function(..., recursive=FALSE)
+c.POSIXlt <- function(..., recursive = FALSE)
     as.POSIXlt(do.call("c", lapply(list(...), as.POSIXct)))
 
-## force absolute comparisons
-all.equal.POSIXct <- function(target, current, ..., scale=1)
-{
-    check_tzones(target, current)
-    NextMethod("all.equal")
-}
 
 
-ISOdatetime <- function(year, month, day, hour, min, sec, tz="")
+ISOdatetime <- function(year, month, day, hour, min, sec, tz = "")
 {
     if(min(sapply(list(year, month, day, hour, min, sec), length)) == 0L)
-        .POSIXct(numeric(), tz=tz)
+        .POSIXct(numeric(), tz = tz)
     else {
-        x <- paste(year, month, day, hour, min, sec, sep="-")
-        as.POSIXct(strptime(x, "%Y-%m-%d-%H-%M-%OS", tz=tz), tz=tz)
+        x <- paste(year, month, day, hour, min, sec, sep = "-")
+        as.POSIXct(strptime(x, "%Y-%m-%d-%H-%M-%OS", tz = tz), tz = tz)
     }
 }
 
-ISOdate <- function(year, month, day, hour=12, min=0, sec=0, tz="GMT")
+ISOdate <- function(year, month, day, hour = 12, min = 0, sec = 0, tz = "GMT")
     ISOdatetime(year, month, day, hour, min, sec, tz)
 
 as.matrix.POSIXlt <- function(x, ...)
@@ -448,11 +449,12 @@ difftime <-
         time2 <- as.POSIXct(time2, tz = tz)
     }
     z <- unclass(time1) - unclass(time2)
+    attr(z, "tzone") <- NULL # it may get copied from args of `-`
     units <- match.arg(units)
     if(units == "auto") {
         if(all(is.na(z))) units <- "secs"
         else {
-            zz <- min(abs(z),na.rm=TRUE)
+            zz <- min(abs(z),na.rm = TRUE)
             if(is.na(zz) || zz < 60) units <- "secs"
             else if(zz < 3600) units <- "mins"
             else if(zz < 86400) units <- "hours"
@@ -460,29 +462,29 @@ difftime <-
         }
     }
     switch(units,
-           "secs" = .difftime(z, units="secs"),
-           "mins" = .difftime(z/60, units="mins"),
-           "hours"= .difftime(z/3600, units="hours"),
-           "days" = .difftime(z/86400, units="days"),
-           "weeks" = .difftime(z/(7*86400), units="weeks")
+           "secs" = .difftime(z, units = "secs"),
+           "mins" = .difftime(z/60, units = "mins"),
+           "hours" = .difftime(z/3600, units = "hours"),
+           "days" = .difftime(z/86400, units = "days"),
+           "weeks" = .difftime(z/(7*86400), units = "weeks")
            )
 }
 
 ## "difftime" constructor
 ## Martin Maechler, Date: 16 Sep 2002
 ## Numeric input version Peter Dalgaard, December 2006
-as.difftime <- function(tim, format="%X", units="auto")
+as.difftime <- function(tim, format = "%X", units = "auto")
 {
     if (inherits(tim, "difftime")) return(tim)
     if (is.character(tim)){
-        difftime(strptime(tim, format=format),
-             strptime("0:0:0", format="%X"), units=units)
+        difftime(strptime(tim, format = format),
+             strptime("0:0:0", format = "%X"), units = units)
     } else {
         if (!is.numeric(tim)) stop("'tim' is not character or numeric")
-	if (units=="auto") stop("need explicit units for numeric conversion")
+	if (units == "auto") stop("need explicit units for numeric conversion")
         if (!(units %in% c("secs", "mins", "hours", "days", "weeks")))
 	    stop("invalid units specified")
-        structure(tim, units=units, class="difftime")
+        structure(tim, units = units, class = "difftime")
     }
 }
 
@@ -499,12 +501,12 @@ units.difftime <- function(x) attr(x, "units")
     if (from == value) return(x)
     if (!(value %in% c("secs", "mins", "hours", "days", "weeks")))
         stop("invalid units specified")
-    sc <- cumprod(c(secs=1, mins=60, hours=60, days=24, weeks=7))
-    newx <- as.vector(x)*sc[from]/sc[value]
+    sc <- cumprod(c(secs = 1, mins = 60, hours = 60, days = 24, weeks = 7))
+    newx <- unclass(x) * as.vector(sc[from]/sc[value])
     .difftime(newx, value)
 }
 
-as.double.difftime <- function(x, units="auto", ...)
+as.double.difftime <- function(x, units = "auto", ...)
 {
     if (units != "auto") units(x) <- units
     as.vector(x, "double")
@@ -519,13 +521,13 @@ format.difftime <- function(x,...) paste(format(unclass(x),...), units(x))
 print.difftime <- function(x, digits = getOption("digits"), ...)
 {
     if(is.array(x) || length(x) > 1L) {
-        cat("Time differences in ", attr(x, "units"), "\n", sep="")
+        cat("Time differences in ", attr(x, "units"), "\n", sep = "")
         y <- unclass(x); attr(y, "units") <- NULL
         print(y)
     }
     else
-        cat("Time difference of ", format(unclass(x), digits=digits), " ",
-            attr(x, "units"), "\n", sep="")
+        cat("Time difference of ", format(unclass(x), digits = digits), " ",
+            attr(x, "units"), "\n", sep = "")
 
     invisible(x)
 }
@@ -549,7 +551,7 @@ Ops.difftime <- function(e1, e2)
                days = 60*60*24*x, weeks = 60*60*24*7*x)
     }
     if (nargs() == 1) {
-        switch(.Generic, "+"= {}, "-" = {e1[] <- -unclass(e1)},
+        switch(.Generic, "+" = {}, "-" = {e1[] <- -unclass(e1)},
                stop(gettextf("unary '%s' not defined for \"difftime\" objects",
                              .Generic), domain = NA, call. = FALSE)
                )
@@ -573,11 +575,11 @@ Ops.difftime <- function(e1, e2)
                              units = attr(e2, "units"), class = "difftime"))
         u1 <- attr(e1, "units")
         if(attr(e2, "units") == u1) {
-            structure(NextMethod(.Generic), units=u1, class="difftime")
+            structure(NextMethod(.Generic), units=u1, class = "difftime")
         } else {
             e1 <- coerceTimeUnit(e1)
             e2 <- coerceTimeUnit(e2)
-            structure(NextMethod(.Generic), units="secs", class="difftime")
+            structure(NextMethod(.Generic), units = "secs", class = "difftime")
         }
     } else {
         ## '*' is covered by a specific method
@@ -607,9 +609,8 @@ Ops.difftime <- function(e1, e2)
 Math.difftime <- function (x, ...)
 {
     switch(.Generic,
-           'abs'=, 'sign'=,
-           'floor'=, 'ceiling'=, 'trunc'=,
-           'round'=, 'signif'= {
+           "abs" =, "sign" =, "floor" =, "ceiling" =, "trunc" =,
+           "round" =, "signif" = {
                units <- attr(x, "units")
                .difftime(NextMethod(), units)
            },
@@ -658,12 +659,12 @@ seq.POSIXt <-
     function(from, to, by, length.out = NULL, along.with = NULL, ...)
 {
     if (missing(from)) stop("'from' must be specified")
-    if (!inherits(from, "POSIXt")) stop("'from' must be a POSIXt object")
+    if (!inherits(from, "POSIXt")) stop("'from' must be a \"POSIXt\" object")
     cfrom <- as.POSIXct(from)
     if(length(cfrom) != 1L) stop("'from' must be of length 1")
     tz <- attr(cfrom , "tzone")
     if (!missing(to)) {
-        if (!inherits(to, "POSIXt")) stop("'to' must be a POSIXt object")
+        if (!inherits(to, "POSIXt")) stop("'to' must be a \"POSIXt\" object")
         if (length(as.POSIXct(to)) != 1) stop("'to' must be of length 1")
     }
     if (!missing(along.with)) {
@@ -691,12 +692,12 @@ seq.POSIXt <-
         by <- switch(attr(by,"units"), secs = 1, mins = 60, hours = 3600,
                      days = 86400, weeks = 7*86400) * unclass(by)
     } else if(is.character(by)) {
-        by2 <- strsplit(by, " ", fixed=TRUE)[[1L]]
+        by2 <- strsplit(by, " ", fixed = TRUE)[[1L]]
         if(length(by2) > 2L || length(by2) < 1L)
             stop("invalid 'by' string")
         valid <- pmatch(by2[length(by2)],
                         c("secs", "mins", "hours", "days", "weeks",
-                          "months", "years", "DSTdays"))
+                          "months", "years", "DSTdays", "quarters"))
         if(is.na(valid)) stop("invalid string for 'by'")
         if(valid <= 5L) {
             by <- c(1, 60, 3600, 86400, 7*86400)[valid]
@@ -709,14 +710,14 @@ seq.POSIXt <-
     if(valid <= 5L) { # secs, mins, hours, days, weeks
         from <- unclass(as.POSIXct(from))
         if(!is.null(length.out))
-            res <- seq.int(from, by=by, length.out=length.out)
+            res <- seq.int(from, by = by, length.out = length.out)
         else {
             to0 <- unclass(as.POSIXct(to))
             ## defeat test in seq.default
             res <- seq.int(0, to0 - from, by) + from
         }
         return(.POSIXct(res, tz))
-    } else {  # months or years or DSTdays
+    } else {  # months or years or DSTdays or quarters
         r1 <- as.POSIXlt(from)
         if(valid == 7L) { # years
             if(missing(to)) { # years
@@ -726,7 +727,8 @@ seq.POSIXt <-
                 yr <- seq.int(r1$year, to$year, by)
             }
             r1$year <- yr
-        } else if(valid == 6L) { # months
+        } else if(valid %in% c(6L, 9L)) { # months or quarters
+            if (valid == 9L) by <- by * 3
             if(missing(to)) {
                 mon <- seq.int(r1$mon, by = by, length.out = length.out)
             } else {
@@ -762,11 +764,11 @@ cut.POSIXt <-
     x <- as.POSIXct(x)
 
     if (inherits(breaks, "POSIXt")) {
-	breaks <- as.POSIXct(breaks)
+	breaks <- sort(as.POSIXct(breaks))
     } else if(is.numeric(breaks) && length(breaks) == 1L) {
 	## specified number of breaks
     } else if(is.character(breaks) && length(breaks) == 1L) {
-        by2 <- strsplit(breaks, " ", fixed=TRUE)[[1L]]
+        by2 <- strsplit(breaks, " ", fixed = TRUE)[[1L]]
         if(length(by2) > 2L || length(by2) < 1L)
             stop("invalid specification of 'breaks'")
 	valid <-
@@ -774,7 +776,7 @@ cut.POSIXt <-
 		   c("secs", "mins", "hours", "days", "weeks",
 		     "months", "years", "DSTdays", "quarters"))
 	if(is.na(valid)) stop("invalid specification of 'breaks'")
-	start <- as.POSIXlt(min(x, na.rm=TRUE))
+	start <- as.POSIXlt(min(x, na.rm = TRUE))
 	incr <- 1
 	if(valid > 1L) { start$sec <- 0L; incr <- 60 }
 	if(valid > 2L) { start$min <- 0L; incr <- 3600 }
@@ -839,7 +841,7 @@ cut.POSIXt <-
 
 julian <- function(x, ...) UseMethod("julian")
 
-julian.POSIXt <- function(x, origin = as.POSIXct("1970-01-01", tz="GMT"), ...)
+julian.POSIXt <- function(x, origin = as.POSIXct("1970-01-01", tz = "GMT"), ...)
 {
     origin <- as.POSIXct(origin)
     if(length(origin) != 1L) stop("'origin' must be of length one")
@@ -863,10 +865,10 @@ quarters <- function(x, abbreviate) UseMethod("quarters")
 quarters.POSIXt <- function(x, ...)
 {
     x <- (as.POSIXlt(x)$mon)%/%3
-    paste("Q", x+1, sep = "")
+    paste0("Q", x+1)
 }
 
-trunc.POSIXt <- function(x, units=c("secs", "mins", "hours", "days"), ...)
+trunc.POSIXt <- function(x, units = c("secs", "mins", "hours", "days"), ...)
 {
     units <- match.arg(units)
     x <- as.POSIXlt(x)
@@ -874,21 +876,21 @@ trunc.POSIXt <- function(x, units=c("secs", "mins", "hours", "days"), ...)
 	switch(units,
 	       "secs" = {x$sec <- trunc(x$sec)},
 	       "mins" = {x$sec[] <- 0},
-	       "hours"= {x$sec[] <- 0; x$min[] <- 0L},
+	       "hours" = {x$sec[] <- 0; x$min[] <- 0L},
                ## start of day need not be on the same DST.
 	       "days" = {x$sec[] <- 0; x$min[] <- 0L; x$hour[] <- 0L; x$isdst[] <- -1L}
 	       )
     x
 }
 
-round.POSIXt <- function(x, units=c("secs", "mins", "hours", "days"))
+round.POSIXt <- function(x, units = c("secs", "mins", "hours", "days"))
 {
     ## this gets the default from the generic, as that has two args.
     if(is.numeric(units) && units == 0.0) units <-"secs"
     units <- match.arg(units)
     x <- as.POSIXct(x)
     x <- x + switch(units,
-                    "secs" = 0.5, "mins" = 30, "hours"= 1800, "days" = 43200)
+                    "secs" = 0.5, "mins" = 30, "hours" = 1800, "days" = 43200)
     trunc.POSIXt(x, units = units)
 }
 
@@ -940,7 +942,7 @@ diff.POSIXt <- function (x, lag = 1L, differences = 1L, ...)
     ismat <- is.matrix(x)
     r <- if(inherits(x, "POSIXlt")) as.POSIXct(x) else x
     xlen <- if (ismat) dim(x)[1L] else length(r)
-    if (length(lag) > 1L || length(differences) > 1L || lag < 1L || differences < 1L)
+    if (length(lag) != 1L || length(differences) > 1L || lag < 1L || differences < 1L)
         stop("'lag' and 'differences' must be integers >= 1")
     if (lag * differences >= xlen) return(.difftime(numeric(), "secs"))
     i1 <- -seq_len(lag)
@@ -1007,3 +1009,28 @@ function(x, value)
     names(x$year) <- value
     x
 }
+
+## 3.1.0
+
+OlsonNames <- function()
+{
+    if(.Platform$OS.type == "windows")
+        tzdir <- Sys.getenv("TZDIR", file.path(R.home("share"), "zoneinfo"))
+    else {
+        tzdirs <- c(Sys.getenv("TZDIR"),
+                    file.path(R.home("share"), "zoneinfo"),
+                    "/usr/share/zoneinfo", # Linux, OS X, FreeBSD
+                    "/usr/share/lib/zoneinfo", # Solaris, AIX
+                    "/usr/lib/zoneinfo",   # early glibc
+                    "/usr/local/etc/zoneinfo", # tzcode default
+                    "/etc/zoneinfo", "/usr/etc/zoneinfo")
+        tzdirs <- tzdirs[file.exists(tzdirs)]
+        if (!length(tzdirs)) {
+            warning("no Olson database found")
+            return(character())
+        } else tzdir <- tzdirs[1]
+    }
+    x <- list.files(tzdir, recursive = TRUE)
+    ## all auxiliary files are l/case.
+    grep("^[ABCDEFGHIJKLMNOPQRSTUVWXYZ]", x, value = TRUE)
+}
