@@ -1,7 +1,7 @@
 #  File src/library/base/R/namespace.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2013 The R Core Team
+#  Copyright (C) 1995-2014 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -129,10 +129,11 @@ attachNamespace <- function(ns, pos = 2L, depends = NULL)
                               conditionMessage(res)),
                      call. = FALSE, domain = NA)
             }
-        } else if (exists(".First.lib", envir = env, inherits = FALSE) &&
-                   nsname == Sys.getenv("R_INSTALL_PKG"))
-            warning(sprintf("ignoring .First.lib() for package %s",
-                            sQuote(nsname)), domain = NA, call. = FALSE)
+        }
+##         else if (exists(".First.lib", envir = env, inherits = FALSE) &&
+##                  nsname == Sys.getenv("R_INSTALL_PKG"))
+##             warning(sprintf("ignoring .First.lib() for package %s",
+##                             sQuote(nsname)), domain = NA, call. = FALSE)
     }
     runUserHook <- function(pkgname, pkgpath) {
         hook <- getHook(packageEvent(pkgname, "attach")) # might be list()
@@ -655,8 +656,9 @@ requireNamespace <- function (package, ..., quietly = FALSE)
     ns <- .Internal(getRegisteredNamespace(as.name(package)))
     res <- TRUE
     if (is.null(ns)) {
-        packageStartupMessage(gettextf("Loading required namespace: %s",
-                                       package), domain = NA)
+        if(!quietly)
+            packageStartupMessage(gettextf("Loading required namespace: %s",
+                                           package), domain = NA)
         value <- tryCatch(loadNamespace(package, ...), error = function(e) e)
         if (inherits(value, "error")) {
             if (!quietly) {
@@ -773,7 +775,7 @@ namespaceImport <- function(self, ..., from = NULL)
     for (ns in list(...))
         namespaceImportFrom(self, asNamespace(ns), from = from)
 
-namespaceImportFrom <- function(self, ns, vars, generics, packages, from = NULL)
+namespaceImportFrom <- function(self, ns, vars, generics, packages, from = "non-package environment")
 {
     addImports <- function(ns, from, what) {
         imp <- structure(list(what), names = getNamespaceName(from))
@@ -786,7 +788,10 @@ namespaceImportFrom <- function(self, ns, vars, generics, packages, from = NULL)
         old <- as.character(spec)
         new <- names(spec)
         if (is.null(new)) new <- old
-        else new[new == ""] <- old[new == ""]
+        else {
+            change <- !nzchar(new)
+            new[change] <- old[change]
+        }
         names(old) <- new
         old
     }
@@ -796,6 +801,11 @@ namespaceImportFrom <- function(self, ns, vars, generics, packages, from = NULL)
         mm <- ".__T__"
         seq_along(impvars)[substr(impvars, 1L, nchar(mm, type = "c")) == mm]
     }
+    genericPackage <- function(f) {
+        if(methods::is(f, "genericFunction")) f@package
+        else if(is.primitive(f)) "base"
+        else "<unknown>"
+    }
     if (is.character(self))
         self <- getNamespace(self)
     ns <- asNamespace(ns)
@@ -871,10 +881,9 @@ namespaceImportFrom <- function(self, ns, vars, generics, packages, from = NULL)
 	    if (.isMethodsDispatchOn() && methods:::isGeneric(n, ns)) {
 		## warn only if generic overwrites a function which
 		## it was not derived from
-		genNs <- get(n, envir = ns)@package
+		genNs <- genericPackage(get(n, envir = ns))
                 genImp <- get(n, envir = impenv)
-                if(methods::is(genImp, "genericFunction") &&
-                   identical(genNs, genImp@package)) next # same generic
+                if(identical(genNs, genericPackage(genImp))) next # same generic
 		genImpenv <- environmentName(environment(genImp))
                 ## May call environment() on a non-function--an undocumented
                 ## "feature" of environment() is that it returns a special
@@ -1013,7 +1022,10 @@ namespaceExport <- function(ns, vars) {
             old <- as.character(spec)
             new <- names(spec)
             if (is.null(new)) new <- old
-            else new[new == ""] <- old[new == ""]
+            else {
+                change <- !nzchar(new)
+                new[change] <- old[change]
+            }
             names(old) <- new
             old
         }
