@@ -1,6 +1,8 @@
 #  File src/library/base/R/octhex.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -18,13 +20,13 @@ format.octmode <- function(x, width = NULL, ...)
 {
     isna <- is.na(x)
     y <- as.integer(x[!isna])
-    fmt <- if(!is.null(width)) paste("%0", width, "o", sep = "") else "%o"
+    fmt <- if(!is.null(width)) paste0("%0", width, "o") else "%o"
     ans <- rep.int(NA_character_, length(x))
     ans0 <- sprintf(fmt, y)
     if(is.null(width) && length(y) > 1L) {
         ## previous version padded with zeroes to a common field width
         nc <- max(nchar(ans0))
-        ans0 <- sprintf(paste("%0", nc, "o", sep = ""), y)
+        ans0 <- sprintf(paste0("%0", nc, "o"), y)
     }
     ans[!isna] <- ans0
     dim(ans) <- dim(x)
@@ -41,7 +43,7 @@ print.octmode <- function(x, ...)
     invisible(x)
 }
 
-"[.octmode" <- function (x, i)
+`[.octmode` <- function (x, i)
 {
     cl <- oldClass(x)
     y <- NextMethod("[")
@@ -58,7 +60,7 @@ as.octmode <- function(x)
         z <- strtoi(x, 8L)
         if(!any(is.na(z) | z < 0)) return(structure(z, class="octmode"))
     }
-    stop("'x' cannot be coerced to 'octmode'")
+    stop("'x' cannot be coerced to class \"octmode\"")
 }
 
 ## BioC packages cellHTS2 and flowCore misuse this for doubles,
@@ -68,13 +70,13 @@ format.hexmode <- function(x, width = NULL, upper.case = FALSE, ...)
     isna <- is.na(x)
     y <- as.integer(x[!isna])
     fmt0 <- if(upper.case) "X" else "x"
-    fmt <- if(!is.null(width)) paste("%0", width, fmt0, sep = "") else paste("%", fmt0, sep = "")
+    fmt <- if(!is.null(width)) paste0("%0", width, fmt0) else paste0("%", fmt0)
     ans <- rep.int(NA_character_, length(x))
     ans0 <- sprintf(fmt, y)
     if(is.null(width) && length(y) > 1L) {
         ## previous version padded with zeroes to a common field width
         nc <- max(nchar(ans0))
-        ans0 <- sprintf(paste("%0", nc, fmt0, sep = ""), y)
+        ans0 <- sprintf(paste0("%0", nc, fmt0), y)
     }
     ans[!isna] <- ans0
     dim(ans) <- dim(x)
@@ -91,7 +93,7 @@ print.hexmode <- function(x, ...)
     invisible(x)
 }
 
-"[.hexmode" <- function (x, i)
+`[.hexmode` <- function (x, i)
 {
     cl <- oldClass(x)
     y <- NextMethod("[")
@@ -108,20 +110,18 @@ as.hexmode <- function(x)
         z <- strtoi(x, 16L)
         if(!any(is.na(z) | z < 0)) return(structure(z, class = "hexmode"))
     }
-    stop("'x' cannot be coerced to hexmode")
+    stop("'x' cannot be coerced to class \"hexmode\"")
 }
 
 
-`&.octmode` <- function(a, b)
-    as.octmode(.Call("bitwiseAnd", as.octmode(a), as.octmode(b), PACKAGE = "base"))
-`|.octmode` <- function(a, b)
-    as.octmode(.Call("bitwiseOr", as.octmode(a), as.octmode(b), PACKAGE = "base"))
-xor.octmode <- function(a, b)
-    as.octmode(.Call("bitwiseXor", as.octmode(a), as.octmode(b), PACKAGE = "base"))
+`!.octmode` <- function(a) as.octmode(bitwNot(as.octmode(a)))
 
-`&.hexmode` <- function(a, b)
-    as.hexmode(.Call("bitwiseAnd", as.hexmode(a), as.hexmode(b), PACKAGE = "base"))
-`|.hexmode` <- function(a, b)
-    as.hexmode(.Call("bitwiseOr", as.hexmode(a), as.hexmode(b), PACKAGE = "base"))
-xor.hexmode <- function(a, b)
-    as.hexmode(.Call("bitwiseXor", as.hexmode(a), as.hexmode(b), PACKAGE = "base"))
+`&.octmode` <- function(a, b) as.octmode(bitwAnd(as.octmode(a), as.octmode(b)))
+`|.octmode` <- function(a, b) as.octmode(bitwOr(as.octmode(a), as.octmode(b)))
+xor.octmode <- function(a, b) as.octmode(bitwXor(as.octmode(a), as.octmode(b)))
+
+`!.hexmode` <- function(a) as.hexmode(bitwNot(as.hexmode(a)))
+
+`&.hexmode` <- function(a, b) as.hexmode(bitwAnd(as.hexmode(a), as.hexmode(b)))
+`|.hexmode` <- function(a, b) as.hexmode(bitwOr(as.hexmode(a), as.hexmode(b)))
+xor.hexmode <- function(a, b) as.hexmode(bitwXor(as.hexmode(a), as.hexmode(b)))
