@@ -1,7 +1,7 @@
 #  File src/library/base/R/rowsum.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@ rowsum.default <- function(x, group, reorder = TRUE, na.rm = FALSE, ...)
 {
     if (!is.numeric(x)) stop("'x' must be numeric")
     if (length(group) != NROW(x)) stop("incorrect length for 'group'")
-    if (any(is.na(group))) warning("missing values for 'group'")
+    if (anyNA(group)) warning("missing values for 'group'")
     ugroup <- unique(group)
     if (reorder) ugroup <- sort(ugroup, na.last = TRUE, method = "quick")
     ## ugroup can be either a vector or a factor, so do as.character here
@@ -33,7 +33,7 @@ rowsum.data.frame <- function(x, group, reorder = TRUE, na.rm = FALSE, ...)
 {
     if (!is.data.frame(x)) stop("not a data frame") ## make MM happy
     if (length(group) != NROW(x)) stop("incorrect length for 'group'")
-    if (any(is.na(group))) warning("missing values for 'group'")
+    if (anyNA(group)) warning("missing values for 'group'")
     ugroup <- unique(group)
     if (reorder) ugroup <- sort(ugroup, na.last = TRUE, method = "quick")
     .Internal(rowsum_df(x, group, ugroup, na.rm, as.character(ugroup)))
