@@ -18,8 +18,6 @@
 
 svd <- function(x, nu = min(n,p), nv = min(n,p), LINPACK = FALSE)
 {
-    if(LINPACK)
-        warning("LINPACK = TRUE is defunct and will be ignored", domain = NA)
     x <- as.matrix(x)
     if (any(!is.finite(x))) stop("infinite or missing values in 'x'")
     dx <- dim(x)
