@@ -1,7 +1,7 @@
 #  File src/library/grDevices/R/postscript.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -260,7 +260,8 @@ postscript <- function(file = ifelse(onefile, "Rplots.ps", "Rplot%03d.ps"),
     }
 
     onefile <- old$onefile # for 'file'
-    if(!checkIntFormat(file)) stop("invalid 'file'")
+    if(!checkIntFormat(file))
+        stop(gettextf("invalid 'file' argument '%s'", file), domain = NA)
     .External(C_PostScript,
               file, old$paper, old$family, old$encoding, old$bg, old$fg,
               old$width, old$height, old$horizontal, old$pointsize,
@@ -282,7 +283,8 @@ xfig <- function (file = ifelse(onefile,"Rplots.fig", "Rplot%03d.fig"),
     ## do initialization if needed
     initPSandPDFfonts()
 
-    if(!checkIntFormat(file)) stop("invalid 'file'")
+    if(!checkIntFormat(file))
+        stop(gettextf("invalid 'file' argument '%s'", file), domain = NA)
     .External(C_XFig, file, paper, family, bg, fg,
               width, height, horizontal, pointsize,
               onefile, pagecentre, defaultfont, textspecial, encoding)
@@ -360,7 +362,8 @@ pdf <- function(file = ifelse(onefile, "Rplots.pdf", "Rplot%03d.pdf"),
         stop("invalid PDF version")
 
     onefile <- old$onefile # needed to set 'file'
-    if(!checkIntFormat(file)) stop("invalid 'file'")
+    if(!checkIntFormat(file))
+        stop(gettextf("invalid 'file' argument '%s'", file), domain = NA)
     .External(C_PDF,
               file, old$paper, old$family, old$encoding, old$bg, old$fg,
               old$width, old$height, old$pointsize, onefile, old$pagecentre,
@@ -769,7 +772,12 @@ postscriptFonts(# Default Serif font is Times
                 URWTimes = Type1Font("URWTimes",
                   c("n021003l.afm", "n021004l.afm",
                     "n021023l.afm", "n021024l.afm",
-                    "s050000l.afm"))
+                    "s050000l.afm")),
+                ## And Monotype Arial
+                ArialMT = Type1Font("ArialMT",
+                  c("ArialMT.afm", "ArialMT-Bold.afm",
+                    "ArialMT-Italic.afm", "ArialMT-BoldItalic.afm",
+                    "Symbol.afm"))
                 )
 
 ## All of the above Type1 fonts are the same for PostScript and PDF
@@ -943,36 +951,23 @@ pdfFonts(Japan1 = CIDFont("KozMinPro-Regular-Acro", "EUC-H", "EUC-JP",
 embedFonts <- function(file, # The ps or pdf file to convert
                        format, # Default guessed from file suffix
                        outfile = file, # By default overwrite file
-                       fontpaths = "",
+                       fontpaths = character(),
                        options = "" # Additional options to ghostscript
                        )
 {
     if(!is.character(file) || length(file) != 1L || !nzchar(file))
         stop("'file' must be a non-empty character string")
+    gsexe <- tools::find_gs_cmd()
+    if(!nzchar(gsexe)) stop("GhostScript was not found")
+    if(.Platform$OS.type == "windows") gsexe <- shortPathName(gsexe)
     suffix <- gsub(".+[.]", "", file)
-    if (missing(format)) {
+    if (missing(format))
         format <- switch(suffix,
                          ps = ,
                          eps = "pswrite",
                          pdf = "pdfwrite")
-    }
-    if (!is.character(format)) {
+    if (!is.character(format))
         stop("Invalid output format")
-    }
-    gsexe <- Sys.getenv("R_GSCMD")
-    if(.Platform$OS.type == "windows" && !nzchar(gsexe))
-        gsexe <- Sys.getenv("GSC")
-    if(is.null(gsexe) || !nzchar(gsexe)) {
-        gsexe <- switch(.Platform$OS.type,
-                        unix = "gs",
-                        windows = {
-                            poss <- Sys.which(c("gswin64c.exe", "gswin32c.exe"))
-                            poss <- poss[nzchar(poss)]
-                            if(length(poss)) poss else "gswin32c.exe"
-                        })
-    } else if(.Platform$OS.type == "windows" &&
-              length(grep(" ", gsexe, fixed=TRUE)))
-        gsexe <- shortPathName(gsexe)
     tmpfile <- tempfile("Rembed")
     if (length(fontpaths))
         fontpaths <-
