@@ -1,7 +1,7 @@
 #  File src/library/grDevices/R/windows/windows.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2014 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -64,6 +64,13 @@ windows <-
              buffered, title, restoreConsole, clickToConfirm,
              fillOddEven, family = "", antialias)
 {
+    check <- Sys.getenv("_R_CHECK_SCREEN_DEVICE_", "")
+    msg <- "screen devices should not be used in examples etc"
+    if (identical(check, "stop"))
+        stop(msg, domain = NA)
+    else if (identical(check, "warn"))
+        warning(msg, immediate. = TRUE, noBreaks. = TRUE, domain = NA)
+
     new <- list()
     if(!missing(width)) new$width <- as.double(width)
     if(!missing(height)) new$height <- as.double(height)
@@ -89,7 +96,7 @@ windows <-
     rescale <- match(d$rescale, c("R", "fit", "fixed"))
     if(is.na(rescale)) stop("invalid value for 'rescale'")
     antialias <- pmatch(d$antialias, aa.win)
-    invisible(.External(Cdevga, "", d$width, d$height, d$pointsize,
+    invisible(.External(C_devga, "", d$width, d$height, d$pointsize,
                         d$record, rescale, d$xpinch, d$ypinch,
                         d$canvas, d$gamma, d$xpos, d$ypos,
                         d$buffered, .PSenv, d$bg,
@@ -97,29 +104,26 @@ windows <-
                         d$fillOddEven, family, antialias))
 }
 
+## historical wrappers
 win.graph <- function(width, height, pointsize)
-{
-    new <- list()
-    if(!missing(width)) new$width <- as.double(width)
-    if(!missing(height)) new$height <- as.double(height)
-    if(!missing(pointsize)) new$pointsize <- as.double(pointsize)
-    d <- check.options(new = new, envir = .WindowsEnv,
-                       name.opt = ".Windows.Options",
-                       reset = FALSE, assign.opt = FALSE)
-    invisible(.External(Cdevga, "", d$width, d$height, d$pointsize,
-                        FALSE, 1L, d$xpinch, d$ypinch, "white",
-                        d$gamma, NA_integer_, NA_integer_, d$buffered,
-                        .PSenv, NA, d$restoreConsole, "", TRUE,
-                        d$fillOddEven, "",
-                        pmatch(d$antialias, aa.win)))
-}
+    windows(width = width, height = height, pointsize = pointsize)
+
+x11 <- X11 <-
+     function(width, height, pointsize, bg, gamma, xpos, ypos, title)
+    windows(width = width, height = height, pointsize = pointsize,
+            bg = bg, gamma = gamma,
+            xpos = xpos, ypos = ypos, title = title)
 
 win.print <-
     function(width = 7, height = 7, pointsize = 12, printer = "",
              family = "", antialias = "default", restoreConsole = TRUE)
 {
+    check <- Sys.getenv("_R_CHECK_WINDOWS_DEVICE_", "")
+    if (identical(check, "stop"))
+        stop("windows devices should not be used in examples etc", domain = NA)
+
     antialias <- match(match.arg(antialias, aa.win), aa.win)
-    invisible(.External(Cdevga, paste("win.print:", printer, sep=""),
+    invisible(.External(C_devga, paste("win.print:", printer, sep=""),
                         width, height, pointsize, FALSE, 1L,
                         NA_real_, NA_real_, "white", 1,
                         NA_integer_, NA_integer_,
@@ -131,9 +135,13 @@ win.metafile <-
     function(filename = "", width = 7, height = 7, pointsize = 12,
              family = "", restoreConsole = TRUE)
 {
+    check <- Sys.getenv("_R_CHECK_WINDOWS_DEVICE_", "")
+    if (identical(check, "stop"))
+        stop("windows devices should not be used in examples etc", domain = NA)
+
     if(!checkIntFormat(filename)) stop("invalid 'filename'")
     filename <- path.expand(filename)
-    invisible(.External(Cdevga, paste("win.metafile:", filename, sep=""),
+    invisible(.External(C_devga, paste("win.metafile:", filename, sep=""),
                         width, height, pointsize, FALSE, 1L,
                         NA_real_, NA_real_, "white", 1,
                         NA_integer_, NA_integer_, FALSE, .PSenv, NA,
@@ -147,7 +155,8 @@ bringToTop <- function(which = dev.cur(), stay = FALSE)
     }
     if(which > 0 && .Devices[[which]] != "windows")
         stop("can only bring windows devices to the front")
-    invisible(.Internal(bringToTop(as.integer(which), as.logical(stay))))
+    .Call(C_bringToTop, as.integer(which), as.logical(stay))
+    invisible()
 }
 
 msgWindow <-
@@ -161,7 +170,8 @@ msgWindow <-
     if(!exists(".Devices")) .Devices <- list("null device")
     if(which > 0 && .Devices[[which]] != "windows")
         stop("can only manipulate windows devices")
-    invisible(.Internal(msgWindow(as.integer(which), as.integer(itype))))
+    .Call(C_msgWindow, as.integer(which), as.integer(itype))
+    invisible()
 }
 
 savePlot <- function(filename = "Rplot",
@@ -180,7 +190,7 @@ savePlot <- function(filename = "Rplot",
     else if(regexpr("\\.",filename) < 0)
         filename <- paste(filename, type, sep = ".")
     filename <- path.expand(filename)
-    invisible(.External(CsavePlot, device, filename, type, restoreConsole))
+    invisible(.External(C_savePlot, device, filename, type, restoreConsole))
 }
 
 print.SavedPlots <- function(x, ...)
