@@ -1,6 +1,8 @@
 #  File src/library/graphics/R/arrows.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -20,6 +22,8 @@ arrows <-
 {
     if (missing(x1) && missing(y1))
         stop("one of 'x1' and 'y1' must be given")
-    .Internal(arrows(x0, y0, x1, y1, length=length, angle=angle, code=code,
-                     col=col, lty=lty, lwd=lwd, ...))
+    .External.graphics(C_arrows, x0, y0, x1, y1, length = length,
+                       angle = angle, code = code, col = col,
+                       lty = lty, lwd = lwd, ...)
+    invisible()
 }
