@@ -17,16 +17,16 @@
 #  http://www.r-project.org/Licenses/
 
 xinch <- function(x=1, warn.log=TRUE) {
-    if(warn.log && par("xlog")) warning("x log scale:  xinch() is non-sense")
+    if(warn.log && par("xlog")) warning("x log scale:  xinch() is nonsense")
     x * diff(par("usr")[1L:2])/par("pin")[1L]
 }
 yinch <- function(y=1, warn.log=TRUE) {
-    if(warn.log && par("ylog")) warning("y log scale:  yinch() is non-sense")
+    if(warn.log && par("ylog")) warning("y log scale:  yinch() is nonsense")
     y * diff(par("usr")[3:4])/par("pin")[2L]
 }
 
 xyinch <- function(xy=1, warn.log=TRUE) {
     if(warn.log && (par("xlog") || par("ylog")))
-	warning("log scale:  xyinch() is non-sense")
+	warning("log scale:  xyinch() is nonsense")
     u <- par("usr"); xy * c(u[2L]-u[1L], u[4L]-u[3L]) / par("pin")
 }
