@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2012  The R Core Team
+ *  Copyright (C) 1997--2014  The R Core Team
  *  Copyright (C) 2002--2011  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,14 @@
 #include <GraphicsBase.h>       /* setBaseDevice */
 #include <Rmath.h>		/* eg. fmax2() */
 
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#undef _
+#define _(String) dgettext ("grDevices", String)
+#else
+#define _(String) (String)
+#endif
+
 /*--->> Documentation now in  ../include/Rgraphics.h  "API" ----- */
 
 double R_Log10(double x)
@@ -639,7 +647,7 @@ double yDevtoNPC(double y, pGEDevDesc dd)
 double xNPCtoUsr(double x, pGEDevDesc dd)
 {
     if (gpptr(dd)->xlog)
-	return pow(10., gpptr(dd)->logusr[0] +
+	return Rexp10(gpptr(dd)->logusr[0] +
 		   x*(gpptr(dd)->logusr[1] - gpptr(dd)->logusr[0]));
     else
 	return gpptr(dd)->usr[0] + x*(gpptr(dd)->usr[1] - gpptr(dd)->usr[0]);
@@ -648,7 +656,7 @@ double xNPCtoUsr(double x, pGEDevDesc dd)
 double yNPCtoUsr(double y, pGEDevDesc dd)
 {
     if (gpptr(dd)->ylog)
-	return pow(10., gpptr(dd)->logusr[2] +
+	return Rexp10(gpptr(dd)->logusr[2] +
 		   y*(gpptr(dd)->logusr[3]-gpptr(dd)->logusr[2]));
     else
 	return gpptr(dd)->usr[2] + y*(gpptr(dd)->usr[3] - gpptr(dd)->usr[2]);
@@ -658,7 +666,7 @@ double xDevtoUsr(double x, pGEDevDesc dd)
 {
     double nfc = xDevtoNFC(x, dd);
     if (gpptr(dd)->xlog)
-	return pow(10., (nfc - gpptr(dd)->win2fig.ax)/gpptr(dd)->win2fig.bx);
+	return Rexp10((nfc - gpptr(dd)->win2fig.ax)/gpptr(dd)->win2fig.bx);
     else
 	return (nfc - gpptr(dd)->win2fig.ax)/gpptr(dd)->win2fig.bx;
 }
@@ -667,7 +675,7 @@ double yDevtoUsr(double y, pGEDevDesc dd)
 {
   double nfc = yDevtoNFC(y, dd);
   if (gpptr(dd)->ylog)
-    return pow(10., (nfc - gpptr(dd)->win2fig.ay)/gpptr(dd)->win2fig.by);
+    return Rexp10((nfc - gpptr(dd)->win2fig.ay)/gpptr(dd)->win2fig.by);
   else
     return (nfc - gpptr(dd)->win2fig.ay)/gpptr(dd)->win2fig.by;
 }
@@ -1875,8 +1883,8 @@ void GAxisPars(double *min, double *max, int *n, Rboolean log, int axis)
 	/* Avoid infinities */
 	if(*max > 308) *max = 308;
 	if(*min < -307) *min = -307;
-	*min = pow(10., *min);
-	*max = pow(10., *max);
+	*min = Rexp10(*min);
+	*max = Rexp10(*max);
 	GLPretty(min, max, n);
     }
     else GEPretty(min, max, n);
@@ -1896,8 +1904,8 @@ void GAxisPars(double *min, double *max, int *n, Rboolean log, int axis)
 	*min += eps;
 	*max -= eps;
 	if(log) {
-	    *min = pow(10., *min);
-	    *max = pow(10., *max);
+	    *min = Rexp10(*min);
+	    *max = Rexp10(*max);
 	}
 	*n = 1;
     }
@@ -1971,14 +1979,14 @@ void GScale(double min, double max, int axis, pGEDevDesc dd)
     }
 
     if (log) { /* 10^max may have gotten +Inf ; or  10^min has become 0 */
-	if((temp = pow(10., min)) == 0.) {/* or < 1.01*DBL_MIN */
+	if((temp = Rexp10(min)) == 0.) {/* or < 1.01*DBL_MIN */
 	    temp = fmin2(min_o, 1.01* DBL_MIN); /* allow smaller non 0 */
 	    min = log10(temp);
 	}
 	if(max >= 308.25) { /* overflows */
 	    tmp2 = fmax2(max_o, .99 * DBL_MAX);
 	    max = log10(tmp2);
-	} else tmp2 = pow(10., max);
+	} else tmp2 = Rexp10(max);
     }
     if(is_xaxis) {
 	if (log) {
@@ -3184,8 +3192,8 @@ void GLPretty(double *ul, double *uh, int *n)
     }
     else { /* extra tickmarks --> CreateAtVector() in ./plot.c */
 	/* round to nice "1e<N>" */
-	*ul = pow(10., (double)p1);
-	*uh = pow(10., (double)p2);
+	*ul = Rexp10((double)p1);
+	*uh = Rexp10((double)p2);
 	if (p2 - p1 <= LPR_SMALL)
 	    *n = 3; /* Small range :	Use 1,2,5,10 times 10^k tickmarks */
 	else if (p2 - p1 <= LPR_MEDIUM)
