@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2013  The R Core Team.
+ *  Copyright (C) 1997--2014  The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -584,8 +584,8 @@ static void Specify(const char *what, SEXP value, pGEDevDesc dd)
 	if (gpptr(dd)->xlog) {
 	    R_DEV_2(logusr[0]) = REAL(value)[0];
 	    R_DEV_2(logusr[1]) = REAL(value)[1];
-	    R_DEV_2(usr[0]) = pow(10., REAL(value)[0]);
-	    R_DEV_2(usr[1]) = pow(10., REAL(value)[1]);
+	    R_DEV_2(usr[0]) = Rexp10(REAL(value)[0]);
+	    R_DEV_2(usr[1]) = Rexp10(REAL(value)[1]);
 	}
 	else {
 	    R_DEV_2(usr[0]) = REAL(value)[0];
@@ -596,8 +596,8 @@ static void Specify(const char *what, SEXP value, pGEDevDesc dd)
 	if (gpptr(dd)->ylog) {
 	    R_DEV_2(logusr[2]) = REAL(value)[2];
 	    R_DEV_2(logusr[3]) = REAL(value)[3];
-	    R_DEV_2(usr[2]) = pow(10., REAL(value)[2]);
-	    R_DEV_2(usr[3]) = pow(10., REAL(value)[3]);
+	    R_DEV_2(usr[2]) = Rexp10(REAL(value)[2]);
+	    R_DEV_2(usr[3]) = Rexp10(REAL(value)[3]);
 	}
 	else {
 	    R_DEV_2(usr[2]) = REAL(value)[2];
