@@ -1,7 +1,7 @@
 #  File src/library/grid/R/grid.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -32,22 +32,24 @@ push.vp.viewport <- function(vp, recording) {
   # Record on the display list
   if (recording)
     record(vp)
-  # Create a pushedvp object for the system to keep track of
-  pvp <- pushedvp(vp)
   # Store the entire set of gpar settings JUST PRIOR to push
   # We refer to this when calculating the viewport transform
   # We cannot simply rely on parent's gpar because we may be
   # being pushed from within a gTree which has enforced gpar
   # settings (i.e., the gTree$gp is enforced between this viewport
   # and the this viewport's parent$gp)
-  pvp$parentgpar <- grid.Call(L_getGPar)
+  vp$parentgpar <- grid.Call(L_getGPar)
   # Enforce gpar settings
   set.gpar(vp$gp)
   # Store the entire set of gpar settings for this viewport
-  pvp$gpar <- grid.Call(L_getGPar)
+  vp$gpar <- grid.Call(L_getGPar)
   # Pass in the pushedvp structure which will be used to store
   # things like the viewport transformation, parent-child links, ...
-  grid.Call.graphics(L_setviewport, pvp, TRUE)
+  # In C code, a pushedvp object is created, with a call to pushedvp(),
+  # for the system to keep track of
+  # (it happens in C code so that a "normal" vp gets recorded on the
+  #  display list rather than a "pushedvp")
+  grid.Call.graphics(L_setviewport, vp, TRUE)
 }
 
 # For all but the last viewport, push the
@@ -88,8 +90,7 @@ push.vp.vpPath <- function(vp, recording) {
 }
 
 push.viewport <- function(..., recording=TRUE) {
-  .Deprecated("pushViewport")
-  pushViewport(..., recording=recording)
+    .Defunct("pushViewport")
 }
 
 pushViewport <- function(..., recording=TRUE) {
@@ -138,10 +139,28 @@ downViewport <- function(name, strict=FALSE, recording=TRUE) {
 # vpPath directly (i.e., w/o calling vpPath)
 downViewport.default <- function(name, strict=FALSE, recording=TRUE) {
   name <- as.character(name)
-  downViewport(vpPathDirect(name), strict, recording=recording)
+  downViewport(vpPath(name), strict, recording=recording)
+}
+
+# Build vpPath from one (pushed) viewport up to another (pushed) viewport
+# 'anc' is assumed to be an ancestor of 'desc'
+# 'depth' is the depth that the final depth should have
+buildPath <- function(desc, anc, depth) {
+    path <- desc$name
+    while (!identical(desc$parent, anc)) {
+        if (is.null(desc$parent))
+            stop("Down viewport failed to record on display list")
+        desc <- desc$parent
+        path <- c(desc$name, path)
+    }
+    result <- vpPath(path)
+    if (depth(result) != depth)
+        warning("Down viewport incorrectly recorded on display list")
+    result
 }
 
 downViewport.vpPath <- function(name, strict=FALSE, recording=TRUE) {
+    start <- grid.Call(L_currentViewport)
     if (name$n == 1)
         result <- grid.Call.graphics(L_downviewport, name$name, strict)
     else
@@ -158,7 +177,9 @@ downViewport.vpPath <- function(name, strict=FALSE, recording=TRUE) {
     # ... including the depth navigated down
     if (recording) {
         attr(name, "depth") <- result
-        record(name)
+        # Record the strict path down
+        path <- buildPath(pvp, start, result)
+        record(path)
     }
     invisible(result)
 }
@@ -184,8 +205,7 @@ vpDepth <- function() {
 }
 
 pop.viewport <- function(n=1, recording=TRUE) {
-  .Deprecated("popViewport")
-  popViewport(n, recording=recording)
+    .Defunct("popViewport")
 }
 
 popViewport <- function(n=1, recording=TRUE) {
@@ -241,15 +261,29 @@ current.vpPath <- function() {
 }
 
 # Function to obtain the current viewport
-current.viewport <- function(vp=NULL) {
-  if (is.null(vp))
+current.viewport <- function() {
     # The system stores a pushedvp;  the user should only
     # ever see normal viewports, so convert.
     vpFromPushedvp(grid.Call(L_currentViewport))
-  else {
-    warning("the 'vp' argument is deprecated")
-    vp
-  }
+}
+
+# Return the parent of the current viewport
+# (could be NULL)
+current.parent <- function(n=1) {
+    if (n < 1)
+        stop("Invalid number of generations")
+    vp <- grid.Call(L_currentViewport)
+    generation <- 1
+    while (generation <= n) {
+        if (is.null(vp))
+            stop("Invalid number of generations")
+        vp <- vp$parent
+        generation <- generation + 1
+    }
+    if (!is.null(vp))
+        vpFromPushedvp(vp)
+    else
+        vp
 }
 
 vpListFromNode <- function(node) {
@@ -293,7 +327,11 @@ current.vpTree <- function(all=TRUE) {
 }
 
 current.transform <- function() {
-  grid.Call(L_currentViewport)$trans
+    grid.Call(L_currentViewport)$trans
+}
+
+current.rotation <- function() {
+    grid.Call(L_currentViewport)$rotation
 }
 
 # Call this function if you want the graphics device erased or moved
@@ -440,7 +478,7 @@ grid.DLapply <- function(FUN, ...) {
 # you are doing -- this will be a bit of overkill, but is for safety
 grid.Call <- function(fnname, ...) {
   .Call(L_gridDirty)
-  .Call(fnname, ...)
+  .Call(dontCheck(fnname), ...)  # skip code analysis checks, keep runtime checks
 }
 
 grid.Call.graphics <- function(fnname, ...) {
@@ -453,10 +491,10 @@ grid.Call.graphics <- function(fnname, ...) {
     # operation;  this is necessary in case the display list is
     # played on another device (e.g., via replayPlot() or dev.copy())
     .Call.graphics(L_gridDirty)
-    result <- .Call.graphics(fnname, ...)
+    result <- .Call.graphics(dontCheck(fnname), ...)
   } else {
     .Call(L_gridDirty)
-    result <- .Call(fnname, ...)
+    result <- .Call(dontCheck(fnname), ...)
   }
   result
 }
