@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 2001-3 Paul Murrell
- *                2003-5 The R Development Core Team
+ *                2003-5 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,13 +13,13 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  A copy of the GNU General Public License is available via WWW at
- *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
- *  writing to the Free Software Foundation, Inc., 59 Temple Place,
- *  Suite 330, Boston, MA  02111-1307  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #include "grid.h"
+#include <string.h>
 
 extern int gridRegisterIndex;
 
@@ -88,7 +88,7 @@ SEXP viewportgpar(SEXP vp) {
     return VECTOR_ELT(vp, PVP_GPAR);
 }
 
-char* viewportFontFamily(SEXP vp) {
+const char* viewportFontFamily(SEXP vp) {
     return CHAR(STRING_ELT(VECTOR_ELT(VECTOR_ELT(vp, PVP_GPAR), GP_FONTFAMILY),
 			   0));
 }
@@ -184,8 +184,8 @@ void copyViewportContext(LViewportContext vpc1, LViewportContext *vpc2)
     vpc2->yscalemax = vpc1.yscalemax;
 }
 
-void gcontextFromViewport(SEXP vp, R_GE_gcontext *gc) {
-    gcontextFromgpar(viewportgpar(vp), 0, gc);
+void gcontextFromViewport(SEXP vp, const pGEcontext gc, pGEDevDesc dd) {
+    gcontextFromgpar(viewportgpar(vp), 0, gc, dd);
 }
 
 /* The idea is to produce a transformation for this viewport which
@@ -198,7 +198,7 @@ void gcontextFromViewport(SEXP vp, R_GE_gcontext *gc) {
  * everything from scratch.
  */
 void calcViewportTransform(SEXP vp, SEXP parent, Rboolean incremental,
-			   GEDevDesc *dd)
+			   pGEDevDesc dd)
 {
     int i, j;
     double vpWidthCM, vpHeightCM, rotationAngle;
@@ -268,7 +268,7 @@ void calcViewportTransform(SEXP vp, SEXP parent, Rboolean incremental,
 	 * 
 	 * WAS gcontextFromViewport(parent, &parentgc);
 	 */
-	gcontextFromgpar(viewportParentGPar(vp), 0, &parentgc);
+	gcontextFromgpar(viewportParentGPar(vp), 0, &parentgc, dd);
 	/* In order for the vp to get its vpl from a layout
 	 * it must have specified a layout.pos and the parent
 	 * must have a layout
@@ -314,7 +314,7 @@ void calcViewportTransform(SEXP vp, SEXP parent, Rboolean incremental,
 	!R_FINITE(yINCHES) || 
 	!R_FINITE(vpWidthCM) || 
 	!R_FINITE(vpHeightCM))
-	error(_("Non-finite location and/or size for viewport"));
+	error(_("non-finite location and/or size for viewport"));
     /* Determine justification required
      */
     justification(vpWidthCM, vpHeightCM, vpl.hjust, vpl.vjust,
@@ -347,18 +347,15 @@ void calcViewportTransform(SEXP vp, SEXP parent, Rboolean incremental,
      */
     if (!isNull(viewportLayout(vp))) {
 	fillViewportContextFromViewport(vp, &vpc);
-	gcontextFromViewport(vp, &gc);
+	gcontextFromViewport(vp, &gc, dd);
 	calcViewportLayout(vp, vpWidthCM, vpHeightCM, vpc, &gc, dd);
     }
     /* Record all of the answers in the viewport
      * (the layout calculations are done within calcViewportLayout)
      */
-    PROTECT(currentWidthCM = allocVector(REALSXP, 1));
-    REAL(currentWidthCM)[0] = vpWidthCM;
-    PROTECT(currentHeightCM = allocVector(REALSXP, 1));
-    REAL(currentHeightCM)[0] = vpHeightCM;
-    PROTECT(currentRotation = allocVector(REALSXP, 1));
-    REAL(currentRotation)[0] = rotationAngle;
+    PROTECT(currentWidthCM = ScalarReal(vpWidthCM));
+    PROTECT(currentHeightCM = ScalarReal(vpHeightCM));
+    PROTECT(currentRotation = ScalarReal(rotationAngle));
     PROTECT(currentTransform = allocMatrix(REALSXP, 3, 3));
     for (i=0; i<3; i++)
 	for (j=0; j<3; j++)
@@ -370,7 +367,7 @@ void calcViewportTransform(SEXP vp, SEXP parent, Rboolean incremental,
     UNPROTECT(4);
 }
 
-void initVP(GEDevDesc *dd)
+void initVP(pGEDevDesc dd)
 {
     SEXP vpfnname, vpfn, vp;
     SEXP xscale, yscale;
