@@ -621,7 +621,7 @@ static SEXP do_dispatch(SEXP fname, SEXP ev, SEXP mlist, int firstTry,
     if(isNull(method)) {
       if(!firstTry)
 	error(_("no matching method for function '%s' (argument '%s', with class \"%s\")"),
-	      CHAR(asChar(fname)), CHAR(PRINTNAME(arg_sym)), class);
+	      EncodeChar(asChar(fname)), EncodeChar(PRINTNAME(arg_sym)), class);
       UNPROTECT(nprotect);
       return(R_NilValue);
     }
