@@ -1,7 +1,7 @@
 #  File src/library/parallel/R/zzz.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -21,13 +21,16 @@
 if (.Platform$OS.type == "windows")
     utils::globalVariables(c("mc_pids", "clean_pids"), add = TRUE)
 
+## dummy, just so we can register a finalizer
+.fin.env <- new.env()
+
 .onLoad <- function(libname, pkgname)
 {
     initDefaultClusterOptions(libname)
     cores <- getOption("mc.cores", NULL)
     if(is.null(cores) && !is.na(nc <- as.integer(Sys.getenv("MC_CORES"))))
         options("mc.cores" = nc)
-    if(.Platform$OS.type == "unix") reg.finalizer(mc_pids, clean_pids, TRUE)
+    if(.Platform$OS.type == "unix") reg.finalizer(.fin.env, clean_pids, TRUE)
 }
 
 .onUnload <-
