@@ -1,7 +1,7 @@
 #  File src/library/stats/R/chisq.test.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ chisq.test <- function(x, y = NULL, correct = TRUE,
                        paste(DNAME2, collapse = "\n"))
     }
 
-    if (any(x < 0) || any(is.na(x)))
+    if (any(x < 0) || anyNA(x))
 	stop("all entries of 'x' must be nonnegative and finite")
     if ((n <- sum(x)) == 0)
 	stop("at least one entry of 'x' must be positive")
@@ -79,24 +79,13 @@ chisq.test <- function(x, y = NULL, correct = TRUE,
 	dimnames(E) <- dimnames(x)
 	if (simulate.p.value && all(sr > 0) && all(sc > 0)) {
 	    setMETH()
-	    tmp <- .C(C_chisqsim,
-		      nr,
-		      nc,
-		      as.integer(sr),
-		      as.integer(sc),
-		      as.integer(n),
-		      as.integer(B),
-		      as.double(E),
-		      integer(nr * nc),
-		      double(n + 1L),
-		      integer(nc),
-		      results = double(B))
+            tmp <- .Call(C_chisq_sim, sr, sc, B, E)
 	    ## Sorting before summing may look strange, but seems to be
 	    ## a sensible way to deal with rounding issues (PR#3486):
 	    STATISTIC <- sum(sort((x - E) ^ 2 / E, decreasing = TRUE))
 	    PARAMETER <- NA
 	    ## use correct significance level for a Monte Carlo test
-	    PVAL <- (1 + sum(tmp$results >= almost.1 * STATISTIC)) / (B + 1)
+	    PVAL <- (1 + sum(tmp >= almost.1 * STATISTIC)) / (B + 1)
 	}
 	else {
 	    if (simulate.p.value)
@@ -126,8 +115,8 @@ chisq.test <- function(x, y = NULL, correct = TRUE,
 	METHOD <- "Chi-squared test for given probabilities"
 	E <- n * p
         V <- n * p * (1 - p)
-	names(E) <- names(x)
 	STATISTIC <- sum((x - E) ^ 2 / E)
+	names(E) <- names(x)
 	if(simulate.p.value) {
 	    setMETH()
 	    nx <- length(x)
