@@ -29,6 +29,13 @@
 #include <R_ext/RS.h>	       	/* for Memcpy */
 
 #include "statsR.h"
+#undef _
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(String) dgettext ("stats", String)
+#else
+#define _(String) (String)
+#endif
 
 
 /* Formerly in src/appl/fmim.c */
