@@ -1,6 +1,8 @@
 #  File src/library/tcltk/R/tclsearch.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -29,8 +31,8 @@ addTclPath <- function(path = ".")
 
 tclRequire <- function(package, warn = TRUE)
 {
-    a <- try(tcl("package", "require", package), silent=TRUE)
-    if (inherits(a, "try-error")){
+    a <- tryCatch(tcl("package", "require", package), error = identity)
+    if (inherits(a, "error")) {
         if (warn)
             warning(gettextf("Tcl package '%s' not found", package),
                     domain = NA)
