@@ -1,7 +1,7 @@
 #  File src/library/tools/R/Rd2ex.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ Rd2ex <-
     WriteLines <- function(x, con, outputEncoding, ...) {
         if (outputEncoding != "UTF-8") {
             x <- iconv(x, "UTF-8", outputEncoding,  mark=FALSE)
-            if (any(is.na(x))) {
+            if (anyNA(x)) {
                 x <- iconv(x, "UTF-8", outputEncoding, sub="byte", mark=FALSE)
                 encode_warn <<- TRUE
             }
@@ -83,7 +83,9 @@ Rd2ex <-
                     writeLines("", con)
                     render(x[[1L]], paste0(if (commentDontrun) "##D ", prefix))
                 } else render(x[[1L]], prefix)
-                for(i in 2:length(x)) render(x[[i]], paste0(if (commentDontrun) "##D ", prefix))
+                if(length(x) >= 2L)
+                    for(i in 2:length(x))
+                        render(x[[i]], paste0(if (commentDontrun) "##D ", prefix))
                 last <- x[[length(x)]]
                 if (!grepl("\n$", last[length(last)], perl = TRUE))
                     writeLines("", con)
