@@ -1,7 +1,7 @@
 #  File src/library/tools/R/Rd2pdf.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -61,12 +61,12 @@
         ## \AsIs is per-para.
         text <- strsplit(text, "\n\n", fixed = TRUE, useBytes = TRUE)[[1L]]
         Encoding(text) <- "unknown"
-        if(f %in% c("Author", "Maintainer"))
+        if(f %in% c("Author", "Maintainer", "Contact"))
             text <- gsub("<([^@ ]+)@([^> ]+)>",
                          "}\\\\email{\\1@\\2}\\\\AsIs{",
                          text, useBytes = TRUE)
-        if(f == "URL")
-            text <- gsub("(http://|ftp://)([^[:space:]]+)",
+        if(f %in% c("URL", "BugReports", "Contact"))
+            text <- gsub("(http://|ftp://|https://)([^[:space:],]+)",
                          "}\\\\url{\\1\\2}\\\\AsIs{",
                          text, useBytes = TRUE)
         text <- paste0("\\AsIs{", text, "}")
@@ -191,21 +191,26 @@
             }
             if (!silent) message(domain = NA)
         } else {
-            files <- c(Sys.glob(file.path(pkgdir, "*.Rd")),
-                       Sys.glob(file.path(pkgdir, "*.rd")))
-            if (!length(files)) {
-                ## is this a source package?  That has man/*.Rd files.
-                files <- c(Sys.glob(file.path(pkgdir, "man", "*.Rd")),
-                           Sys.glob(file.path(pkgdir, "man", "*.rd")))
+            ## As from R 2.15.3, give priority to a man dir.
+            mandir <- file.path(pkgdir, "man")
+            if (file_test("-d", mandir)) {
+                files <- c(Sys.glob(file.path(mandir, "*.Rd")),
+                           Sys.glob(file.path(mandir, "*.rd")))
+                if (is.null(extraDirs)) extraDirs <- .Platform$OS.type
+                for(e in extraDirs)
+                    files <- c(files,
+                               Sys.glob(file.path(mandir, e, "*.Rd")),
+                               Sys.glob(file.path(mandir, e, "*.rd")))
+                if (!length(files))
+                    stop("this package has a ", sQuote("man"), " directory but no .Rd files",
+                         domain = NA)
+           } else {
+                files <- c(Sys.glob(file.path(pkgdir, "*.Rd")),
+                           Sys.glob(file.path(pkgdir, "*.rd")))
                 if (!length(files))
                     stop("this package does not have either a ", sQuote("latex"),
                          " or a (source) ", sQuote("man"), " directory",
                          domain = NA)
-                if (is.null(extraDirs)) extraDirs <- .Platform$OS.type
-                for(e in extraDirs)
-                    files <- c(files,
-                               Sys.glob(file.path(pkgdir, "man", e, "*.Rd")),
-                               Sys.glob(file.path(pkgdir, "man", e, "*.rd")))
             }
             paths <- files
             ## Use a partial Rd db if there is one.
@@ -672,9 +677,10 @@ setEncoding2, "
             "  -o, --output=FILE	write output to FILE",
             "      --force		overwrite output file if it exists",
             "      --title=NAME	use NAME as the title of the document",
-            "      --no-index	don't index output",
-            "      --no-description	don't typeset the description of a package",
+            "      --no-index	do not index output",
+            "      --no-description	do not typeset the description of a package",
             "      --internals	typeset 'internal' documentation (usually skipped)",
+            "      --build_dir=DIR	use DIR as the working directory",
             "",
             "The output papersize is set by the environment variable R_PAPERSIZE.",
             "The PDF previewer is set by the environment variable R_PDFVIEWER.",
@@ -756,7 +762,7 @@ setEncoding2, "
         } else if (substr(a, 1, 17) == "--outputEncoding=") {
             outenc <- substr(a, 18, 1000)
         } else if (substr(a, 1, 12) == "--build-dir=") {
-            build_dir<- substr(a, 13, 1000)
+            build_dir <- substr(a, 13, 1000)
         } else if (a == "--no-index") {
             index <- FALSE
         } else if (a == "--no-description") {
@@ -824,7 +830,7 @@ setEncoding2, "
 
     res <- try(texi2pdf('Rd2.tex', quiet = FALSE, index = index))
     if (inherits(res, "try-error")) {
-        message("Error in running tools::texi2pdf")
+        message("Error in running tools::texi2pdf()")
         do_cleanup()
         q("no", status = 1L, runLast = FALSE)
     }
