@@ -1,7 +1,7 @@
 #  File src/library/tools/R/sotools.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 2011-2 The R Core Team
+#  Copyright (C) 2011-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -88,6 +88,9 @@ so_symbol_names_table <-
       "linux, C, gcc, vprintf, vprintf",
       "linux, C++, gxx, std::cout, _ZSt4cout",
       "linux, C++, gxx, std::cerr, _ZSt4cerr",
+      ## libcxx variants
+      "linux, C++, gxx, std::cout, _ZNSt3__14coutE",
+      "linux, C++, gxx, std::cerr, _ZNSt3__14cerrE",
       "linux, Fortran, gfortran, write, _gfortran_st_write",
       "linux, Fortran, gfortran, print, _gfortran_st_write",
       "linux, Fortran, gfortran, stop, _gfortran_stop_numeric_f08",
@@ -105,6 +108,9 @@ so_symbol_names_table <-
       "osx, C, gcc, vprintf, _vprintf",
       "osx, C++, gxx, std::cout, __ZSt4cout",
       "osx, C++, gxx, std::cerr, __ZSt4cerr",
+      ## libcxx variants
+      "osx, C++, gxx, std::cout, __ZNSt3__14coutE",
+      "osx, C++, gxx, std::cerr, __ZNSt3__14cerrE",
       "osx, Fortran, gfortran, write, __gfortran_st_write",
       "osx, Fortran, gfortran, print, __gfortran_st_write",
       "osx, Fortran, gfortran, stop, __gfortran_stop_numeric",
