@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2003-11   The R Development Core Team.
+ *  Copyright (C) 2003-11   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #endif
 
 SEXP delim_match(SEXP x, SEXP delims);
+SEXP dirchmod(SEXP dr);
 SEXP Rmd5(SEXP files);
 SEXP check_nonASCII(SEXP text, SEXP ignore_quotes);
 SEXP check_nonASCII2(SEXP text);
@@ -36,4 +37,13 @@ SEXP doTabExpand(SEXP strings, SEXP starts);
 SEXP ps_kill(SEXP pid, SEXP signal);
 SEXP ps_sigs(SEXP);
 SEXP ps_priority(SEXP pid, SEXP value);
+SEXP codeFilesAppend(SEXP f1, SEXP f2);
+SEXP getfmts(SEXP format);
+SEXP startHTTPD(SEXP sIP, SEXP sPort);
+SEXP stopHTTPD(void);
+
+SEXP C_parseLatex(SEXP call, SEXP op, SEXP args, SEXP env);
+SEXP C_parseRd(SEXP call, SEXP op, SEXP args, SEXP env);
+SEXP C_deparseRd(SEXP e, SEXP state);
+
 #endif
