@@ -1,7 +1,7 @@
 #  File src/library/utils/R/package.skeleton.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 package.skeleton <-
     function(name = "anRpackage", list = character(), environment = .GlobalEnv,
-	     path = ".", force = FALSE, namespace = TRUE,
+	     path = ".", force = FALSE,
              code_files = character())
 {
     safe.dir.create <- function(path)
@@ -104,9 +104,6 @@ package.skeleton <-
 	file = description, sep = "")
     close(description)
 
-    if(!missing(namespace))
-	warning("From R 2.14.0 on, every package gets a NAMESPACE.",
-		" Argument 'namespace' is deprecated.", domain = NA)
     ## NAMESPACE
     ## <NOTE>
     ## For the time being, we export all non-internal objects using the pattern
