@@ -60,7 +60,10 @@ getSrcDirectory <- function(x, unique=TRUE) {
 getSrcref <- function(x) {
     if (inherits(x, "srcref")) return(x)
     if (!is.null(srcref <- attr(x, "srcref"))) return(srcref)
-    if (is.function(x)) return(getSrcref(body(x)))
+    if (is.function(x) && !is.null(srcref <- getSrcref(body(x)))) 
+	return(srcref)
+    if (methods::is(x, "MethodDefinition")) 
+	return(getSrcref(unclass(methods::unRematchDefinition(x))))
     NULL
 }
 
@@ -91,7 +94,7 @@ getSrcfile <- function(x) {
 }
 
 substr_with_tabs <- function(x, start, stop, tabsize = 8) {
-    widths <- rep(1, nchar(x))
+    widths <- rep_len(1, nchar(x))
     tabs <- which(strsplit(x,"")[[1]] == "\t")
     for (i in tabs) {
 	cols <- cumsum(widths)
