@@ -176,6 +176,7 @@ static const long double tbl[] =
     1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19,
     1e20, 1e21, 1e22, 1e23, 1e24, 1e25, 1e26, 1e27
 };
+#define KP_MAX 27
 #else
 static const double tbl[] =
 {
@@ -184,6 +185,7 @@ static const double tbl[] =
     1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19,
     1e20, 1e21, 1e22
 };
+#define KP_MAX 22
 #endif
 
 static void 
@@ -230,9 +232,9 @@ scientific(double *x, int *sgn, int *kpower, int *nsig, int *roundingwidens)
             r_prec /= powl(10.0, (long double) kp);
 #else
         else if (kp <= R_dec_min_exponent)
-            r_prec = (r_prec * 1e+303)/pow(10.0, (double)(kp+303));
+            r_prec = (r_prec * 1e+303)/Rexp10((double)(kp+303));
         else
-            r_prec /= pow(10.0, (double) kp);
+            r_prec /= Rexp10((double) kp);
 #endif
         if (r_prec < tbl[R_print.digits]) {
             r_prec *= 10.0;
@@ -254,9 +256,9 @@ scientific(double *x, int *sgn, int *kpower, int *nsig, int *roundingwidens)
            is in range. Representation of 1e+303 has low error.
          */
         else if (kp <= R_dec_min_exponent)
-            r_prec = (r_prec * 1e+303)/pow(10.0, (double)(kp+303));
+            r_prec = (r_prec * 1e+303)/Rexp10((double)(kp+303));
         else
-            r_prec /= pow(10.0, (double)kp);
+            r_prec /= Rexp10((double)kp);
         if (r_prec < tbl[R_print.digits]) {
             r_prec *= 10.0;
             kp--;
@@ -280,7 +282,19 @@ scientific(double *x, int *sgn, int *kpower, int *nsig, int *roundingwidens)
             kp += 1;
         }
         *kpower = kp + R_print.digits - 1;
-	*roundingwidens = *kpower > 0 && r < tbl[*kpower + 1];
+
+	/* Scientific format may do more rounding than fixed format, e.g.
+	   9996 with 3 digits is 1e+04 in scientific, but 9996 in fixed.
+	   This happens when the true value r is less than 10^(kpower+1)
+	   and would not round up to it in fixed format.
+	   Here rgt is the decimal place that will be cut off by rounding */
+	   
+	int rgt = R_print.digits - *kpower;
+	/* bound rgt by 0 and KP_MAX */
+	rgt = rgt < 0 ? 0 : rgt > KP_MAX ? KP_MAX : rgt;
+	double fuzz = 0.5/(double)tbl[1 + rgt];
+	// kpower can be bigger than the table.	
+	*roundingwidens = *kpower > 0 && *kpower <= KP_MAX && r < tbl[*kpower + 1] - fuzz;
     }
 }
 
