@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2005   The R Core Team
+ *  Copyright (C) 2005-2013 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 /*  This file was contributed by Ei-ji Nakama.
  *  It exports locale2charset for use in gram.y, and rlocale.c on MacOS X.
+ *  And sysutils.c, grDevices/src/devPS.c
  */
 
 /* setlocale(LC_CTYPE,NULL) to encodingname cf nl_langinfo(LC_CTYPE) */
@@ -52,7 +53,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include <R_ext/rlocale.h> /* To get the correct linkage for locale2charset */
+//#include <rlocale.h> /* To get the correct linkage for locale2charset */
 
 /* name_value struct */
 typedef struct {
@@ -61,6 +62,7 @@ typedef struct {
 } name_value;
 
 
+#ifndef __APPLE__
 /*
  * codeset name defined.
  *
@@ -476,6 +478,7 @@ static const name_value guess[] = {
     {"zu_ZA",                          ENC_ISO8859_1},
 };
 static const int guess_count = (sizeof(guess)/sizeof(name_value));
+#endif
 
 static const name_value known[] = {
     {"iso88591", "ISO8859-1"},
@@ -579,7 +582,7 @@ static char* name_value_search(const char *name, const name_value table[],
 }
 #endif
 
-char *locale2charset(const char *locale)
+const char *locale2charset(const char *locale)
 {
     static char charset[128];
 
@@ -637,7 +640,7 @@ char *locale2charset(const char *locale)
 	*/
     case 1257: return "ISO8859-13";
     default:
-	sprintf(charset, "CP%u", cp);
+	snprintf(charset, 128, "CP%u", cp);
 	return charset;
     }
 #endif
@@ -654,32 +657,32 @@ char *locale2charset(const char *locale)
     if (0 == strcmp(enc, "UTF-8")) strcpy(enc, "utf8");
 
     if(strcmp(enc, "") && strcmp(enc, "utf8")) {
-	for(i = 0; enc[i]; i++) enc[i] = tolower(enc[i]);
+	for(i = 0; enc[i]; i++) enc[i] = (char) tolower(enc[i]);
 
 	for(i = 0; i < known_count; i++)
 	    if (0 == strcmp(known[i].name,enc)) return known[i].value;
 
 	/* cut encoding old linux cp- */
 	if (0 == strncmp(enc, "cp-", 3)){
-	    sprintf(charset,"CP%s", enc+3);
+	    snprintf(charset, 128, "CP%s", enc+3);
 	    return charset;
 	}
 	/* cut encoding IBM ibm- */
 	if (0 == strncmp(enc, "ibm", 3)){
 	    cp = atoi(enc + 3);
-	    sprintf(charset, "IBM-%d", abs(cp));
+	    snprintf(charset, 128, "IBM-%d", abs(cp));
 	    /* IBM-[0-9]+ case */
 	    if(cp != 0) return charset;
 	    /* IBM-eucXX case */
 	    strncpy(charset, (enc[3] == '-') ? enc+4: enc+3, sizeof(charset));
 	    if(strncmp(charset, "euc", 3)) {
 		if (charset[3] != '-') {
-		    for(i = strlen(charset)-3; 0 < i; i--)
+		    for(i = (int) strlen(charset)-3; 0 < i; i--)
 			charset[i+1] = charset[i];
 		    charset[3] = '-';
 		}
 		for(i = 0; charset[i]; i++)
-		    charset[i] = toupper(charset[i]);
+		    charset[i] = (char) toupper(charset[i]);
 		return charset;
 	    }
 	}
@@ -706,7 +709,7 @@ char *locale2charset(const char *locale)
     if(0 == strcmp(enc, "utf8")) return "UTF-8";
 
     value = name_value_search(la_loc, guess, guess_count);
-    return value == NULL ? (char *) "ASCII" : value;
+    return value == NULL ? "ASCII" : value;
 #endif
 }
 
