@@ -78,7 +78,8 @@ static void PrintLanguageEtc(SEXP, Rboolean, Rboolean);
 
 
 #define TAGBUFLEN 256
-static char tagbuf[TAGBUFLEN + 5];
+#define TAGBUFLEN0 TAGBUFLEN + 6
+static char tagbuf[TAGBUFLEN0];
 
 
 /* Used in X11 module for dataentry */
@@ -323,7 +324,7 @@ static void PrintGenericVector(SEXP s, SEXP env)
 {
     int i, taglen, ns, w, d, e, wr, dr, er, wi, di, ei;
     SEXP dims, t, names, newcall, tmp;
-    char pbuf[115], *ptag, save[TAGBUFLEN + 5];
+    char pbuf[115], *ptag, save[TAGBUFLEN0];
 
     ns = length(s);
     if((dims = getAttrib(s, R_DimSymbol)) != R_NilValue && length(dims) > 1) {
@@ -608,7 +609,7 @@ static void printList(SEXP s, SEXP env)
 		    else if( isValidName(CHAR(PRINTNAME(TAG(s)))) )
 			sprintf(ptag, "$%s", CHAR(PRINTNAME(TAG(s))));
 		    else
-			sprintf(ptag, "$`%s`", CHAR(PRINTNAME(TAG(s))));
+			sprintf(ptag, "$`%s`", EncodeChar(PRINTNAME(TAG(s))));
 		}
 	    }
 	    else {
@@ -835,7 +836,7 @@ static void printAttributes(SEXP s, SEXP env, Rboolean useSlots)
 {
     SEXP a;
     char *ptag;
-    char save[TAGBUFLEN + 5] = "\0";
+    char save[TAGBUFLEN0] = "\0";
 
     a = ATTRIB(s);
     if (a != R_NilValue) {
@@ -983,6 +984,12 @@ void attribute_hidden PrintValueEnv(SEXP s, SEXP env)
 	else /* S3 */
 	    PROTECT(call = lang2(install("print"), s));
 
+	if (TYPEOF(s) == SYMSXP || TYPEOF(s) == LANGSXP)
+	    /* If s is not self-evaluating wrap it in a promise. Doing
+	       this unconditionally seems to create problems in the S4
+	       case. */
+	    SETCADR(call, R_mkEVPROMISE(s, s));
+
 	eval(call, env);
 	UNPROTECT(1);
     } else PrintValueRec(s, env);
