@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1999--2008  The R Development Core Team
+ *  Copyright (C) 1999--2014  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -66,12 +66,29 @@
 
 #include "RBufferUtils.h"
 
+
+#if !defined(__STDC_ISO_10646__) && (defined(__APPLE__) || defined(__FreeBSD__))
+/* This may not be 100% true (see the comment in rlocales.h),
+   but it seems true in normal locales */
+# define __STDC_ISO_10646__
+#endif
+
+#ifdef Win32
+int trio_vsnprintf(char *buffer, size_t bufferSize, const char *format,
+		   va_list args);
+# define vsnprintf trio_vsnprintf
+#endif
+
 extern int R_OutputCon; /* from connections.c */
 
+#ifndef min
+#define min(a, b) (((a)<(b))?(a):(b))
+#endif
 
 #define BUFSIZE 8192  /* used by Rprintf etc */
 
 /* Only if ierr < 0 or not is currently used */
+attribute_hidden
 R_size_t R_Decode2Long(char *p, int *ierr)
 {
     R_size_t v = strtol(p, &p, 10);
@@ -82,11 +99,11 @@ R_size_t R_Decode2Long(char *p, int *ierr)
 	REprintf("R_Decode2Long(): v=%ld\n", v);
     if(p[0] == 'G') {
 	if((Giga * (double)v) > R_SIZE_T_MAX) { *ierr = 4; return(v); }
-	return (Giga*v);
+	return (R_size_t) Giga * v;
     }
     else if(p[0] == 'M') {
 	if((Mega * (double)v) > R_SIZE_T_MAX) { *ierr = 1; return(v); }
-	return (Mega*v);
+	return (R_size_t) Mega * v;
     }
     else if(p[0] == 'K') {
 	if((1024 * (double)v) > R_SIZE_T_MAX) { *ierr = 2; return(v); }
@@ -119,21 +136,24 @@ const char *EncodeInteger(int x, int w)
 {
     static char buff[NB];
     if(x == NA_INTEGER) snprintf(buff, NB, "%*s", w, CHAR(R_print.na_string));
-    else snprintf(buff, NB, "%*d", w, x);
+    else snprintf(buff, NB, "%*d", min(w, (NB-1)), x);
     buff[NB-1] = '\0';
     return buff;
 }
 
-const char *EncodeRaw(Rbyte x)
+attribute_hidden
+const char *EncodeRaw(Rbyte x, const char * prefix)
 {
     static char buff[10];
-    sprintf(buff, "%02x", x);
+    sprintf(buff, "%s%02x", prefix, x);
     return buff;
 }
 
+attribute_hidden
 const char *EncodeEnvironment(SEXP x)
 {
-    static char ch[100];
+    const void *vmax = vmaxget();
+    static char ch[1000];
     if (x == R_GlobalEnv)
 	sprintf(ch, "<environment: R_GlobalEnv>");
     else if (x == R_BaseEnv)
@@ -141,13 +161,14 @@ const char *EncodeEnvironment(SEXP x)
     else if (x == R_EmptyEnv)
 	sprintf(ch, "<environment: R_EmptyEnv>");
     else if (R_IsPackageEnv(x))
-	sprintf(ch, "<environment: %s>",
+	snprintf(ch, 1000, "<environment: %s>",
 		translateChar(STRING_ELT(R_PackageEnvName(x), 0)));
     else if (R_IsNamespaceEnv(x))
-	sprintf(ch, "<environment: namespace:%s>",
+	snprintf(ch, 1000, "<environment: namespace:%s>",
 		translateChar(STRING_ELT(R_NamespaceEnvSpec(x), 0)));
-    else sprintf(ch, "<environment: %p>", (void *)x);
+    else snprintf(ch, 1000, "<environment: %p>", (void *)x);
 
+    vmaxset(vmax);
     return ch;
 }
 
@@ -166,16 +187,16 @@ const char *EncodeReal(double x, int w, int d, int e, char cdec)
     }
     else if (e) {
 	if(d) {
-	    sprintf(fmt,"%%#%d.%de", w, d);
+	    sprintf(fmt,"%%#%d.%de", min(w, (NB-1)), d);
 	    snprintf(buff, NB, fmt, x);
 	}
 	else {
-	    sprintf(fmt,"%%%d.%de", w, d);
+	    sprintf(fmt,"%%%d.%de", min(w, (NB-1)), d);
 	    snprintf(buff, NB, fmt, x);
 	}
     }
     else { /* e = 0 */
-	sprintf(fmt,"%%%d.%df", w, d);
+	sprintf(fmt,"%%%d.%df", min(w, (NB-1)), d);
 	snprintf(buff, NB, fmt, x);
     }
     buff[NB-1] = '\0';
@@ -202,16 +223,16 @@ const char *EncodeReal2(double x, int w, int d, int e)
     }
     else if (e) {
 	if(d) {
-	    sprintf(fmt,"%%#%d.%de", w, d);
+	    sprintf(fmt,"%%#%d.%de", min(w, (NB-1)), d);
 	    snprintf(buff, NB, fmt, x);
 	}
 	else {
-	    sprintf(fmt,"%%%d.%de", w, d);
+	    sprintf(fmt,"%%%d.%de", min(w, (NB-1)), d);
 	    snprintf(buff, NB, fmt, x);
 	}
     }
     else { /* e = 0 */
-	sprintf(fmt,"%%#%d.%df", w, d);
+	sprintf(fmt,"%%#%d.%df", min(w, (NB-1)), d);
 	snprintf(buff, NB, fmt, x);
     }
     buff[NB-1] = '\0';
@@ -271,7 +292,7 @@ const char
    which Western versions at least do not.).
 */
 
-#include <R_ext/rlocale.h> /* redefines isw* functions */
+#include <rlocale.h> /* redefines isw* functions */
 
 #ifdef Win32
 #include "rgui_UTF8.h"
@@ -296,8 +317,8 @@ int Rstrwid(const char *str, int slen, cetype_t ienc, int quote)
 
 	if(ienc != CE_UTF8)  mbs_init(&mb_st);
 	for (i = 0; i < slen; i++) {
-	    res = (ienc == CE_UTF8) ? utf8toucs(&wc, p):
-		mbrtowc(&wc, p, MB_CUR_MAX, NULL);
+	    res = (ienc == CE_UTF8) ? (int) utf8toucs(&wc, p):
+		(int) mbrtowc(&wc, p, MB_CUR_MAX, NULL);
 	    if(res >= 0) {
 		k = wc;
 		if(0x20 <= k && k < 0x7f && iswprint(wc)) {
@@ -307,6 +328,7 @@ int Rstrwid(const char *str, int slen, cetype_t ienc, int quote)
 			break;
 		    case L'\'':
 		    case L'"':
+		    case L'`':
 			len += (quote == *p) ? 2 : 1;
 			break;
 		    default:
@@ -355,6 +377,7 @@ int Rstrwid(const char *str, int slen, cetype_t ienc, int quote)
 			len += 2; break;
 		    case '\'':
 		    case '"':
+		    case '`':
 			len += (quote == *p)? 2 : 1; break;
 		    default:
 			len++; break;
@@ -403,11 +426,16 @@ int Rstrlen(SEXP s, int quote)
    format().
  */
 
+attribute_hidden
 const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 {
     int b, b0, i, j, cnt;
     const char *p; char *q, buf[11];
-    cetype_t ienc = CE_NATIVE;
+    cetype_t ienc = getCharCE(s);
+    Rboolean useUTF8 = w < 0;
+    const void *vmax = vmaxget();
+
+    if (w < 0) w = w + 1000000;
 
     /* We have to do something like this as the result is returned, and
        passed on by EncodeElement -- so no way could be end user be
@@ -418,13 +446,12 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 
     if (s == NA_STRING) {
 	p = quote ? CHAR(R_print.na_string) : CHAR(R_print.na_string_noquote);
-	cnt = i = quote ? strlen(CHAR(R_print.na_string)) :
-	    strlen(CHAR(R_print.na_string_noquote));
+	cnt = i = (int)(quote ? strlen(CHAR(R_print.na_string)) :
+			strlen(CHAR(R_print.na_string_noquote)));
 	quote = 0;
     } else {
 #ifdef Win32
 	if(WinUTF8out) {
-	    ienc = getCharCE(s);
 	    if(ienc == CE_UTF8) {
 		p = CHAR(s);
 		i = Rstrlen(s, quote);
@@ -444,8 +471,9 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 #endif
 	{
 	    if(IS_BYTES(s)) {
+		ienc = CE_NATIVE;
 		p = CHAR(s);
-		cnt = strlen(p);
+		cnt = (int) strlen(p);
 		const char *q;
 		char *pp = R_alloc(4*cnt+1, 1), *qq = pp, buf[5];
 		for (q = p; *q; q++) {
@@ -462,13 +490,18 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 		*qq = '\0';
 		p = pp;
 		i = cnt;
+	    } else if (useUTF8 && ienc == CE_UTF8) {
+		p = CHAR(s);
+		i = Rstrlen(s, quote);
+		cnt = LENGTH(s);
 	    } else {
+		ienc = CE_NATIVE;
 		p = translateChar(s);
 		if(p == CHAR(s)) {
 		    i = Rstrlen(s, quote);
 		    cnt = LENGTH(s);
 		} else {
-		    cnt = strlen(p);
+		    cnt = (int) strlen(p);
 		    i = Rstrwid(p, cnt, CE_NATIVE, quote);
 		}
 	    }
@@ -493,7 +526,7 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 	for(i = 0 ; i < b0 ; i++) *q++ = ' ';
 	b -= b0;
     }
-    if(quote) *q++ = quote;
+    if(quote) *q++ = (char) quote;
     if(mbcslocale || ienc == CE_UTF8) {
 	int j, res;
 	mbstate_t mb_st;
@@ -507,8 +540,8 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 	else if(WinUTF8out) { memcpy(q, UTF8in, 3); q += 3; }
 #endif
 	for (i = 0; i < cnt; i++) {
-	    res = (ienc == CE_UTF8) ? utf8toucs(&wc, p):
-		mbrtowc(&wc, p, MB_CUR_MAX, NULL);
+	    res = (int)((ienc == CE_UTF8) ? utf8toucs(&wc, p):
+			mbrtowc(&wc, p, MB_CUR_MAX, NULL));
 	    if(res >= 0) { /* res = 0 is a terminator */
 		k = wc;
 		/* To be portable, treat \0 explicitly */
@@ -519,6 +552,7 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 			break;
 		    case L'\'':
 		    case L'"':
+		    case L'`':
 			if(quote == *p)  *q++ = '\\'; *q++ = *p++;
 			break;
 		    default:
@@ -561,7 +595,8 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 			else
 #endif
 			    snprintf(buf, 11, "\\u%04x", k);
-			memcpy(q, buf, j = strlen(buf));
+			j = (int) strlen(buf);
+			memcpy(q, buf, j);
 			q += j;
 			p += res;
 		    }
@@ -588,6 +623,7 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 		    case '\\': *q++ = '\\'; *q++ = '\\'; break;
 		    case '\'':
 		    case '"':
+		    case '`':
 			if(quote == *p)  *q++ = '\\'; *q++ = *p; break;
 		    default: *q++ = *p; break;
 		    }
@@ -626,15 +662,20 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 #ifdef Win32
     if(WinUTF8out && ienc == CE_UTF8)  { memcpy(q, UTF8out, 3); q += 3; }
 #endif
-    if(quote) *q++ = quote;
+    if(quote) *q++ = (char) quote;
     if(b > 0 && justify != Rprt_adj_right) {
 	for(i = 0 ; i < b ; i++) *q++ = ' ';
     }
     *q = '\0';
+
+    vmaxset(vmax);
     return buffer->data;
 }
 
 /* EncodeElement is called by cat(), write.table() and deparsing. */
+
+/* NB this is called by R.app even though it is in no public header, so 
+   alter there if you alter this */
 const char *EncodeElement(SEXP x, int indx, int quote, char dec)
 {
     int w, d, e, wi, di, ei;
@@ -662,7 +703,7 @@ const char *EncodeElement(SEXP x, int indx, int quote, char dec)
 	res = EncodeComplex(COMPLEX(x)[indx], w, d, e, wi, di, ei, dec);
 	break;
     case RAWSXP:
-	res = EncodeRaw(RAW(x)[indx]);
+	res = EncodeRaw(RAW(x)[indx], "");
 	break;
     default:
 	res = NULL; /* -Wall */
@@ -671,20 +712,14 @@ const char *EncodeElement(SEXP x, int indx, int quote, char dec)
     return res;
 }
 
-#if 0
-char *Rsprintf(char *format, ...)
+/* EncodeChar is a simple wrapper for EncodeString
+   called by error messages to display CHARSXP values */
+//attribute_hidden
+const char *EncodeChar(SEXP x)
 {
-    static char buffer[1001]; /* unsafe, as assuming max length, but all
-				 internal uses are for a few characters */
-    va_list(ap);
-
-    va_start(ap, format);
-    vsnprintf(buffer, 1000, format, ap);
-    va_end(ap);
-    buffer[1000] = '\0';
-    return buffer;
+    return EncodeString(x, 0, 0, Rprt_adj_left);
 }
-#endif
+
 
 void Rprintf(const char *format, ...)
 {
@@ -717,6 +752,7 @@ int vasprintf(char **strp, const char *fmt, va_list ap)
 #endif
 
 # define R_BUFSIZE BUFSIZE
+attribute_hidden
 void Rcons_vprintf(const char *format, va_list arg)
 {
     char buf[R_BUFSIZE], *p = buf;
@@ -752,7 +788,7 @@ void Rcons_vprintf(const char *format, va_list arg)
 	}
     }
 #endif /* HAVE_VASPRINTF */
-    R_WriteConsole(p, strlen(p));
+    R_WriteConsole(p, (int) strlen(p));
     if(usedRalloc) vmaxset(vmax);
     if(usedVasprintf) free(p);
 }
@@ -819,16 +855,16 @@ void REvprintf(const char *format, va_list arg)
 
 	vsnprintf(buf, BUFSIZE, format, arg);
 	buf[BUFSIZE-1] = '\0';
-	R_WriteConsoleEx(buf, strlen(buf), 1);
+	R_WriteConsoleEx(buf, (int) strlen(buf), 1);
     }
 }
 
-int attribute_hidden IndexWidth(int n)
+int attribute_hidden IndexWidth(R_xlen_t n)
 {
     return (int) (log10(n + 0.5) + 1);
 }
 
-void attribute_hidden VectorIndex(int i, int w)
+void attribute_hidden VectorIndex(R_xlen_t i, int w)
 {
 /* print index label "[`i']" , using total width `w' (left filling blanks) */
     Rprintf("%*s[%ld]", w-IndexWidth(i)-2, "", i);
