@@ -348,11 +348,11 @@ static int InInteger(R_inpstream_t stream)
     switch (stream->type) {
     case R_pstream_ascii_format:
 	InWord(stream, word, sizeof(word));
-	sscanf(word, "%s", buf);
+	if(sscanf(word, "%s", buf) != 1) error(_("read error"));
 	if (strcmp(buf, "NA") == 0)
 	    return NA_INTEGER;
 	else
-	    sscanf(buf, "%d", &i);
+	    if(sscanf(buf, "%d", &i) != 1) error(_("read error"));
 	return i;
     case R_pstream_binary_format:
 	stream->InBytes(stream, &i, sizeof(int));
@@ -374,7 +374,7 @@ static double InReal(R_inpstream_t stream)
     switch (stream->type) {
     case R_pstream_ascii_format:
 	InWord(stream, word, sizeof(word));
-	sscanf(word, "%s", buf);
+	if(sscanf(word, "%s", buf) != 1) error(_("read error"));
 	if (strcmp(buf, "NA") == 0)
 	    return NA_REAL;
 	else if (strcmp(buf, "Inf") == 0)
@@ -382,7 +382,7 @@ static double InReal(R_inpstream_t stream)
 	else if (strcmp(buf, "-Inf") == 0)
 	    return R_NegInf;
 	else
-	    sscanf(buf, "%lg", &d);
+	    if(sscanf(buf, "%lg", &d) != 1) error(_("read error"));
 	return d;
     case R_pstream_binary_format:
 	stream->InBytes(stream, &d, sizeof(double));
@@ -2539,7 +2539,7 @@ static SEXP appendRawToFile(SEXP file, SEXP bytes)
 {
     FILE *fp;
     size_t len, out;
-    long pos;
+    long pos;  // what ftell gives: won't work for > 2GB files
     SEXP val;
 
     if (! IS_PROPER_STRING(file))
@@ -2724,7 +2724,7 @@ static SEXP R_getVarsFromFrame(SEXP vars, SEXP env, SEXP forcesxp)
 	if (tmp == R_UnboundValue) {
 /*		PrintValue(env);
 		PrintValue(R_GetTraceback(0)); */  /* DJM debugging */
-	    error(_("object '%s' not found"), CHAR(STRING_ELT(vars, i)));
+	    error(_("object '%s' not found"), EncodeChar(STRING_ELT(vars, i)));
 	    }
 	if (force && TYPEOF(tmp) == PROMSXP) {
 	    PROTECT(tmp);
@@ -2754,7 +2754,7 @@ SEXP R_decompress3(SEXP in, Rboolean *err);
    result to a file.  Returns the key position/length key for
    retrieving the value */
 
-SEXP attribute_hidden
+static SEXP
 R_lazyLoadDBinsertValue(SEXP value, SEXP file, SEXP ascii,
 			SEXP compsxp, SEXP hook)
 {
