@@ -1,7 +1,7 @@
 /*
   R : A Computer Language for Statistical Data Analysis
   Copyright (C) 1995-1996   Robert Gentleman and Ross Ihaka
-  Copyright (C) 1997-2010   The R Development Core Team
+  Copyright (C) 1997-2013   The R Core Team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,8 +20,7 @@
 
 /*
   See ../unix/system.txt for a description of some of these functions
-  Formally part of ../unix/sys-common.c
- */
+*/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -113,12 +112,19 @@ static char workspace_name[1000] = ".RData";
 #else
 static char workspace_name[PATH_MAX] = ".RData";
 
+attribute_hidden
 void set_workspace_name(const char *fn)
 {
     strncpy(workspace_name, fn, PATH_MAX);
 }
 #endif
 
+attribute_hidden
+const char* get_workspace_name()
+{
+    return workspace_name;
+}
+
 void R_RestoreGlobalEnv(void)
 {
     if(RestoreAction == SA_RESTORE) {
@@ -153,6 +159,7 @@ void R_DefParams(Rstart Rp)
     Rp->max_nsize = R_SIZE_T_MAX;
     Rp->ppsize = R_PPSSIZE;
     Rp->NoRenviron = FALSE;
+    R_SizeFromEnv(Rp);
 }
 
 #define Max_Nsize 50000000	/* about 1.4Gb 32-bit, 2.8Gb 64-bit */
@@ -187,21 +194,23 @@ static void SetSize(R_size_t vsize, R_size_t nsize)
 {
     char msg[1024];
 
-    /* vsize >0 to catch long->int overflow */
+    /* vsize > 0 to catch long->int overflow */
     if (vsize < 1000 && vsize > 0) {
 	R_ShowMessage("WARNING: vsize ridiculously low, Megabytes assumed\n");
-	vsize *= Mega;
+	vsize *= (R_size_t) Mega;
     }
     if(vsize < Min_Vsize || vsize > Max_Vsize) {
-	sprintf(msg, "WARNING: invalid v(ector heap)size `%lu' ignored\n"
+	snprintf(msg, 1024, 
+		 "WARNING: invalid v(ector heap)size `%lu' ignored\n"
 		 "using default = %gM\n", (unsigned long) vsize,
-		R_VSIZE / Mega);
+		 R_VSIZE / Mega);
 	R_ShowMessage(msg);
 	R_VSize = R_VSIZE;
     } else
 	R_VSize = vsize;
     if(nsize < Min_Nsize || nsize > Max_Nsize) {
-	sprintf(msg, "WARNING: invalid language heap (n)size `%lu' ignored,"
+	snprintf(msg, 1024,
+		 "WARNING: invalid language heap (n)size `%lu' ignored,"
 		 " using default = %ld\n", (unsigned long) nsize, R_NSIZE);
 	R_ShowMessage(msg);
 	R_NSize = R_NSIZE;
