@@ -605,6 +605,33 @@ static int ExtractExactArg(SEXP args)
     return exact;
 }
 
+/* Version of DispatchOrEval for "[" and friends that speeds up simple cases.
+   Also defined in subassign.c */
+static R_INLINE
+int R_DispatchOrEvalSP(SEXP call, SEXP op, const char *generic, SEXP args,
+		    SEXP rho, SEXP *ans)
+{
+    SEXP prom = NULL;
+    if (args != R_NilValue && CAR(args) != R_DotsSymbol) {
+	SEXP x = eval(CAR(args), rho);
+	PROTECT(x);
+	if (! OBJECT(x)) {
+	    *ans = CONS_NR(x, evalListKeepMissing(CDR(args), rho));
+	    UNPROTECT(1);
+	    return FALSE;
+	}
+	prom = mkPROMISE(CAR(args), R_GlobalEnv);
+	SET_PRVALUE(prom, x);
+	args = CONS(prom, CDR(args));
+	UNPROTECT(1);
+    }
+    PROTECT(args);
+    int disp = DispatchOrEval(call, op, generic, args, rho, ans, 0, 0);
+    if (prom) DECREMENT_REFCNT(PRVALUE(prom));
+    UNPROTECT(1);
+    return disp;
+}
+
 /* The "[" subset operator.
  * This provides the most general form of subsetting. */
 
@@ -618,7 +645,7 @@ SEXP attribute_hidden do_subset(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* to the generic code below.  Note that evaluation */
     /* retains any missing argument indicators. */
 
-    if(DispatchOrEval(call, op, "[", args, rho, &ans, 0, 0)) {
+    if(R_DispatchOrEvalSP(call, op, "[", args, rho, &ans)) {
 /*     if(DispatchAnyOrEval(call, op, "[", args, rho, &ans, 0, 0)) */
 	if (NAMED(ans))
 	    SET_NAMED(ans, 2);
@@ -634,8 +661,8 @@ static R_INLINE R_xlen_t scalarIndex(SEXP s)
 {
     if (ATTRIB(s) == R_NilValue)
 	switch (TYPEOF(s)) {
-	case REALSXP:
-	    if (XLENGTH(s) == 1 && ! ISNAN(REAL(s)[0]))
+	case REALSXP: // treat infinite indices as NA, like asInteger
+	    if (XLENGTH(s) == 1 && R_FINITE(REAL(s)[0]))
 		return (R_xlen_t) REAL(s)[0];
 	    else return -1;
 	case INTSXP:
@@ -848,8 +875,7 @@ SEXP attribute_hidden do_subset2(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* through to the generic code below.  Note that */
     /* evaluation retains any missing argument indicators. */
 
-    if(DispatchOrEval(call, op, "[[", args, rho, &ans, 0, 0)) {
-/*     if(DispatchAnyOrEval(call, op, "[[", args, rho, &ans, 0, 0)) */
+    if(R_DispatchOrEvalSP(call, op, "[[", args, rho, &ans)) {
 	if (NAMED(ans))
 	    SET_NAMED(ans, 2);
 	return(ans);
@@ -937,8 +963,24 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	SEXP thesub = CAR(subs);
 	int len = length(thesub);
 
-	if (len > 1)
+	if (len > 1) {
+#ifdef SWITCH_TO_REFCNT
+	    if (IS_GETTER_CALL(call)) {
+		/* this is (most likely) a getter call in a complex
+		   assighment so we duplicate as needed. The original
+		   x should have been duplicated if it might be
+		   shared */
+		if (MAYBE_SHARED(x))
+		    error("getter call used outside of a complex assignment.");
+		x = vectorIndex(x, thesub, 0, len-1, pok, call, TRUE);
+	    }
+	    else
+		x = vectorIndex(x, thesub, 0, len-1, pok, call, FALSE);
+#else
 	    x = vectorIndex(x, thesub, 0, len-1, pok, call, FALSE);
+#endif
+	    named_x = NAMED(x);
+	}
 	    
 	offset = get1index(thesub, getAttrib(x, R_NamesSymbol),
 			   xlength(x), pok, len > 1 ? len-1 : -1, call);
@@ -1097,7 +1139,7 @@ SEXP attribute_hidden do_subset3(SEXP call, SEXP op, SEXP args, SEXP env)
     /* through to the generic code below.  Note that */
     /* evaluation retains any missing argument indicators. */
 
-    if(DispatchOrEval(call, op, "$", args, env, &ans, 0, 0)) {
+    if(R_DispatchOrEvalSP(call, op, "$", args, env, &ans)) {
 	UNPROTECT(2);
 	if (NAMED(ans))
 	    SET_NAMED(ans, 2);
