@@ -1,6 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1999-2010  Guido Masarotto and the R Core Team
+ *  Copyright (C) 1999       Guido Masarotto
+ *  Copyright (C) 1999-2014  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -505,8 +506,13 @@ int R_SaveAsTIFF(void  *d, int width, int height,
     */
     TIFFSetField(out, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
 #endif
-    if(compression > 1)
-	TIFFSetField(out, TIFFTAG_COMPRESSION, compression);
+    if(compression > 1) {
+	if (compression > 10) {
+	    TIFFSetField(out, TIFFTAG_COMPRESSION, compression - 10);
+	    TIFFSetField(out, TIFFTAG_PREDICTOR, 2);
+	} else 
+	    TIFFSetField(out, TIFFTAG_COMPRESSION, compression);
+    }
 
     if (res > 0) {
 	TIFFSetField(out, TIFFTAG_RESOLUTIONUNIT, RESUNIT_INCH);
@@ -647,7 +653,9 @@ int R_SaveAsBmp(void  *d, int width, int height,
     BMPW((unsigned short) biBitCount); /* biBitCount */
     BMPDW(0); /* biCompression=BI_RGB */
     BMPDW(0); /* biSizeImage (with BI_RGB not needed)*/
-    lres = (int)(0.5 + res/0.0254);
+    if (res > 0)
+	lres = (int)(0.5 + res/0.0254);
+    else lres = 2835; // 72ppi = 2835 pixels/metre.
     BMPDW(lres); /* XPels/M */
     BMPDW(lres); /* XPels/M */
     BMPDW(biClrUsed); /* biClrUsed */
