@@ -11,21 +11,37 @@
  *    distribution function. Appl.Statist., 41, 478-482.
 
  *  Other parts
- *  Copyright (C) 2000-2009  The R Development Core Team
- *  Copyright (C) 2003-2009  The R Foundation
+ *  Copyright (C) 2000-2014  The R Core Team
+ *  Copyright (C) 2003-2014  The R Foundation
  */
 
+
+
 #include "nmath.h"
 #include "dpq.h"
 
 /*----------- DEBUGGING -------------
  *
  *	make CFLAGS='-DDEBUG_pnch ....'
+(cd ~/R/D/r-devel/debian-64-inst/src/nmath; gcc -I. -I../../src/include -I../../../R/src/include -I/usr/local/include -DHAVE_CONFIG_H -DDEBUG_pnch -fopenmp -g -pedantic -Wall --std=gnu99 -DDEBUG_q -Wcast-align -Wclobbered  -c ../../../R/src/nmath/pnchisq.c -o pnchisq.o )
 
  * -- Feb.6, 2000 (R pre0.99); M.Maechler:  still have
  * bad precision & non-convergence in some cases (x ~= f, both LARGE)
  */
 
+#ifdef HAVE_LONG_DOUBLE
+# define EXP expl
+# define FABS fabsl
+# define LOG logl
+#else
+# define EXP exp
+# define FABS fabs
+# define LOG log
+#endif
+
+static const double _dbl_min_exp = M_LN2 * DBL_MIN_EXP;
+/*= -708.3964 for IEEE double precision */
+
 
 double pnchisq(double x, double df, double ncp, int lower_tail, int log_p)
 {
@@ -39,24 +55,33 @@ double pnchisq(double x, double df, double ncp, int lower_tail, int log_p)
 
     if (df < 0. || ncp < 0.) ML_ERR_return_NAN;
 
-    ans = pnchisq_raw(x, df, ncp, 1e-12, 8*DBL_EPSILON, 1000000, lower_tail);
+    ans = pnchisq_raw(x, df, ncp, 1e-12, 8*DBL_EPSILON, 1000000, lower_tail, log_p);
     if(ncp >= 80) {
 	if(lower_tail) {
-	    /* if(ans >= 1-1e-10) have no idea how close to 1 the true value is,
-	     *   but ML_ERROR(ME_PRECISION, "pnchisq") seems too harsh */
-	    ans = fmin2(ans, 1.0);  /* e.g., pchisq(555, 1.01, ncp = 80) */
-	}
-	else { /* !lower_tail */
-	    if(ans < 1e-10) ML_ERROR(ME_PRECISION, "pnchisq");
-	    ans = fmax2(ans, 0.0);  /* Precaution PR#7099 */
+	    ans = fmin2(ans, R_D__1);  /* e.g., pchisq(555, 1.01, ncp = 80) */
+	} else { /* !lower_tail */
+	    /* since we computed the other tail cancellation is likely */
+	    if(ans < (log_p ? (-10. * M_LN10) : 1e-10)) ML_ERROR(ME_PRECISION, "pnchisq");
+	    if(!log_p) ans = fmax2(ans, 0.0);  /* Precaution PR#7099 */
 	}
     }
-    return log_p ? log(ans) : ans;
+    if (!log_p || ans < -1e-8)
+	return ans;
+    else { // log_p  &&  ans > -1e-8
+	// prob. = exp(ans) is near one: we can do better using the other tail
+#ifdef DEBUG_pnch
+	REprintf("   pnchisq_raw(*, log_p): ans=%g => 2nd call, other tail\n", ans);
+#endif
+	// FIXME: (sum,sum2) will be the same (=> return them as well and reuse here ?)
+	ans = pnchisq_raw(x, df, ncp, 1e-12, 8*DBL_EPSILON, 1000000, !lower_tail, FALSE);
+	return log1p(-ans);
+    }
 }
 
 double attribute_hidden
-pnchisq_raw(double x, double f, double theta,
-	    double errmax, double reltol, int itrmax, Rboolean lower_tail)
+pnchisq_raw(double x, double f, double theta /* = ncp */,
+	    double errmax, double reltol, int itrmax,
+	    Rboolean lower_tail, Rboolean log_p)
 {
     double lam, x2, f2, term, bound, f_x_2n, f_2n;
     double l_lam = -1., l_x = -1.; /* initialized for -Wall */
@@ -64,9 +89,6 @@ pnchisq_raw(double x, double f, double theta,
     Rboolean lamSml, tSml, is_r, is_b, is_it;
     LDOUBLE ans, u, v, t, lt, lu =-1;
 
-    static const double _dbl_min_exp = M_LN2 * DBL_MIN_EXP;
-    /*= -708.3964 for IEEE double precision */
-
     if (x <= 0.) {
 	if(x == 0. && f == 0.)
 	    return lower_tail ? exp(-0.5*theta) : -expm1(-0.5*theta);
@@ -81,23 +103,60 @@ pnchisq_raw(double x, double f, double theta,
 #endif
 
     if(theta < 80) { /* use 110 for Inf, as ppois(110, 80/2, lower.tail=FALSE) is 2e-20 */
-	LDOUBLE sum = 0, sum2 = 0, lambda = 0.5*theta, pr = exp(-lambda);
-	double ans;
+	LDOUBLE sum, sum2, lambda = 0.5 * theta, pr, ans;
 	int i;
-	/* we need to renormalize here: the result could be very close to 1 */
-	for(i = 0; i < 110;  pr *= lambda/++i) {
-	    sum2 += pr;
-	    /* could break once sum2 is essentially 1 */
-	    sum += pr * pchisq(x, f+2*i, lower_tail, FALSE);
-	}
-	ans = sum/sum2;
-	return ans;
-    }
-
-
+	// Have  pgamma(x,s) < x^s / Gamma(s+1) (< and ~= for small x)
+	// ==> pchisq(x, f) = pgamma(x, f/2, 2) = pgamma(x/2, f/2)
+	//                  <  (x/2)^(f/2) / Gamma(f/2+1) < eps
+	// <==>  f/2 * log(x/2) - log(Gamma(f/2+1)) < log(eps) ( ~= -708.3964 )
+	// <==>        log(x/2) < 2/f*(log(Gamma(f/2+1)) + log(eps))
+	// <==> log(x) < log(2) + 2/f*(log(Gamma(f/2+1)) + log(eps))
+	if(lower_tail && f > 0. &&
+	   log(x) < M_LN2 + 2/f*(lgamma(f/2. + 1) + _dbl_min_exp)) {
+	    // all  pchisq(x, f+2*i, lower_tail, FALSE), i=0,...,110 would underflow to 0.
+	    // ==> work in log scale
+	    sum = sum2 = ML_NEGINF;
+	    pr = -lambda;
+	    /* we need to renormalize here: the result could be very close to 1 */
+	    for(i = 0; i < 110;  pr += LOG(lambda) - LOG(++i)) {
+		sum2 = logspace_add(sum2, pr);
+		sum = logspace_add(sum, pr + pchisq(x, f+2*i, lower_tail, TRUE));
+		if (sum2 >= -1e-15) /*<=> EXP(sum2) >= 1-1e-15 */ break;
+	    }
+	    ans = sum - sum2;
 #ifdef DEBUG_pnch
-    REprintf("pnchisq(x=%g, f=%g, theta=%g): ",x,f,theta);
+	    REprintf("pnchisq(x=%g, f=%g, th.=%g); th. < 80, logspace: i=%d, ans=(sum=%g)-(sum2=%g)\n",
+		     x,f,theta, i, (double)sum, (double)sum2);
 #endif
+	    return (double) (log_p ? ans : EXP(ans));
+	}
+	else {
+	    sum = sum2 = 0;
+	    pr = EXP(-lambda); // does this need a feature test?
+	    /* we need to renormalize here: the result could be very close to 1 */
+	    for(i = 0; i < 110;  pr *= lambda/++i) {
+		// pr == exp(-lambda) lambda^i / i!  ==  dpois(i, lambda)
+		sum2 += pr;
+		// pchisq(*, i, *) is  strictly decreasing to 0 for lower_tail=TRUE
+		//                 and strictly increasing to 1 for lower_tail=FALSE
+		sum += pr * pchisq(x, f+2*i, lower_tail, FALSE);
+		if (sum2 >= 1-1e-15) break;
+	    }
+	    ans = sum/sum2;
+#ifdef DEBUG_pnch
+	    REprintf("pnchisq(x=%g, f=%g, theta=%g); theta < 80: i=%d, sum=%g, sum2=%g\n",
+		     x,f,theta, i, (double)sum, (double)sum2);
+#endif
+	    return (double) (log_p ? LOG(ans) : ans);
+	}
+    } // if(theta < 80)
+
+    // else: theta == ncp >= 80 --------------------------------------------
+#ifdef DEBUG_pnch
+    REprintf("pnchisq(x=%g, f=%g, theta=%g >= 80): ",x,f,theta);
+#endif
+    // Series expansion ------- FIXME: log_p=TRUE, lower_tail=FALSE only applied at end
+
     lam = .5 * theta;
     lamSml = (-lam < _dbl_min_exp);
     if(lamSml) {
@@ -122,11 +181,11 @@ pnchisq_raw(double x, double f, double theta,
 #endif
 
     if(f2 * DBL_EPSILON > 0.125 && /* very large f and x ~= f: probably needs */
-       fabs(t = x2 - f2) <         /* another algorithm anyway */
+       FABS(t = x2 - f2) <         /* another algorithm anyway */
        sqrt(DBL_EPSILON) * f2) {
 	/* evade cancellation error */
 	/* t = exp((1 - t)*(2 - t/(f2 + 1))) / sqrt(2*M_PI*(f2 + 1));*/
-        lt = (1 - t)*(2 - t/(f2 + 1)) - 0.5 * log(2*M_PI*(f2 + 1));
+        lt = (1 - t)*(2 - t/(f2 + 1)) - M_LN_SQRT_2PI - 0.5 * log(f2 + 1);
 #ifdef DEBUG_pnch
 	REprintf(" (case I) ==> ");
 #endif
@@ -141,23 +200,26 @@ pnchisq_raw(double x, double f, double theta,
 
     tSml = (lt < _dbl_min_exp);
     if(tSml) {
+#ifdef DEBUG_pnch
+	REprintf(" is very small\n");
+#endif
 	if (x > f + theta +  5* sqrt( 2*(f + 2*theta))) {
 	    /* x > E[X] + 5* sigma(X) */
-	    return lower_tail ? 1. : 0.; /* FIXME: We could be more accurate than 0. */
+	    return R_DT_1; /* FIXME: could be more accurate than 0. */
 	} /* else */
 	l_x = log(x);
-	ans = term = t = 0.;
+	ans = term = 0.; t = 0;
     }
     else {
-	t = exp(lt);
+	t = EXP(lt);
 #ifdef DEBUG_pnch
  	REprintf(", t=exp(lt)= %g\n", t);
 #endif
-	ans = term = v * t;
+	ans = term = (double) (v * t);
     }
 
     for (n = 1, f_2n = f + 2., f_x_2n += 2.;  ; n++, f_2n += 2, f_x_2n += 2) {
-#ifdef DEBUG_pnch
+#ifdef DEBUG_pnch_n
 	REprintf("\n _OL_: n=%d",n);
 #endif
 #ifndef MATHLIB_STANDALONE
@@ -168,8 +230,8 @@ pnchisq_raw(double x, double f, double theta,
 	if (f_x_2n > 0) {
 	    /* find the error bound and check for convergence */
 
-	    bound = t * x / f_x_2n;
-#ifdef DEBUG_pnch
+	    bound = (double) (t * x / f_x_2n);
+#ifdef DEBUG_pnch_n
 	    REprintf("\n L10: n=%d; term= %g; bound= %g",n,term,bound);
 #endif
 	    is_r = is_it = FALSE;
@@ -195,11 +257,11 @@ pnchisq_raw(double x, double f, double theta,
             lu += l_lam - log(n); /* u = u* lam / n */
             if(lu >= _dbl_min_exp) {
 		/* no underflow anymore ==> change regime */
-#ifdef DEBUG_pnch
+#ifdef DEBUG_pnch_n
                 REprintf(" n=%d; nomore underflow in u = exp(lu) ==> change\n",
 			 n);
 #endif
-                v = u = exp(lu); /* the first non-0 'u' */
+                v = u = EXP(lu); /* the first non-0 'u' */
                 lamSml = FALSE;
             }
         } else {
@@ -211,17 +273,16 @@ pnchisq_raw(double x, double f, double theta,
             if(lt >= _dbl_min_exp) {
 		/* no underflow anymore ==> change regime */
 #ifdef DEBUG_pnch
-                REprintf("  n=%d; nomore underflow in t = exp(lt) ==> change\n",
-			 n);
+                REprintf("  n=%d; nomore underflow in t = exp(lt) ==> change\n", n);
 #endif
-                t = exp(lt); /* the first non-0 't' */
+                t = EXP(lt); /* the first non-0 't' */
                 tSml = FALSE;
             }
         } else {
 	    t *= x / f_2n;
 	}
         if(!lamSml && !tSml) {
-	    term = v * t;
+	    term = (double) (v * t);
 	    ans += term;
 	}
 
@@ -234,5 +295,5 @@ pnchisq_raw(double x, double f, double theta,
 #ifdef DEBUG_pnch
     REprintf("\n == L_End: n=%d; term= %g; bound=%g\n",n,term,bound);
 #endif
-    return lower_tail ? ans : 1 - ans;
+    return (double) R_DT_val(ans);
 }
